/* Constants */
#define UDRE 5

/* Prototype Functions */
void init_uart0_tx(unsigned long cpu_freq, unsigned long baud_rate);
void uart0_send_byte_array(char *data, unsigned char length);
void uart0_send_byte(char data);

/**
 * UART transmission function for byte arrays
 *
 * Can be used to transmitt strings
 * Length terminated (versus null terminated)
 */
void uart0_send_byte_array(unsigned char *data, unsigned char length) {
	unsigned char counter = 0;

	while (counter++ < length) {
		while ( !( UCSR0A & (1<<UDRE)) );	// Wait for empty buffer
		UDR0 = *data++;						// Put data into buffer, sends the data 
	}
}

/**
 * UART transmission function for a single byte
 */
void uart0_send_byte(unsigned char data) {
	while ( !( UCSR0A & (1<<UDRE)) );	// Wait for empty buffer
 	UDR0 = data;							// Put data into buffer, sends the data 
}

/**
 * Initialize UART RX
 */
void init_uart0_tx(unsigned long cpu_freq, unsigned long baud_rate){
	float temp_1;
	unsigned int temp_2;
	
	// UBBR0 = CPU_FREQ / (16 * BAUD_RATE) - 1
	cpu_freq = cpu_freq >> 4;
	temp_1 = (float) cpu_freq / (float) baud_rate - 1;
	temp_2 = (unsigned int) (temp_1 + 0.5);
	
	UBRR0L = (temp_2 >> 8);
	UBRR0L = ((temp_2 << 8) >> 8);

	UCSR0A = 0b00000000;// 7 - UART RX complete flag
						// 6 - UART TX complete flag
						// 5 - UART DRE flag
						// 4 - Frame error flag
						// 3 - Data overrun flag
						// 2 - Parity error flag
						// 1 - Double UART speed mode
						// 0 - Multi processor communication mode

	UCSR0B = 0b00001000; // 7 - RX Complete Interrupt Enable
						// 6 - TX Complete Interrupt Enable
						// 5 - UART DRE Interrupt Enable
						// 4 - Enable receiver
						// 3 - Enable transmitter
						// 2 - Character size
						// 1:0 - Ninth data bit

	UCSR0C = 0b00000110;// 7:6 - UART Mode Select
						//		00 - Asynchronous UART
						//		01 - Synchronous UART
						//		10 - Reserved
						//		11 - Master SPI
						// 5:4 - Parity mode
						//		00 - Disabled
						//		01 - Reserved
						//		10 - Enabled, Even
						//		11 - Enabled, Odd
						// 3 - Select stop bit length
						//		0 - 1-bit
						//		1 - 2-bit
						// 2:1 - Character size, with UCSRB.2
						//		000 - 5 bit
						//		001 - 6 bit
						//		010 - 7 bit
						//		011 - 8 bit
						//		100 - Reserved
						//		101 - Reserved
						//		110 - Reserved
						//		111 - 9 bit
						// 0 - Select clock polarity
						//		0 - Rising XCK edge
						//		1 - Falling XCK edge 						
}