/**
 * A Mega644P Local DHCP Sentinel Library
 *
 * Ruibing Wang (rw98@cornell.edu)
 */

/* RF Protocol */
#define SENTINEL_RF_LOCAL_DHCP_DISCOVERY			0x00	// Header | NodeType
#define SENTINEL_RF_LOCAL_DHCP_OFFER				0x01	// Header | OfferedAddrL | OfferedAddrH
#define SENTINEL_RF_LOCAL_DHCP_REQUEST				0x02	// Header | NodeType | ServerAddrL | ServerAddrH | OfferedAddrL | OfferedAddrH
#define SENTINEL_RF_LOCAL_DHCP_ACK					0x03	// Header | ClientAddrL | ClientAddrH
#define SENTINEL_RF_SENSOR_DETECTION				0x10	// Header

#define SENTINEL_RF_PARSE_IDLE						0x00
#define SENTINEL_RF_PARSE_TRANSMIT					0x01
#define SENTINEL_RF_PARSE_DHCP_OFFER				0x10
#define SENTINEL_RF_PARSE_DHCP_ACK					0x11
#define SENTINEL_RF_PARSE_SENSOR_DETECTION			0x20

/* UART Protocol */
#define UART_TX_SENTINEL_STATUS						0x00
#define UART_TX_SENTINEL_LOCAL_DHCP					0x01

#define UART_TX_SENTINEL_SENSOR_DETECTION_RELAY		0x10
#define UART_TX_SENTINEL_SENSOR_DETECTION_REPORT	0x11

#define UART_RX_SENTINEL_DISCOVER					0x20
#define UART_TX_SENTINEL_DISCOVER					0x21
#define UART_RX_SENTINEL_INFO						0x22
#define UART_TX_SENTINEL_INFO						0x23

#define UART_RX_SENTINEL_CLEAR_CLIENT_LIST			0x30
#define UART_RX_SENTINEL_CLEAR_RELAY_LIST			0x31
#define UART_RX_SENTINEL_CLEAR_SENSOR_LIST			0x32
#define UART_RX_SENTINEL_RESET_LOCAL_DHCP			0x33

#define UART_TX_ZIGBEE_STATE						0xA0
#define UART_TX_ZIGBEE_IRQ_STATUS					0xA1
#define UART_TX_ZIGBEE_TRAC_STATUS					0xA2

#define UART_TX_ZIGBEE_PHY_DATA						0xB0
#define UART_TX_ZIGBEE_PHY_LQI						0xB1
#define UART_TX_ZIGBEE_MAC_DATA						0xB2
#define UART_TX_ZIGBEE_MAC_DEST_PAN_ID				0xB3
#define UART_TX_ZIGBEE_MAC_DEST_ADDR				0xB4
#define UART_TX_ZIGBEE_MAC_SRC_PAN_ID				0xB5
#define UART_TX_ZIGBEE_MAC_SRC_ADDR					0xB6

#define UART_TX_ZIGBEE_CCA_MODE						0xC0
#define UART_TX_ZIGBEE_CCA_ED_THRES					0xC1
#define UART_TX_ZIGBEE_CCA_VALUE					0xC2
#define UART_TX_ZIGBEE_ED_LEVEL						0xC3

#define UART_TX_ZIGBEE_PART_NUM						0xD0
#define UART_TX_ZIGBEE_VER_NUM						0xD1
#define UART_TX_ZIGBEE_MAN_ID_0						0xD2
#define UART_TX_ZIGBEE_MAN_ID_1						0xD3

/* UART Sentinel Status */
#define SENTINEL_STATUS_P_ON						0x00
#define SENTINEL_STATUS_IDLE						0x01
#define SENTINEL_STATUS_SLEEP						0x02
#define SENTINEL_STATUS_TRX_OFF						0x10
#define SENTINEL_STATUS_TX							0x20
#define SENTINEL_STATUS_RX_LISTEN					0x30
#define SENTINEL_STATUS_RX_LISTEN_SLEEP				0x31
#define SENTINEL_STATUS_RX_BUSY						0x32
#define SENTINEL_STATUS_RX_SUCCESS					0x33
#define SENTINEL_STATUS_RX_LOW_LQI					0x34
#define SENTINEL_STATUS_RX_DUPLICATE_FRAME			0x35

/* UART Sentinel Local DHCP */				
#define SENTINEL_LOCAL_DHCP_TX_DISCOVERY			0x00
#define SENTINEL_LOCAL_DHCP_RX_DISCOVERY			0x01
#define SENTINEL_LOCAL_DHCP_RX_DISCOVERY_MAX_RELAY	0x02
#define SENTINEL_LOCAL_DHCP_RX_DISCOVERY_MAX_SENSOR	0x03
#define SENTINEL_LOCAL_DHCP_TX_OFFER				0x10
#define SENTINEL_LOCAL_DHCP_RX_OFFER_MODE			0x11
#define SENTINEL_LOCAL_DHCP_RX_OFFER				0x12
#define SENTINEL_LOCAL_DHCP_TX_REQUEST				0x20
#define SENTINEL_LOCAL_DHCP_RX_REQUEST				0x21
#define SENTINEL_LOCAL_DHCP_RX_REQUEST_NO_MATCH		0x22
#define SENTINEL_LOCAL_DHCP_TX_ACK					0x30
#define SENTINEL_LOCAL_DHCP_RX_ACK					0x31
#define SENTINEL_LOCAL_DHCP_CONFIG					0x40
#define SENTINEL_LOCAL_DHCP_COMPLETE				0x41
#define SENTINEL_LOCAL_DHCP_RX_ACK_TIMEOUT			0x50
#define SENTINEL_LOCAL_DHCP_RX_OFFER_TIMEOUT		0x51


/* Node Constants */
#define SENTINEL_LOCAL_DHCP_NO_ACK_TIMEOUT_MAX_COUNT	4
#define SENTINEL_LOCAL_DHCP_NO_ACK_RETRY_MAX_COUNT		4

#define SENTINEL_BASE_EXTENDED_ADDR					0x0000000000000000
#define SENTINEL_BASE_SHORT_ADDR					0x0000
#define SENTINEL_BASE_ADDR_MODE						ZIGBEE_MAC_ADDR_MODE_SHORT
#define SENTINEL_BASE_PAN_ID						0x0000

#define SENTINEL_RELAY_INIT_EXTENDED_ADDR			0x0000000000000000
#define SENTINEL_RELAY_INIT_SHORT_ADDR				0x0000
#define SENTINEL_RELAY_INIT_ADDR_MODE				ZIGBEE_MAC_ADDR_MODE_SHORT
#define SENTINEL_RELAY_INIT_PAN_ID					0x0002
#define SENTINEL_RELAY_RETRY_EXTENDED_ADDR			0x0000000000000000
#define SENTINEL_RELAY_RETRY_SHORT_ADDR				0x0000
#define SENTINEL_RELAY_RETRY_ADDR_MODE				ZIGBEE_MAC_ADDR_MODE_SHORT
#define SENTINEL_RELAY_RETRY_PAN_ID					0x0003
#define SENTINEL_RELAY_PAN_ID						0x0000

#define SENTINEL_SENSOR_INIT_EXTENDED_ADDR			0x0000000000000001
#define SENTINEL_SENSOR_INIT_SHORT_ADDR				0x0001
#define SENTINEL_SENSOR_INIT_ADDR_MODE				ZIGBEE_MAC_ADDR_MODE_SHORT
#define SENTINEL_SENSOR_INIT_PAN_ID					0x0002
#define SENTINEL_SENSOR_RETRY_EXTENDED_ADDR			0x0000000000000001
#define SENTINEL_SENSOR_RETRY_SHORT_ADDR			0x0001
#define SENTINEL_SENSOR_RETRY_ADDR_MODE				ZIGBEE_MAC_ADDR_MODE_SHORT
#define SENTINEL_SENSOR_RETRY_PAN_ID				0x0003
#define SENTINEL_SENSOR_PAN_ID						0x0001

#define SENTINEL_GLOBAL_EXTENDED_ADDR				0xFFFFFFFFFFFFFFFF
#define SENTINEL_GLOBAL_SHORT_ADDR					0xFFFF
#define SENTINEL_GLOBAL_ADDR_MODE					ZIGBEE_MAC_ADDR_MODE_SHORT
#define SENTINEL_GLOBAL_PAN_ID						0xFFFF

/* Includes */
#include <avr/io.h>			// Include AVR-GCC Mega644P specific I/O, register lib
#include <avr/interrupt.h>	// Include AVR-GCC ISR lib
#include <avr/sleep.h>		// Include AVR-GCC sleep lib
#include <avr/eeprom.h>		// Include AVR-GCC eeprom lib
#include <util/delay.h>		// Include AVR-GCC delay lib
#include <string.h>			// Include AVR-GCC string lib
#include <stdio.h>			// Include AVR-GCC stdio lib
#include "mcu.h"			// Include user MCU lib
#include "uart.h"			// Include user UART lib
#include "spi.h"			// Include user SPI lib
#include "zigbee.h"			// Include user Zigbee lib
#include "timer.h"			// Include user timer lib
#include "ext_int.h"		// Include user external interrupt lib
#include "util.h"			// Include user misc utility lib

/* EEPROM Variables */
#define EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT		15
#define EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT	15
#define EEPROM_LOCAL_DHCP_RELAY_LEASE_INIT_INDEX	1
#define EEPROM_LOCAL_DHCP_SENSOR_LEASE_INIT_INDEX	1
#define EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_DEPTH		3

#if (SENTINEL_NODE_TYPE == SENTINEL_BASE_NODE)
	uint8_t EEMEM eeprom_local_dhcp_relay_lease_counter = 0;
	uint8_t EEMEM eeprom_local_dhcp_relay_lease_enable_flag[EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT];
	uint16_t EEMEM eeprom_local_dhcp_relay_lease_short_addr[EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT];
	uint8_t EEMEM eeprom_local_dhcp_sensor_lease_counter = 0;
	uint8_t EEMEM eeprom_local_dhcp_sensor_lease_enable_flag[EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT];
	uint16_t EEMEM eeprom_local_dhcp_sensor_lease_short_addr[EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT];
	
	uint8_t EEMEM eeprom_local_dhcp_complete_flag = TRUE;
	uint8_t EEMEM eeprom_local_dhcp_local_depth = 0;
	uint16_t EEMEM eeprom_local_dhcp_local_short_addr = SENTINEL_BASE_SHORT_ADDR;
#elif (SENTINEL_NODE_TYPE == SENTINEL_RELAY_NODE)
	uint8_t EEMEM eeprom_local_dhcp_relay_lease_counter = 0;
	uint8_t EEMEM eeprom_local_dhcp_relay_lease_enable_flag[EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT];
	uint16_t EEMEM eeprom_local_dhcp_relay_lease_short_addr[EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT];
	uint8_t EEMEM eeprom_local_dhcp_sensor_lease_counter = 0;
	uint8_t EEMEM eeprom_local_dhcp_sensor_lease_enable_flag[EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT];
	uint16_t EEMEM eeprom_local_dhcp_sensor_lease_short_addr[EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT];

	uint8_t EEMEM eeprom_local_dhcp_complete_flag = FALSE;
	uint8_t EEMEM eeprom_local_dhcp_local_depth = 0;
	uint16_t EEMEM eeprom_local_dhcp_local_short_addr = 0;
	uint16_t EEMEM eeprom_local_dhcp_server_short_addr = 0;
	uint16_t EEMEM eeprom_local_dhcp_server_pan_id = 0;
	uint8_t EEMEM eeprom_local_dhcp_server_node_type = 0;
	uint8_t EEMEM eeprom_local_dhcp_server_depth = 0;
#elif (SENTINEL_NODE_TYPE == SENTINEL_SENSOR_NODE)
	uint8_t EEMEM eeprom_local_dhcp_complete_flag = FALSE;
	uint8_t EEMEM eeprom_local_dhcp_local_depth = 0;
	uint16_t EEMEM eeprom_local_dhcp_local_short_addr = 0;
	uint16_t EEMEM eeprom_local_dhcp_server_short_addr = 0;
	uint16_t EEMEM eeprom_local_dhcp_server_pan_id = 0;
	uint8_t EEMEM eeprom_local_dhcp_server_node_type = 0;
	uint8_t EEMEM eeprom_local_dhcp_server_depth = 0;
#endif


/* Global Variables */
enum{MCU_STATUS_P_ON,
	MCU_STATUS_DHCP,
	MCU_STATUS_IDLE,
	MCU_STATUS_SLEEP,
	MCU_STATUS_TX_IDLE,
	MCU_STATUS_TX,
	MCU_STATUS_RX_IDLE,
	MCU_STATUS_RX};
uint8_t mcu_status;

enum{LOCAL_DHCP_STATUS_INIT,
	LOCAL_DHCP_STATUS_TX_DISCOVERY,
	LOCAL_DHCP_STATUS_RX_OFFER,
	LOCAL_DHCP_STATUS_TX_REQUEST,
	LOCAL_DHCP_STATUS_RX_ACK,
	LOCAL_DHCP_STATUS_CONFIG,
	LOCAL_DHCP_STATUS_COMPLETE};
uint8_t local_dhcp_status;

/**
 *
 */
void sentinel_reset_local_dhcp(void) {
	#if SENTINEL_NODE_TYPE != SENTINEL_BASE_NODE
		if (eeprom_read_byte(&eeprom_local_dhcp_complete_flag)) {
			eeprom_write_byte(&eeprom_local_dhcp_complete_flag, FALSE);
		}
		local_dhcp_status = LOCAL_DHCP_STATUS_INIT;
		mcu_status = MCU_STATUS_DHCP;
	#endif
}

/**
 *
 */
void sentinel_retry_local_dhcp(void) {
	#if SENTINEL_NODE_TYPE != SENTINEL_BASE_NODE
		if (eeprom_read_byte(&eeprom_local_dhcp_complete_flag)) {
			eeprom_write_byte(&eeprom_local_dhcp_complete_flag, FALSE);
		}
		local_dhcp_status = LOCAL_DHCP_STATUS_TX_DISCOVERY;
		mcu_status = MCU_STATUS_DHCP;
	#endif
}

/**
 *
 */
void sentinel_uart_rx_parse(void) {
	switch(UDR0) {
		case UART_RX_SENTINEL_DISCOVER:
				uart0_send_byte(UART_TX_SENTINEL_DISCOVER);
				uart0_send_byte(SENTINEL_NODE_TYPE);
			break;
		case UART_RX_SENTINEL_INFO:
				uart0_send_byte(UART_TX_SENTINEL_INFO);
				uart0_send_byte(SENTINEL_NODE_TYPE);
				#if (SENTINEL_NODE_TYPE == SENTINEL_BASE_NODE)
					uart0_send_int(SENTINEL_BASE_SHORT_ADDR);
					uart0_send_int(SENTINEL_BASE_PAN_ID);
					uart0_send_byte(eeprom_read_byte(&eeprom_local_dhcp_relay_lease_counter));
					uart0_send_byte(eeprom_read_byte(&eeprom_local_dhcp_sensor_lease_counter));
				#elif (SENTINEL_NODE_TYPE == SENTINEL_RELAY_NODE)
					uart0_send_byte(eeprom_read_byte(&eeprom_local_dhcp_local_depth));
					uart0_send_int(eeprom_read_word(&eeprom_local_dhcp_local_short_addr));
					uart0_send_int(SENTINEL_RELAY_PAN_ID);
					uart0_send_int(eeprom_read_word(&eeprom_local_dhcp_server_short_addr));
					uart0_send_int(eeprom_read_word(&eeprom_local_dhcp_server_pan_id));
					uart0_send_byte(eeprom_read_byte(&eeprom_local_dhcp_relay_lease_counter));
					uart0_send_byte(eeprom_read_byte(&eeprom_local_dhcp_sensor_lease_counter));
				#elif (SENTINEL_NODE_TYPE == SENTINEL_SENSOR_NODE)
					uart0_send_byte(eeprom_read_byte(&eeprom_local_dhcp_local_depth));
					uart0_send_int(eeprom_read_word(&eeprom_local_dhcp_local_short_addr));
					uart0_send_int(SENTINEL_RELAY_PAN_ID);
					uart0_send_int(eeprom_read_word(&eeprom_local_dhcp_server_short_addr));
					uart0_send_int(eeprom_read_word(&eeprom_local_dhcp_server_pan_id));
				#endif
			break;
		case UART_RX_SENTINEL_CLEAR_CLIENT_LIST:
			#if ((SENTINEL_NODE_TYPE == SENTINEL_RELAY_NODE) || (SENTINEL_NODE_TYPE == SENTINEL_BASE_NODE))
				eeprom_write_byte(&eeprom_local_dhcp_relay_lease_counter, 0);
				eeprom_write_byte(&eeprom_local_dhcp_sensor_lease_counter, 0);
				for (uint8_t i=0; i<EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT; i++) {
					if (eeprom_read_byte(&eeprom_local_dhcp_relay_lease_enable_flag[i])) {
						eeprom_write_byte(&eeprom_local_dhcp_relay_lease_enable_flag[i], FALSE);
					}
					if (eeprom_read_byte(&eeprom_local_dhcp_sensor_lease_enable_flag[i])) {
						eeprom_write_byte(&eeprom_local_dhcp_sensor_lease_enable_flag[i], FALSE);
					} 
				}
			#endif
			break;
		case UART_RX_SENTINEL_CLEAR_RELAY_LIST:
			#if ((SENTINEL_NODE_TYPE == SENTINEL_RELAY_NODE) || (SENTINEL_NODE_TYPE == SENTINEL_BASE_NODE))
				eeprom_write_byte(&eeprom_local_dhcp_relay_lease_counter, 0);
				for (uint8_t i=0; i<EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT; i++) {
					if (eeprom_read_byte(&eeprom_local_dhcp_relay_lease_enable_flag[i])) {
						eeprom_write_byte(&eeprom_local_dhcp_relay_lease_enable_flag[i], FALSE);
					}
				}
			#endif
			break;
		case UART_RX_SENTINEL_CLEAR_SENSOR_LIST:
			#if ((SENTINEL_NODE_TYPE == SENTINEL_RELAY_NODE) || (SENTINEL_NODE_TYPE == SENTINEL_BASE_NODE))
				eeprom_write_byte(&eeprom_local_dhcp_sensor_lease_counter, 0);
				for (uint8_t i=0; i<EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT; i++) {
					if (eeprom_read_byte(&eeprom_local_dhcp_sensor_lease_enable_flag[i])) {
						eeprom_write_byte(&eeprom_local_dhcp_sensor_lease_enable_flag[i], FALSE);
					} 
				}
			#endif
			break;
		case UART_RX_SENTINEL_RESET_LOCAL_DHCP:
			#if ((SENTINEL_NODE_TYPE == SENTINEL_SENSOR_NODE) || (SENTINEL_NODE_TYPE == SENTINEL_RELAY_NODE))
				sentinel_reset_local_dhcp();
			#endif
			break;
	}
}

/**
 *
 */
uint8_t sentinel_rf_rx_parse(uint8_t *rxtx_data, uint8_t *rxtx_data_length) {
	uint16_t local_dhcp_offered_short_addr = 0;
	uint8_t local_dhcp_server_depth = 0;
	uint8_t local_dhcp_server_node_type = 0;
	uint16_t local_dhcp_server_short_addr = 0;
	uint16_t local_dhcp_server_pan_id = 0;
	uint8_t local_dhcp_leased_depth = 0;
	uint16_t local_dhcp_leased_short_addr = 0;
	switch(rxtx_data[0]) {
		#if (SENTINEL_NODE_TYPE != SENTINEL_SENSOR_NODE)
		case SENTINEL_RF_LOCAL_DHCP_DISCOVERY:	// Sensor/Relay to Relay/Base
												// Structure: DHCPDiscovery | ClientNodeType
			#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
				uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
				uart0_send_byte(SENTINEL_LOCAL_DHCP_RX_DISCOVERY);
				uart0_send_byte(rxtx_data[1]);
			#endif
			switch (rxtx_data[1]) {
				case SENTINEL_SENSOR_NODE:				// Sensor
					if (eeprom_read_byte(&eeprom_local_dhcp_sensor_lease_counter) == EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT) {
						#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
							uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
							uart0_send_byte(SENTINEL_LOCAL_DHCP_RX_DISCOVERY_MAX_SENSOR);
						#endif
						return SENTINEL_RF_PARSE_IDLE;
					}
					else {
						for (uint8_t i=0; i<EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT; i++) {
							if (!eeprom_read_byte(&eeprom_local_dhcp_sensor_lease_enable_flag[i])) {
								local_dhcp_offered_short_addr = (eeprom_read_word(&eeprom_local_dhcp_local_short_addr) << 4) | (i + EEPROM_LOCAL_DHCP_SENSOR_LEASE_INIT_INDEX);
								i = EEPROM_LOCAL_DHCP_SENSOR_LEASE_MAX_COUNT;
							}
						}
						
					}
					break;
				case SENTINEL_RELAY_NODE:				// Relay
					if (eeprom_read_byte(&eeprom_local_dhcp_relay_lease_counter) == EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT) {
						#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
							uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
							uart0_send_byte(SENTINEL_LOCAL_DHCP_RX_DISCOVERY_MAX_RELAY);
						#endif
						return SENTINEL_RF_PARSE_IDLE;
					}
					else {
						for (uint8_t i=0; i<EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT; i++) {
							if (!eeprom_read_byte(&eeprom_local_dhcp_relay_lease_enable_flag[i])) {
								local_dhcp_offered_short_addr = (eeprom_read_word(&eeprom_local_dhcp_local_short_addr) << 4) | (i + EEPROM_LOCAL_DHCP_RELAY_LEASE_INIT_INDEX);
								i = EEPROM_LOCAL_DHCP_RELAY_LEASE_MAX_COUNT;
							}
						}
						
					}
					break;
			}
			local_dhcp_server_depth = eeprom_read_byte(&eeprom_local_dhcp_local_depth);
			rxtx_data[0] = SENTINEL_RF_LOCAL_DHCP_OFFER;
			rxtx_data[1] = SENTINEL_NODE_TYPE;
			rxtx_data[2] = local_dhcp_server_depth;
			rxtx_data[3] = (uint8_t) (local_dhcp_offered_short_addr & 0xFF);
			rxtx_data[4] = (uint8_t) ((local_dhcp_offered_short_addr >> 8) & 0xFF);
			*rxtx_data_length = 5;
			zigbee_set_tx_dest_pan_id(zigbee_get_rx_src_pan_id());
			zigbee_set_tx_dest_short_addr(zigbee_get_rx_src_short_addr());
			#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
				uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
				uart0_send_byte(SENTINEL_LOCAL_DHCP_TX_OFFER);
				uart0_send_int(local_dhcp_offered_short_addr);
			#endif
			return SENTINEL_RF_PARSE_TRANSMIT;
			break;
		#endif
		case SENTINEL_RF_LOCAL_DHCP_OFFER:		// Relay/Base to Sensor/Relay
												// Structure: DHCPOffer | ServerNodeType | ServerDepth | OfferedAddrL | OfferedAddrH
			local_dhcp_server_node_type = rxtx_data[1];
			local_dhcp_server_depth = rxtx_data[2];
			local_dhcp_offered_short_addr = (((uint16_t) rxtx_data[4]) << 8) | ((uint16_t) rxtx_data[3]);		// TODO: Redundant
			local_dhcp_server_short_addr = zigbee_get_rx_src_short_addr();
			
			rxtx_data[0] = local_dhcp_server_node_type;
			rxtx_data[1] = local_dhcp_server_depth;
			rxtx_data[2] = (uint8_t) (local_dhcp_offered_short_addr & 0xFF);
			rxtx_data[3] = (uint8_t) ((local_dhcp_offered_short_addr >> 8) & 0xFF);
			rxtx_data[4] = (uint8_t) (local_dhcp_server_short_addr & 0xFF);
			rxtx_data[5] = (uint8_t) ((local_dhcp_server_short_addr >> 8) & 0xFF);
			*rxtx_data_length = 6;
			return SENTINEL_RF_PARSE_DHCP_OFFER;
			break;
		#if SENTINEL_NODE_TYPE != SENTINEL_SENSOR_NODE
		case SENTINEL_RF_LOCAL_DHCP_REQUEST:	// Sensor/Relay to Relay/Base
												// Structure: DHCPRequest | ClientNodeType | ServerAddrL | ServerAddrH | OfferedAddrL | OfferedAddrH
			local_dhcp_server_node_type = rxtx_data[1];
			local_dhcp_server_short_addr = (((uint16_t) rxtx_data[3]) << 8) | ((uint16_t) rxtx_data[2]);
			local_dhcp_offered_short_addr = (((uint16_t) rxtx_data[5]) << 8) | ((uint16_t) rxtx_data[4]);
			#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
				uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
				uart0_send_byte(SENTINEL_LOCAL_DHCP_RX_REQUEST);
				uart0_send_int(local_dhcp_server_short_addr);
			#endif
			if (local_dhcp_server_short_addr == eeprom_read_word(&eeprom_local_dhcp_local_short_addr)) {
				uint8_t local_dhcp_offered_index;
				switch (rxtx_data[1]) {
					case SENTINEL_SENSOR_NODE:				// Sensor
						local_dhcp_offered_index = (uint8_t) (local_dhcp_offered_short_addr & 0x0F) - EEPROM_LOCAL_DHCP_SENSOR_LEASE_INIT_INDEX;
						eeprom_write_byte(&eeprom_local_dhcp_sensor_lease_counter, eeprom_read_byte(&eeprom_local_dhcp_sensor_lease_counter)+1);
						eeprom_write_word(&eeprom_local_dhcp_sensor_lease_short_addr[local_dhcp_offered_index], local_dhcp_offered_short_addr);
						eeprom_write_byte(&eeprom_local_dhcp_sensor_lease_enable_flag[local_dhcp_offered_index], TRUE);
						break;
					case SENTINEL_RELAY_NODE:				// Relay
						local_dhcp_offered_index = (uint8_t) (local_dhcp_offered_short_addr & 0x0F) - EEPROM_LOCAL_DHCP_RELAY_LEASE_INIT_INDEX;
						eeprom_write_byte(&eeprom_local_dhcp_relay_lease_counter, eeprom_read_byte(&eeprom_local_dhcp_relay_lease_counter)+1);
						eeprom_write_word(&eeprom_local_dhcp_relay_lease_short_addr[local_dhcp_offered_index], local_dhcp_offered_short_addr);
						eeprom_write_byte(&eeprom_local_dhcp_relay_lease_enable_flag[local_dhcp_offered_index], TRUE);
						break;
				}
				local_dhcp_leased_short_addr = local_dhcp_offered_short_addr;
				// Send acknowledgement
				local_dhcp_server_depth = eeprom_read_byte(&eeprom_local_dhcp_local_depth);
				local_dhcp_leased_depth = local_dhcp_server_depth + 1;
				rxtx_data[0] = SENTINEL_RF_LOCAL_DHCP_ACK;
				rxtx_data[1] = SENTINEL_NODE_TYPE;
				rxtx_data[2] = local_dhcp_server_depth;
				rxtx_data[3] = local_dhcp_leased_depth;
				rxtx_data[4] = (uint8_t) (local_dhcp_leased_short_addr & 0xFF);
				rxtx_data[5] = (uint8_t) ((local_dhcp_leased_short_addr >> 8) & 0xFF);
				*rxtx_data_length = 6;
				zigbee_set_tx_dest_pan_id(zigbee_get_rx_src_pan_id());
				zigbee_set_tx_dest_short_addr(zigbee_get_rx_src_short_addr());
				#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
					uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
					uart0_send_byte(SENTINEL_LOCAL_DHCP_TX_ACK);
					uart0_send_byte(local_dhcp_leased_depth);
					uart0_send_int(local_dhcp_leased_short_addr);
				#endif
				return SENTINEL_RF_PARSE_TRANSMIT;
			}
			else {
				#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
					uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
					uart0_send_byte(SENTINEL_LOCAL_DHCP_RX_REQUEST_NO_MATCH);
					uart0_send_int(eeprom_read_word(&eeprom_local_dhcp_local_short_addr));
				#endif
				return SENTINEL_RF_PARSE_IDLE;
			}
			break;
		#endif
		case SENTINEL_RF_LOCAL_DHCP_ACK:		// Relay/Base to Sensor/Relay
												// Structure: Header | NodeType | OfferedAddrL | OfferedAddrH
			local_dhcp_server_node_type = rxtx_data[1];
			local_dhcp_server_depth = rxtx_data[2];
			local_dhcp_leased_depth = rxtx_data[3];
			local_dhcp_leased_short_addr = (((uint16_t) rxtx_data[5]) << 8) | ((uint16_t) rxtx_data[4]);
			local_dhcp_server_short_addr = zigbee_get_rx_src_short_addr();
			local_dhcp_server_pan_id = zigbee_get_rx_src_pan_id();
			#if UART0_SENTINEL_LOCAL_DHCP_DEBUG
				uart0_send_byte(UART_TX_SENTINEL_LOCAL_DHCP);
				uart0_send_byte(SENTINEL_LOCAL_DHCP_RX_ACK);
				uart0_send_byte(local_dhcp_server_node_type);
				uart0_send_int(local_dhcp_offered_short_addr);
				uart0_send_int(local_dhcp_server_short_addr);
			#endif
			rxtx_data[0] = local_dhcp_server_node_type;
			rxtx_data[1] = local_dhcp_server_depth;
			rxtx_data[2] = local_dhcp_leased_depth;
			rxtx_data[3] = (uint8_t) (local_dhcp_leased_short_addr & 0xFF);
			rxtx_data[4] = (uint8_t) ((local_dhcp_leased_short_addr >> 8) & 0xFF);
			rxtx_data[5] = (uint8_t) (local_dhcp_server_short_addr & 0xFF);
			rxtx_data[6] = (uint8_t) ((local_dhcp_server_short_addr >> 8) & 0xFF); 
			rxtx_data[7] = (uint8_t) (local_dhcp_server_pan_id & 0xFF);
			rxtx_data[8] = (uint8_t) ((local_dhcp_server_pan_id >> 8) & 0xFF); 
			*rxtx_data_length = 9;
			return SENTINEL_RF_PARSE_DHCP_ACK;
			break;
		case SENTINEL_RF_SENSOR_DETECTION:		// Sensor/Relay to Relay/Base
												// Structure: Header | SensorAddrL | SensorAddrH
			return SENTINEL_RF_PARSE_SENSOR_DETECTION;
	}
	return SENTINEL_RF_PARSE_IDLE;
}
