#define MCU_PRR_ALL		0xFF
#define MCU_PRR_NONE	0x00

void init_mcu(uint8_t prr_value);

/**
 * Initialize MCU
 *
 * Enable Power Reduction registers for unused peripherals
 */
void init_mcu(uint8_t prr_value) {
	SMCR = 0b00000000;	// 7:4 - Reserved bits
						// 3:1 - SM2:0: Sleep Mode
						//		000 - Idle
						//		001 - ADC Noise Reduction
						//		010 - Power-down
						//		011 - Power-save
						//		100 - Reserved
						//		101 - Reserved
						//		110 - Standby
						//		111 - Extended Standby
						// 0 - SE: Sleep Enable
			
	cbi(MCUCR, 6);		// 6 - BODS: BOD Sleep
	cbi(MCUCR, 5);		// 5 - BODSE: BOD Sleep Enable
	
	PRR = prr_value;	// 7 - PRTWI: Power Reduction TWI
						// 6 - PRTIM2: Power Reduction Timer2
						// 5 - PRTIM0: Power Reduction Timer0
						// 4 - PRUSART1: Power Reduction UART1
						// 3 - PRTIM1: Power Reduction Timer1
						// 2 - PRSPI: Power Reduction SPI
						// 1 - PRUSART0: Power Reduction UART0
						// 0 - PRADC: Power Reduction ADC
}