/* ZigBee Constants */
#define ZIGBEE_TX_START_REG_ENABLE		TRUE
#define ZIGBEE_TX_RETRY_LATENCY_US		100
#define ZIGBEE_TX_RETRY_COUNTR_MAX		3

/* ZigBee Frame Constants */
#define ZIGBEE_PHY_DATA_LENGTH		127	// Maximum PHY Service Data Unit (PSDU) length
#define ZIGBEE_MAC_FOOTER_LENGTH	2	// MAC Footer length (contains 16-bit CRC)

/* ZigBee MCU Ports */
#define ZIGBEE_DDR			DDRB
#define ZIGBEE_PORT			PORTB
#define ZIGBEE_SLP_TR		PB0
#define ZIGBEE_RST			PB1

/* ZigBee Registers */
#define ZIGBEE_REG_TRX_STATUS	0x01	// Contains current state of the radio transceiver and the the status of the CCA measurement.
										//	7 - CCA_DONE		- indicates if a CCA request is completed (clears by read)
										//		0 - CCA calculation CCA_DONE in progress (*)
										//		1 - CCA calculation finished
										//	6 - CCA_STATUS		- indicates the result of a CCA request (clears by read)
										//		0 - Channel is busy (*)
										//		1 - Channel is idle
										//	5 - Reserved
										//	4:0 - TRX_STATUS	- current radio transceiver status
										//		(see below)
#define ZIGBEE_REG_TRX_STATE	0x02	// Controls the radio transceiver states via register bits TRX_CMD
										//	7:5 - TRAC_STATUS	- status of the TX_ARET algorithm
										//		(see below)
										//	4:0 - TRX_CMD		- initiates a state transition towards the new state
										//		(see below)
#define ZIGBEE_REG_TRX_CTRL_0	0x03	// Controls the drive current of the digital output pads and the CLKM clock rate
										//	7:6 - PAD_IO		- sets the output driver current of digital output MISO and IRQ
										//		0 - 2mA (*)
										//		1 - 4mA
										//		2 - 6mA
										//		3 - 8mA
										//	5:4 - PAD_IO_CLKM	- sets the output driver current of pin CLKM
										//		0 - 2mA
										//		1 - 4mA (*)
										//		2 - 6mA
										//		3 - 8mA
										//	3 - CLKM_SHA_SEL	- defines the commencement of the CLKM clock rate modifications when changing register bits CLKM_CTRL
										//		0 - CLKM clock rate changes immediately
										//		1 - CLKM clock rate changes after SLEEP cycle (*)
										//	2:0 - CLKM_CTRL
										//		(see below)
#define ZIGBEE_REG_PHY_TX_PWR	0x05	// Sets the transmit power and controls the FCS algorithm for TX operation
										//	7 - TX_AUTO_CRC_ON 	- controls the automatic FCS generation for TX operation
										//		0 - Disabled (*)
										//		1 - Enabled
										//	6:4 - Reserved
										//	3:0 - TX_PWR		- sets the TX output power (dBm) of the AT86RF230
										//		0b0000 - +3.0 (*)
										//		0b0001 - +2.6
										//		0b0010 - +2.1
										//		0b0011 - +1.6
										//		0b0100 - +1.1
										//		0b0101 - +0.5
										//		0b0110 - -0.2
										//		0b0111 - -1.2
										//		0b1000 - -2.2
										//		0b1001 - -3.2
										//		0b1010 - -4.2
										//		0b1011 - -5.2
										//		0b1100 - -7.2
										//		0b1101 - -9.2
										//		0b1110 - -12.2
										//		0b1111 - -17.2
#define ZIGBEE_REG_PHY_RSSI		0x06	// Indicates the current received signal strength (RSSI) and the FCS validity of a received frame
										//	7 - RX_CRC_VALID 	- indicates whether a received frame has a valid FCS
										//		0 - Invalid (*)
										//		1 - Valid
										//	6:5 - Reserved
										//	4:0 - RSSI 			- contain the result of the automated RSSI measurement
										//		0 - RF input power of < -91 dBm
										//		...
										//		28 - " ≥ -10 dBm
#define ZIGBEE_REG_PHY_ED_LEVEL	0x07	// Energy Detection Level
#define ZIGBEE_REG_PHY_CC_CCA	0x08	// Contains control bits for the CCA measurement and the channel center frequency
										//	7 - CCA_REQUEST		- initiates manual CCA measurement request
										//	6:5 - CCA_MODE
										//		0 - Reserved
										//		1 - Mode 1, Energy above threshold (*)
										//		2 - Mode 2, Carrier sense only
										//		3 - Mode 3, Carrier sense with energy above threshold
										//	4:0 - CHANNEL		- define the RX/TX channel
										//		(see below)
#define ZIGBEE_REG_CCA_THRES	0x09	// Contains the threshold level for CCA-ED measurement
										//	7:4 - Reserved
										//	3:0 - CCA_ED_THRES	- define the threshold value of the CCA-ED measurement
										//						A cannel is indicated as busy when RSSI_BASE_VAL + 2 � CCA_ED_THRES [dBm]
										//		0b0111 - (*)
#define ZIGBEE_REG_IRQ_MASK		0x0E	// Used to enable (1) or disable (0) interrupt events
										//	7 - MASK_BAT_LOW
										//	6 - MASK_TRX_UR
										//	5:4 - Reserved
										//	3 - MASK_TRX_END
										//	2 - MASK_RX_START
										//	1 - MASK_PLL_UNLOCK
										//	0 - MASK_PLL_LOCK
#define ZIGBEE_REG_IRQ_STATUS	0x0F	// Contains the status of the individual interrupts (clears by read)
										//	7 - BAT_LOW
										//	6 - TRX_UR
										//	5:4 - Reserved
										//	3 - TRX_END
										//	2 - RX_START
										//	1 - PLL_UNLOCK
										//	0 - PLL_LOCK
#define ZIGBEE_REG_VREG_CTRL	0x10	// controls the use of the voltage regulators and indicates the status of these
										//	7 - AVREG_EXT		- defines supply for the analog low voltage building blocks of the radio transceiver
										//		0 - Use internal voltage regulator (*)
										//		1 - Use external voltage regulator, internal voltage regulator is disabled
										//	6 - AVDD_OK			- indicates if the internal 1.8V regulated supply voltage AVDD has settled.
										//		0 - Analog voltage regulator disabled or supply voltage not stable (*)
										//		1 - Analog supply voltage has settled or if AVREG_EXT = 1
										//	5:4 - Reserved
										//	3 - DVREG_EXT		- defines supply for the digital low voltage building blocks of the radio transceiver
										//		0 - Use internal voltage regulator (*)
										//		1 - Use external voltage regulator, internal voltage regulator is disabled
										//	2 - DVDD_OK			- indicates if the internal 1.8V regulated supply voltage DVDD has settled.
										//		0 - Digital voltage regulator disabled or supply voltage not stable (*)
										//		1 - Digital supply voltage has settled or if DVREG_EXT = 1
										//	1:0 - Reserved
#define ZIGBEE_REG_BATMON		0x11	// Configures the battery monitor
										//	7:6 - Reserved
										//	5 - BATMON_OK		- indicates the external supply voltage compared to the programmed threshold BATMON_VTH
										//		0 - VDD < BATMON_VTH (*)
										//		1 - VDD > BATMON_VTH
										//	4 - BATMON_HR		- selects the range and resolution of the battery monitor
										//		0 - Enables the low range, see BATMON_VTH (*)
										//		1 - Enables the high range, see BATMON_VTH
										//	3:0 - BATMON_VTH	- The threshold value for the battery monitor
										//				BATMON_HR = 1	" = 2
										//		0x0		2.550			1.70
										//		0x1		2.625			1.75
										//		0x2		2.700			1.80 (*)
										//		0x3		2.775			1.85
										//		0x4		2.850			1.90
										//		0x5		2.925			1.95
										//		0x6		3.000			2.00
										//		0x7		3.075			2.05
										//		0x8		3.150			2.10
										//		0x9		3.225			2.15
										//		0xA		3.300			2.20
										//		0xB		3.375			2.25
										//		0xC		3.450			2.30
										//		0xD		3.525			2.35
										//		0xE		3.600			2.40
										//		0xF		3.675			2.45
#define ZIGBEE_REG_XOSC_CTRL	0x12    // Controls the operation of the crystal oscillator
										//	7:4 - XTAL_MODE		- sets the operating mode of the crystal oscillator
										//		0x0 - Crystal oscillator disabled, no clock signal is fed to the internal circuitry
										//		0x4 - Internal crystal oscillator disabled, use external reference frequency
										//		0xF - Internal crystal oscillator enabled (*)
										//	3:0 - XTAL_TRIM		- control two internal capacitance arrays connected to pins XTAL1 and XTAL2
										//		0x0 - 0.0 pF, trimming capacitors disconnected (*)
										//		0x1 - 0.3 pF, trimming capacitor switched on
										//		...
										//		0xF - 4.5 pF, trimming capacitor switched on
#define ZIGBEE_REG_FTN_CTRL		0x18	//
#define ZIGBEE_REG_PLL_CF		0x1A	// Controls the operation of the center frequency calibration loop
										//	7 - PLL_CF_START	- initiates the center frequency calibration, 80us
										//	6:0 - Reserved
#define ZIGBEE_REG_PLL_DCU		0x1B	// Controls the operation of the delay cell calibration loop
										//	7 - PLL_DCU_START	- initiates the delay cell calibration, 6us
										//	6:0 - Reserved
#define ZIGBEE_REG_PART_NUM		0x1C	// Contains the radio transceiver part number.
#define ZIGBEE_REG_VER_NUM		0x1D	// Contains the radio transceiver version number.
#define ZIGBEE_REG_MAN_ID_0		0x1E	// Contains bits [7:0] of the 32 bit JEDEC manufacturer ID
#define ZIGBEE_REG_MAN_ID_1		0x1F	// Contains bits [15:8] of "
#define ZIGBEE_REG_SHORT_ADDR_0	0x20	// Contains bits [7:0] of the 16 bit short address for address filtering
#define ZIGBEE_REG_SHORT_ADDR_1	0x21	// Contains bits [15:8] of "
#define ZIGBEE_REG_PAN_ID_0		0x22	// Contains bits [7:0] of the 16 bit PAN ID for address filtering
#define ZIGBEE_REG_PAN_ID_1		0x23	// Contains bits [15:8] of "
#define ZIGBEE_REG_IEEE_ADDR_0	0x24	// Contains bits [7:0] of the 64 bit IEEE address for address filtering
#define ZIGBEE_REG_IEEE_ADDR_1	0x25	// Contains bits [15:8] of "
#define ZIGBEE_REG_IEEE_ADDR_2	0x26	// Contains bits [23:16] of "
#define ZIGBEE_REG_IEEE_ADDR_3	0x27	// Contains bits [31:24] of "
#define ZIGBEE_REG_IEEE_ADDR_4	0x28	// Contains bits [39:32] of "
#define ZIGBEE_REG_IEEE_ADDR_5	0x29	// Contains bits [47:40] of "
#define ZIGBEE_REG_IEEE_ADDR_6	0x2A	// Contains bits [55:48] of "
#define ZIGBEE_REG_IEEE_ADDR_7	0x2B	// Contains bits [63:56] of "
#define ZIGBEE_REG_XAH_CTRL		0x2C	// Controls the TX_ARET transaction in the Extended Mode
										//	7:4 - MAX_FRAME_RETRIES	- specifies the max # of frame retransmission in TX_ARET transaction
										//		0b0011 (*)
										//	3:1 - MAX_CSMA_RETRIES	- specifies the max # of CSMA retries in TX_ARET transaction
										//		0b100 (*)
										//	0 - Reserved
#define ZIGBEE_REG_CSMA_SEED_0	0x2D	// Contains part of the CSMA_SEED value for the CSMA-CA algorithm.
										//	7:0 - CSMA_SEED_0	- contain the lower 8 bits of the CSMA_SEED
#define ZIGBEE_REG_CSMA_SEED_1	0x2E	// Contains part of the CSMA_SEED value and configuration bits for address filtering in RX_AACK operation
										//	7:6 - MIN_BE		- sets the delay of the CSMA-CA algorithm after rising edge of SLP_TR pin
										//	5 - AACK_SET_PD		- sets the content of the frame pending subfield for acknowledgement frames in RX_AACK mode
										//	4 - Reserved
										//	3 - I_AM_COORD		- sets whether the node is a PAN coordinator in RX_AACK mode
										//	2:0 - CSMA_SEED_1	- contain the upper 3 bits of the CSMA_SEED

/* ZigBee Transceiver States (from ZIGBEE_TRX_STATUS[4:0]'s TRX_STATUS subregister) */
#define ZIGBEE_TRX_STATUS_MASK					0b00011111
#define ZIGBEE_TRX_STATUS_P_ON 					0x00	// Power-on after VDD (XOSC=On, Pull-up=On)
#define ZIGBEE_TRX_STATUS_TRX_OFF				0x08	// Clock state (XOSC=On, Pull-up=Off)
#define ZIGBEE_TRX_STATUS_SLEEP 				0x0F	// Sleep State (XOSC=Off, Pull-up=Off)
#define ZIGBEE_TRX_STATUS_PLL_ON 				0x09	// PLL State
#define ZIGBEE_TRX_STATUS_BUSY_TX				0x02	// Transmit State
#define ZIGBEE_TRX_STATUS_RX_ON 				0x06	// Rx Listen State
#define ZIGBEE_TRX_STATUS_BUSY_RX 				0x01	// Receive State
#define ZIGBEE_TRX_STATUS_RX_ON_NOCLK			0x1C	// Rx Listen State (CLKM=Off)
#define ZIGBEE_TRX_STATUS_BUSY_RX_AACK			0x11
#define ZIGBEE_TRX_STATUS_BUSY_TX_ARET			0x12
#define ZIGBEE_TRX_STATUS_RX_AACK_ON			0x16
#define ZIGBEE_TRX_STATUS_TX_ARET_ON			0x19
#define ZIGBEE_TRX_STATUS_RX_AACK_NOCLK			0x1D
#define ZIGBEE_TRX_STATUS_BUSY_RX_AACK_NOCLK	0x1E
#define ZIGBEE_TRX_STATUS_TRANSITION			0x1F

/* ZigBee Transceiver States (from ZIGBEE_TRX_STATE[7:5]'s TRAC_STATUS subregister) */
#define ZIGBEE_TRAC_STATUS_MASK						0b11100000
#define ZIGBEE_TRAC_STATUS_MASK_SH					5
#define ZIGBEE_TRAC_STATUS_SUCCESS					0
#define ZIGBEE_TRAC_STATUS_SUCCESS_DATA_PENDING		1
#define ZIGBEE_TRAC_STATUS_CHANNEL_ACCESS_FAILURE	3
#define ZIGBEE_TRAC_STATUS_NO_ACK					5
#define ZIGBEE_TRAC_STATUS_INVALID					7

/* Radio state transition commands (from ZIGBEE_TRX_STATE[4:0]'s TRX_CMD subregister) */
#define ZIGBEE_TRX_CMD_MASK				0b00011111
#define ZIGBEE_TRX_CMD_NOP				0x00
#define ZIGBEE_TRX_CMD_TX_START			0x02
#define ZIGBEE_TRX_CMD_FORCE_TRX_OFF	0x03
#define ZIGBEE_TRX_CMD_RX_ON			0x06
#define ZIGBEE_TRX_CMD_TRX_OFF			0x08
#define ZIGBEE_TRX_CMD_PLL_ON			0x09
#define ZIGBEE_TRX_CMD_RX_AACK_ON		0x16
#define ZIGBEE_TRX_CMD_TX_ARET_ON		0x19

/* Clock rate of CLKM (from ZIGBEE_TRX_CTRL_0[2:0]'s CLKM_CTRL subregister) */
#define ZIGBEE_CLKM_SHA_MASK		0b00001000
#define ZIGBEE_CLKM_SHA_MASK_SH		3	// ZIGBEE_TRX_CTRL_0[3]'s CLKM_SHA subregister
#define ZIGBEE_CLKM_SHA_IMM			0	// Immediate
#define ZIGBEE_CLKM_SHA_SLP			1	// After sleep
#define ZIGBEE_CLKM_MASK			0b00000111
#define ZIGBEE_CLKM_OFF				0
#define ZIGBEE_CLKM_1MHZ			1	// Default
#define ZIGBEE_CLKM_2MHZ			2
#define ZIGBEE_CLKM_4MHZ			3
#define ZIGBEE_CLKM_8MHZ			4
#define ZIGBEE_CLKM_16MHZ			5

/* Output driver current of pin CLKM */
#define ZIGBEE_PAD_IO_CLKM_MASK		0b00110000
#define ZIGBEE_PAD_IO_CLKM_MASK_SH	4
#define ZIGBEE_PAD_IO_CLKM_2MA		0
#define ZIGBEE_PAD_IO_CLKM_4MA		1
#define ZIGBEE_PAD_IO_CLKM_6MA		2
#define ZIGBEE_PAD_IO_CLKM_8MA		3

/* RXTX Channel (from ZIGBEE_PHY_CC_CCA[4:0]'s CHANNEL subregister) */
#define ZIGBEE_CHANNEL_MASK	0b00011111
#define ZIGBEE_CHANNEL_11	0x0B	// 2405 MHz (Default)
#define ZIGBEE_CHANNEL_12	0x0C	// 2410 MHz
#define ZIGBEE_CHANNEL_13	0x0D	// 2415 MHz
#define ZIGBEE_CHANNEL_14	0x0E	// 2420 MHz
#define ZIGBEE_CHANNEL_15	0x0F	// 2425 MHz
#define ZIGBEE_CHANNEL_16	0x10	// 2430 MHz
#define ZIGBEE_CHANNEL_17	0x11	// 2435 MHz
#define ZIGBEE_CHANNEL_18	0x12	// 2440 MHz
#define ZIGBEE_CHANNEL_19	0x13	// 2445 MHz
#define ZIGBEE_CHANNEL_20	0x14	// 2450 Mhz
#define ZIGBEE_CHANNEL_21	0x15	// 2455 Mhz
#define ZIGBEE_CHANNEL_22	0x16	// 2460 Mhz
#define ZIGBEE_CHANNEL_23	0x17	// 2465 Mhz
#define ZIGBEE_CHANNEL_24	0x18	// 2470 Mhz
#define ZIGBEE_CHANNEL_25	0x19	// 2475 Mhz
#define ZIGBEE_CHANNEL_26	0x1A	// 2480 Mhz

/* Zigbee IRQ Masks from the ZIGBEE_IRQ_MASK & ZIGBEE_IRQ_STATUS */
#define ZIGBEE_IRQ_MASK_ALL			0b11001111
#define ZIGBEE_IRQ_MASK_NONE		0b00000000
#define ZIGBEE_IRQ_MASK_BAT_LOW		7			// 0b10000000
#define	ZIGBEE_IRQ_MASK_TRX_UR		6			// 0b01000000
#define	ZIGBEE_IRQ_MASK_TRX_END		3			// 0b00001000
#define ZIGBEE_IRQ_MASK_RX_START	2			// 0b00000100
#define ZIGBEE_IRQ_MASK_PLL_UNLOCK	1			// 0b00000010
#define ZIGBEE_IRQ_MASK_PLL_LOCK	0			// 0b00000001

/* Zigbee Extended Operation */
#define ZIGBEE_MAC_FRAME_TYPE_BEACON	0
#define ZIGBEE_MAC_FRAME_TYPE_DATA		1
#define ZIGBEE_MAC_FRAME_TYPE_ACK		2
#define ZIGBEE_MAC_FRAME_TYPE_MAC		3
#define ZIGBEE_MAC_ADDR_MODE_NONE		0
#define ZIGBEE_MAC_ADDR_MODE_SHORT		2
#define ZIGBEE_MAC_ADDR_MODE_EXTENDED	3
#define ZIGBEE_MAC_FRAME_TYPE			0b001

/* Zigbee Operation */
#define ZIGBEE_RXTX_SUCCESS					0	// RXTX successful
#define ZIGBEE_RXTX_SUCCESS_DATA_PENDING	1	// RXTX successful with data pending
#define ZIGBEE_RXTX_INVALID_ARGUMENT		2
#define ZIGBEE_RXTX_BUFFER_UNDERRUN			3	// RXTX unsuccessful due to bufferrun
#define ZIGBEE_RXTX_CHANNEL_ACCESS_FAILURE	4	// RXTX unsuccessful due to channel access failure
#define ZIGBEE_RXTX_LOW_LQI					5	// 
#define ZIGBEE_RXTX_NO_ACK					6	// RXTX unsuccessful due to no acknowledgement frame
#define ZIGBEE_RXTX_INVALID					7	//

/*
#define ZIGBEE_MSG_WRONG_STATE			6	// The end-user tried to do an invalid state transition.
#define ZIGBEE_MSG_BUSY_STATE			7	// The requested state transition could not be completed.
#define ZIGBEE_MSG_STATE_TRANSITION_FAILED	8 // The requested state transition could not be completed.
#define ZIGBEE_MSG_CCA_IDLE				9	// Channel in idle. Ready to transmit a new frame.
#define ZIGBEE_MSG_CCA_BUSY				10	// Channel busy.
#define ZIGBEE_MSG_TRX_BUSY				11	// Transceiver is busy receiving or transmitting data.
#define ZIGBEE_MSG_BAT_LOW				12	// Measured battery voltage is lower than voltage threshold.
*/

/* Zigbee Hardware ID data */
uint8_t zigbee_hw_part_num, zigbee_hw_ver_num, zigbee_hw_man_id_0, zigbee_hw_man_id_1;

/* Global Variables */
volatile uint8_t zigbee_irq_flag;
uint8_t zigbee_irq_pll_lock_flag;	// Indicates locking and unlocking events of the PLL
uint8_t zigbee_irq_trx_end_flag;	// Signals the end of any transmit or receive operation
uint8_t zigbee_irq_rx_start_flag;	// Signals that the radio transceiver has received a valid SFD
uint8_t zigbee_irq_trx_ur_flag;		// Signals a buffer underrrun
uint8_t zigbee_irq_bat_low_flag;	// Signals when the supply voltage drops below the adjusted threshold

uint8_t zigbee_extended_mode_flag;
uint16_t zigbee_addr_filter_mac_pan_id, zigbee_addr_filter_mac_short_addr;
uint64_t zigbee_addr_filter_mac_extended_addr;
uint8_t zigbee_addr_filter_mac_addr_mode;

uint8_t zigbee_tx_mac_sequence_number, zigbee_tx_mac_intra_pan_flag, zigbee_tx_mac_ack_request_flag;
uint16_t zigbee_tx_mac_dest_pan_id, zigbee_tx_mac_dest_short_addr;
uint64_t zigbee_tx_mac_dest_extended_addr;
uint8_t zigbee_tx_mac_dest_addr_mode;
uint16_t zigbee_tx_mac_src_pan_id, zigbee_tx_mac_src_short_addr;
uint64_t zigbee_tx_mac_src_extended_addr;
uint8_t zigbee_tx_mac_src_addr_mode;

uint8_t zigbee_rx_mac_sequence_number, zigbee_rx_mac_intra_pan_flag;
uint16_t zigbee_rx_mac_dest_pan_id, zigbee_rx_mac_dest_short_addr;
uint64_t zigbee_rx_mac_dest_extended_addr;
uint8_t zigbee_rx_mac_dest_addr_mode;
uint16_t zigbee_rx_mac_src_pan_id, zigbee_rx_mac_src_short_addr;
uint64_t zigbee_rx_mac_src_extended_addr;
uint8_t zigbee_rx_mac_src_addr_mode;

/* Prototype Functions */
uint8_t zigbee_get_trx_status(void);
void zigbee_set_trx_status(uint8_t target_trx_status);
void zigbee_set_trx_cmd(uint8_t state_trx_cmd);
void zigbee_enable_sleep(void);
void zigbee_enable_wake(void);
uint8_t zigbee_get_trac_status(void);
void zigbee_set_irq_flag(void);
void zigbee_clear_irq_flag(void);
uint8_t zigbee_get_irq_flag(void);

void zigbee_tx_start_pin(void);
void zigbee_tx_start_reg(void);
uint8_t zigbee_get_rx_ready(void);
void zigbee_clear_rx_ready(void);
void zigbee_set_extended_mode(uint8_t enable_flag);
uint8_t zigbee_get_extended_mode(void);

void zigbee_enable_trx_off(void);
void zigbee_enable_basic_tx(void);
uint8_t zigbee_basic_tx(uint8_t *tx_data, uint8_t tx_data_length, uint8_t tx_on_the_fly);
void zigbee_enable_basic_rx(void);
void zigbee_enable_basic_rx_sleep(void);
uint8_t zigbee_basic_rx(uint8_t *rx_data, uint8_t *rx_data_length);

void zigbee_extended_frame_header(uint8_t *tx_header, uint8_t *tx_header_length);
void zigbee_enable_extended_tx(void);
uint8_t zigbee_extended_tx(uint8_t *tx_data, uint8_t tx_data_length, uint8_t tx_on_the_fly);
void zigbee_set_tx_ack_request(uint8_t enable_flag);
uint8_t zigbee_get_tx_ack_request(void);
void zigbee_inc_tx_sequence_number(void);
void zigbee_set_tx_sequence_number(uint8_t sequence_value);
uint8_t zigbee_get_tx_sequence_number(void);
void zigbee_set_tx_intra_pan(uint8_t enable_flag);
uint8_t zigbee_get_tx_intra_pan(void);
void zigbee_set_tx_dest_pan_id(uint16_t pan_id_value);
void zigbee_set_tx_dest_short_addr(uint16_t short_addr_value);
void zigbee_set_tx_dest_extended_addr(uint64_t extended_addr_value);
void zigbee_set_tx_src_pan_id(uint16_t pan_id_value);
void zigbee_set_tx_src_short_addr(uint16_t short_addr_value);
void zigbee_set_tx_src_extended_addr(uint64_t extended_addr_value);

void zigbee_enable_extended_rx(void);
uint8_t zigbee_extended_rx(uint8_t *rx_data, uint8_t *rx_data_length);
uint8_t zigbee_get_rx_sequence_number(void);
uint8_t zigbee_get_rx_intra_pan(void);
uint16_t zigbee_get_rx_dest_pan_id(void);
uint8_t zigbee_get_rx_dest_addr_mode(void);
uint16_t zigbee_rx_dest_short_addr(void);
uint64_t zigbee_rx_dest_extended_addr(void);
uint16_t zigbee_get_rx_src_pan_id(void);
uint8_t zigbee_get_rx_src_addr_mode(void);
uint16_t zigbee_rx_src_short_addr(void);
uint64_t zigbee_rx_src_extended_addr(void);
void zigbee_set_max_frame_retries(uint8_t max_frame_retries_value);
void zigbee_set_max_csma_retries(uint8_t max_csma_retries_value);
void zigbee_set_csma_seed(uint16_t csma_seed_value);
void zigbee_set_min_be(uint8_t min_be_value);
void zigbee_set_aack_set_pd(uint8_t aack_set_pd_value);
void zigbee_set_i_am_coord(uint8_t i_am_coord_value);
void zigbee_set_crc(uint8_t enable_flag);
void zigbee_set_addr_filter_pan_id(uint16_t pan_id_value);
void zigbee_set_addr_filter_short_addr(uint16_t short_addr_value);
void zigbee_set_addr_filter_extended_addr(uint64_t extended_addr_value);

void zigbee_set_cca_mode(uint8_t mode);
uint8_t zigbee_get_cca_mode(void);
void zigbee_set_cca_ed_thres(uint8_t ed_thres_value);
uint8_t zigbee_get_cca_ed_thres(void);
uint8_t zigbee_man_cca(void);
uint8_t zigbee_get_rssi(void);
uint8_t zigbee_get_ed(void);
uint8_t zigbee_get_frame_ed(void);

void zigbee_set_clkm_fast(uint8_t clkm);
void zigbee_set_clkm_safe(uint8_t clkm);
void zigbee_set_channel(uint8_t chan);
uint8_t zigbee_get_channel(void);
void zigbee_calibrate_ftn(void);
void zigbee_calibrate_pll(void);
void zigbee_set_tx_pwr(uint8_t pwr_value);
uint8_t zigbee_get_tx_pwr(void);
void zigbee_set_batmon(uint8_t vth_value, uint8_t high_flag);
uint8_t zigbee_get_batmon_vth(void);
uint8_t zigbee_get_batmon_range(void);
uint8_t zigbee_get_batmon_status(void);

void zigbee_set_irq_mask_bit(uint8_t irq_bit, uint8_t en);
void zigbee_set_irq_mask(uint8_t reg_value);
void zigbee_update_irq_status(void);
void zigbee_clear_irq_status(void);

void zigbee_init(void);
void zigbee_hw_rst(void);
void zigbee_sw_rst(void);

/**
 * Read the radio transceiver status from TRX_STATUS
 */
uint8_t zigbee_get_trx_status(void) {
	uint8_t trx_status = spi_read_register(ZIGBEE_REG_TRX_STATUS) & ZIGBEE_TRX_STATUS_MASK;
	#if UART0_ZIGBEE_STATE_DEBUG
		uart0_send_byte(UART_TX_ZIGBEE_STATE);
		uart0_send_byte(trx_status);
	#endif
	return trx_status;
}

/**
 * Transition the radio transceiver to a new status
 */
void zigbee_set_trx_status(uint8_t tgt_trx_status) {
	uint8_t src_trx_status = zigbee_get_trx_status();
	while (src_trx_status == ZIGBEE_TRX_STATUS_TRANSITION) {	// Do not try to initiate further state changes while in transition
		src_trx_status = zigbee_get_trx_status();
	}
	
	switch(tgt_trx_status) {
		case ZIGBEE_TRX_STATUS_TRX_OFF:
			switch(src_trx_status) {
				case ZIGBEE_TRX_STATUS_TRX_OFF:
				case ZIGBEE_TRX_STATUS_SLEEP:		// TODO: Add SLP_TR pin pull low command here
					break;
				case ZIGBEE_TRX_STATUS_P_ON:
				case ZIGBEE_TRX_STATUS_PLL_ON:		// Any PLL active state
				case ZIGBEE_TRX_STATUS_RX_ON:
				case ZIGBEE_TRX_STATUS_RX_AACK_ON:
				case ZIGBEE_TRX_STATUS_TX_ARET_ON:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
				case ZIGBEE_TRX_STATUS_RX_ON_NOCLK:
				case ZIGBEE_TRX_STATUS_BUSY_RX_AACK:
				case ZIGBEE_TRX_STATUS_BUSY_TX_ARET:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_FORCE_TRX_OFF);
					_delay_us(1);
					if (zigbee_irq_flag) {	// Check for interrupt
						zigbee_irq_flag = FALSE;
						zigbee_update_irq_status();		// TRX_END interrupt
					}
					break;
			}
			while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_TRX_OFF);
			break;
		case ZIGBEE_TRX_STATUS_SLEEP:		// TODO: Add toggle SLP_TR pin command here
			break;
		case ZIGBEE_TRX_STATUS_PLL_ON:
			switch(src_trx_status) {
				case ZIGBEE_TRX_STATUS_P_ON:
				case ZIGBEE_TRX_STATUS_SLEEP:
					break;
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_PLL_ON);
					while (!zigbee_irq_flag);	// Typically delays for 180us
					zigbee_irq_flag = FALSE;
					zigbee_update_irq_status();		// PLL_LOCK interrupt
					break;
				case ZIGBEE_TRX_STATUS_PLL_ON:
					break;
				case ZIGBEE_TRX_STATUS_RX_ON:		// Any PLL active state
				case ZIGBEE_TRX_STATUS_RX_AACK_ON:
				case ZIGBEE_TRX_STATUS_TX_ARET_ON:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_PLL_ON);
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
				case ZIGBEE_TRX_STATUS_BUSY_RX_AACK:
				case ZIGBEE_TRX_STATUS_BUSY_TX_ARET:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_PLL_ON);
					_delay_us(1);
					if (zigbee_irq_flag) {
						zigbee_irq_flag = FALSE;
						zigbee_update_irq_status();		// TRX_END interrupt
					}
					break;
			}
			while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_PLL_ON);	// PLL_ON state
			break;
		case ZIGBEE_TRX_STATUS_BUSY_TX:		// TODO: Add toggle SLP_TR pin command here
			break;
		case ZIGBEE_TRX_STATUS_RX_ON:
			switch(src_trx_status) {
				case ZIGBEE_TRX_STATUS_P_ON:
				case ZIGBEE_TRX_STATUS_SLEEP:
					break;
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_RX_ON);
					while (!zigbee_irq_flag);	// Typically delays for 180us
					zigbee_irq_flag = FALSE;
					zigbee_update_irq_status();		// PLL_LOCK interrupt
					break;
				case ZIGBEE_TRX_STATUS_RX_ON:
					break;
				case ZIGBEE_TRX_STATUS_PLL_ON:		// Any PLL active state
				case ZIGBEE_TRX_STATUS_RX_AACK_ON:
				case ZIGBEE_TRX_STATUS_TX_ARET_ON:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_RX_ON);
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
				case ZIGBEE_TRX_STATUS_BUSY_RX_AACK:
				case ZIGBEE_TRX_STATUS_BUSY_TX_ARET:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_RX_ON);
					_delay_us(1);
					if (zigbee_irq_flag) {
						zigbee_irq_flag = FALSE;
						zigbee_update_irq_status();		// TRX_END interrupt
					}
					break;
			}
			while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_RX_ON);	// RX_ON state
			break;
		case ZIGBEE_TRX_STATUS_BUSY_RX:			// Transitions after SFD detection
			break;
		case ZIGBEE_TRX_STATUS_RX_ON_NOCLK:
			break;
		case ZIGBEE_TRX_STATUS_BUSY_RX_AACK:	// Transitions after SFD detection
			break;
		case ZIGBEE_TRX_STATUS_BUSY_TX_ARET:	// TODO: Add toggle SLP_TR pin command here
			break;
		case ZIGBEE_TRX_STATUS_RX_AACK_ON:
			switch(src_trx_status) {
				case ZIGBEE_TRX_STATUS_P_ON:
				case ZIGBEE_TRX_STATUS_SLEEP:
					break;
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					zigbee_set_trx_status(ZIGBEE_TRX_STATUS_RX_ON);	// Transition to RX_ON
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_RX_AACK_ON);
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_RX_AACK_ON:
					break;
				case ZIGBEE_TRX_STATUS_TX_ARET_ON:
					zigbee_set_trx_status(ZIGBEE_TRX_STATUS_PLL_ON);// Transition to PLL_ON
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_RX_AACK_ON);
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
				case ZIGBEE_TRX_STATUS_BUSY_RX_AACK:
				case ZIGBEE_TRX_STATUS_BUSY_TX_ARET:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_RX_AACK_ON);
					_delay_us(1);
					if (zigbee_irq_flag) {
						zigbee_irq_flag = FALSE;
						zigbee_update_irq_status();					// TRX_END interrupt
					}
					break;
				
			}
			while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_RX_AACK_ON);	// RX_AACK_ON state
			break;
		case ZIGBEE_TRX_STATUS_TX_ARET_ON:
			switch(src_trx_status) {
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					zigbee_set_trx_status(ZIGBEE_TRX_STATUS_PLL_ON);// Transition to PLL_ON
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_TX_ARET_ON);
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_RX_AACK_ON:
					zigbee_set_trx_status(ZIGBEE_TRX_STATUS_RX_ON);	// Transition to RX_ON
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_TX_ARET_ON);
					_delay_us(1);
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
				case ZIGBEE_TRX_STATUS_BUSY_RX_AACK:
				case ZIGBEE_TRX_STATUS_BUSY_TX_ARET:
					zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_TX_ARET_ON);
					_delay_us(1);
					if (zigbee_irq_flag) {
						zigbee_irq_flag = FALSE;
						zigbee_update_irq_status();					// TRX_END interrupt
					}
					break;
			}
			while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_TX_ARET_ON);	// TX_ARET_ON state
			break;
	}
}

/**
 * Transition the radio transceiver to sleep mode
 *
 * Force the device into TRX_OFF
 * Enter sleep through SLP_TR
 */
void zigbee_enable_sleep(void) {
	zigbee_enable_trx_off();
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(1);
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
}

/**
 * Transition the radio transceiver from sleep mode
 *
 * Check if the radio transceiver is actually sleeping.
 * Pull SLP_TR Low
 * Delay to transition to TRX_OFF
 * Read to ensure TRX_OFF state
 */
void zigbee_wake(void) {
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(1);
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
}

/**
 * The radio transceiver state is controlled by two signal pins (SLP_TR, RST ) 
 * and the register 0x02 (TRX_STATE). A successful state change shall be 
 * confirmed by reading the radio transceiver status from register 0x01 (TRX_STATUS).
 */
void zigbee_set_trx_cmd(uint8_t trx_cmd) {
	uint8_t trx_status = zigbee_get_trx_status();
	while (trx_status == ZIGBEE_TRX_STATUS_TRANSITION) {	// Do not try to initiate further state changes while in transition
		trx_status = zigbee_get_trx_status();
	}
	
	uint8_t trx_state_value = spi_read_register(ZIGBEE_REG_TRX_STATE);
	trx_state_value = (trx_state_value & ~ZIGBEE_TRX_CMD_MASK) | trx_cmd; // TRX_STATE = TRAC_STATUS | TRX_CMD
	spi_write_register(ZIGBEE_REG_TRX_STATE, trx_state_value);
}

/**
 * Read the radio transceiver status from TRX_STATUS
 */
uint8_t zigbee_get_trac_status(void) {
	uint8_t trac_status = (spi_read_register(ZIGBEE_REG_TRX_STATE) & ZIGBEE_TRAC_STATUS_MASK) >> ZIGBEE_TRAC_STATUS_MASK_SH;
	#if UART0_ZIGBEE_TRAC_STATUS_DEBUG
		uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_TRAC_STATUS);
		uart0_send_byte(trac_status);
	#endif
	return trac_status;
}

void zigbee_set_irq_flag(void) {
	zigbee_irq_flag = TRUE;
}

void zigbee_clear_irq_flag(void) {
	zigbee_irq_flag = FALSE;
}

uint8_t zigbee_get_irq_flag(void) {
	return zigbee_irq_flag;
}














/**
 *
 */
void zigbee_tx_start_pin(void) {
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(1);						// Actual delay should be 65 ns
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
}

/**
 *
 */
void zigbee_tx_start_reg(void) {
	zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_TX_START);
}

/**
 *
 */
void zigbee_clear_rx_ready(void) {
	if (zigbee_extended_mode_flag) {
		zigbee_irq_trx_end_flag = FALSE;
	}
	else {
		zigbee_irq_trx_end_flag = FALSE;
		zigbee_irq_rx_start_flag = FALSE;
	}
}

/**
 *
 */
uint8_t zigbee_get_rx_ready(void) {
	if (zigbee_extended_mode_flag) {
		return zigbee_irq_trx_end_flag;
	}
	else {
		return zigbee_irq_trx_end_flag && zigbee_irq_rx_start_flag;
	}
}

/**
 *
 */
void zigbee_set_extended_mode(uint8_t enable_flag) {
	zigbee_extended_mode_flag = enable_flag;
}

/**
 *
 */
uint8_t zigbee_get_extended_mode(void) {
	return zigbee_extended_mode_flag;
}
















/**
 *
 */
void zigbee_enable_trx_off(void) {
	zigbee_set_trx_status(ZIGBEE_TRX_STATUS_TRX_OFF);
}

/**
 * Changes the TRX to the PLL_ON state for transmission
 */
void zigbee_enable_basic_tx(void) {
	zigbee_set_trx_status(ZIGBEE_TRX_STATUS_PLL_ON);
}

/**
 * Performs a manual basic transmission
 * 
 * The transceiver must be in the PLL_ON state beforehand
 * The SPI data rate must be higher than 250 kBit/s to ensure that no Frame Buffer under run occurs
 */
uint8_t zigbee_basic_tx(uint8_t *tx_data, uint8_t tx_data_length, uint8_t tx_on_the_fly) {
	if (tx_data_length > ZIGBEE_PHY_DATA_LENGTH) {
		return ZIGBEE_RXTX_INVALID_ARGUMENT;
	}
	
	#if PROTOBOARD_LED_DEBUG
		cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	if (tx_on_the_fly) {
		#if ZIGBEE_TX_START_REG_ENABLE
			zigbee_tx_start_reg();
		#else
			zigbee_tx_start_pin();
		#endif
		spi_write_frame_buffer(tx_data_length, tx_data);	
	}
	else {		// Uploads the entire frame before transmission to prevent a buffer underrun interrupt
		spi_write_frame_buffer(tx_data_length, tx_data);
		#if ZIGBEE_TX_START_REG_ENABLE
			zigbee_tx_start_reg();
		#else
			zigbee_tx_start_pin();
		#endif
	}

	while (!zigbee_irq_flag);
	zigbee_irq_flag = FALSE;
	zigbee_update_irq_status(); 				// TRX_END interrupt

	/* PLL_ON state */

	#if PROTOBOARD_LED_DEBUG
		sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif

	if (zigbee_irq_trx_ur_flag) {	// TRX_UR interrupt occurs when the frame was incorrectly transmitted
		zigbee_irq_trx_ur_flag = FALSE;
		return ZIGBEE_RXTX_BUFFER_UNDERRUN;
	}
	
	return ZIGBEE_RXTX_SUCCESS;
}

/**
 *
 */
void zigbee_enable_basic_rx(void) {
	zigbee_clear_rx_ready();
	zigbee_set_trx_status(ZIGBEE_TRX_STATUS_RX_ON);
}

/**
 *
 */
void zigbee_enable_basic_rx_sleep(void) {
	zigbee_enable_basic_rx();
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(1);
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
}

/**
 *
 */
uint8_t zigbee_basic_rx(uint8_t *rx_data, uint8_t *rx_data_length) {
	#if PROTOBOARD_LED_DEBUG
		cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	//ZigBee networks require to determine the “best” route between two nodes. Both, the
	//	LQI and the RSSI/ED can be used for this, depending on the optimization criteria. As a
	//	rule of thumb RSSI/ED is useful to differentiate between links with high LQI values.
	//	Transmission links with low LQI values should be discarded for routing decisions even if
	//	the RSSI/ED values are high.
	uint8_t rx_data_lqi;
	spi_read_frame_buffer(rx_data_length, rx_data, &rx_data_lqi);
	
	#if UART0_ZIGBEE_RX_PHY_LQI_DEBUG
		uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_PHY_LQI);
		uart0_send_byte(rx_data_lqi);
	#endif
	
	#if PROTOBOARD_LED_DEBUG
		sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	if (rx_data_lqi == 0) {
		*rx_data_length = 0;
		return ZIGBEE_RXTX_LOW_LQI;
	}
	return ZIGBEE_RXTX_SUCCESS;
}


















/**
 * IEEE 802.15.4-2003 Frame Header Generation
 *
 * Intra PAN subfield is enabled, so PAN ID is omitted from source address field
 * Source and destination address mode are fixed at 16-bit
 * Source address is simply the zigbee MAC address
 * Frame header length fixed at 9
 */
void zigbee_extended_frame_header(uint8_t *tx_header, uint8_t *tx_header_length) {
	/*
	tx_frame[0] = 0b01100001;	//FCF.
	tx_frame[1] = 0b10001000;	//FCF.
	tx_frame[2] = zigbee_extended_frame_sequence_number++; //Sequence number set during frame transmission.
	tx_frame[3] = PAN_ID & 0xFF; //Dest. PANID.
	tx_frame[4] = (PAN_ID >> 8 ) & 0xFF; //Dest. PANID.
	tx_frame[5] = DEST_ADDRESS & 0xFF; //Dest. Addr.
	tx_frame[6] = (DEST_ADDRESS >> 8 ) & 0xFF; //Dest. Addr.
	tx_frame[7] = SHORT_ADDRESS & 0xFF; //Source Addr.
	tx_frame[8] = (SHORT_ADDRESS >> 8 ) & 0xFF; //Source Addr.
	*/

	/**
	 * MAC Header
	 *	2:0 - Frame Types
	 *		000 - Beacon
	 *		001 - Data
	 *		010 - Acknowledge
	 *		011 - MAC command
	 *		1xx - Reserved
	 *	3 - Secuirty enabled subfield (ignore for AT86RF230)
	 *	4 - Frame pending subfield indicating if the device sending the frame has additional data to send to the recipient
	 *		If more data are pending, the recipient shall retrieve them by sending another data request command to the device
	 * 		The frame pending subfield shall be used only in frames transmitted either during the CAP by devices
	 *		operating on a beacon-enabled PAN or at any time by devices operating on a nonbeacon-enabled PAN.
	 *		This field can be set in an acknowledgement frame. It indicates that the transmitter of the acknowledgement 
	 *		frame has more data to send for the recipient of the acknowledgement frame.
	 *	5 - Acknowledgement request subfield indicating whether an acknowledgment is required from the recipient device 
	 *		on receipt of a data or MAC command frame.
	 *	6 - Intra-PAN subfield indicating whether the MAC frame is to be sent within the same PAN (intra-PAN) or to another PAN (inter-PAN).
	 *	11:10 - Destination addressing mode subfield
	 *		00 - No dest PAN id or address - If not an acknowledgement or beacon frame, the source addressing subfield must 
	 *				be non-zero, implying the frame is directed to the PAN coordinator with the PAN identifier as specified 
	 *				in the source PAN identifier field.
	 *		01 - Reserved
	 *		10 - 16-bit short address
	 *		11 - 64-bit extended address
	 *	15:14 - Source addressing mode subfield
	 *		00 - No src PAN id or address - If not an acknowledgement frame, the destination addressing subfield must be 
	 *				non-zero, implying that the frame has originated from the PAN coordinator with the PAN identifier as 
	 *				specified in the destination PAN identifier field.
	 *		01 - Reserved
	 *		10 - 16-bit short address
	 *		11 - 64-bit extended address
	 *	23:16 - Sequence number field
	 *		For a beacon frame, it shall specify a BSN. Each coordinator shall store its current BSN value in the MAC PIB 
	 *			attribute macBSN and initialize it to a random value. The coordinator shall copy the value of the macBSN
	 *			attribute into the sequence number field of a beacon frame, each time one is generated, and shall then
	 *			increment macBSN by one.
	 *		For a data, acknowledgment, or MAC command frame, it shall specify a data sequence number (DSN) that is used 
	 *			to match an acknowledgment frame to the data or MAC command frame. Each device shall support exactly one 
	 *			DSN regardless of the number of unique devices with which it wishes to communicate. Each device shall 
	 *			store its current DSN value in the MAC PIB attribute macDSN and initialize it to a random value. The 
	 *			algorithm for choosing a random number is out of the scope of this standard. The device shall copy the 
	 *			value of the macDSN attribute into the sequence number field of a data or MAC command frame, each time one 
	 *			is generated, and shall then increment macDSN by one.
	 *		If an acknowledgment is requested, the recipient device shall copy the DSN received in the data or MAC
	 *			command frame into the DSN field of the corresponding acknowledgment frame. If the acknowledgment was not 
	 *			received after macAckWaitDuration symbols, the MAC sublayer of the originating device shall retransmit 
	 *			the frame using the same DSN as was used in the original transmission.
	 *	Destination PAN id field - 16-bit value indicating the unique PAN identified of the intended recipient of the frame.
	 *		A value of 0xFFFF in this field represents the broadcast PAN identifier, which shall be accepted as a valid PAN 
	 *			identifier by all devices currently listening to the channel.
	 *		This field shall be included in the MAC frame only if the destination addressing mode subfield of the frame
	 *			control field is nonzero.
	 *	Destination address field - 16/64-bit value indicating the address of the intended recipient of the frame.
	 *		A value of 0xFFFF in this field shall represent the broadcast short address, which shall be accepted as a valid 
	 *			short address by all devices currently listening to the channel.
	 *		This field shall be included in the MAC frame only if the destination addressing mode subfield of the frame
	 *			control field is nonzero.
	 *	Source PAN id field - 16-bit value indicating the unique PAN identifier of the originator of the frame.
	 *		This field shall be included in the MAC frame only if the source addressing mode is non-zero and the intra-PAN
	 *			subfields of the frame control field iz zero.
	 *		The PAN identifier of a device is initially determined during association on a PAN, but may changefollowing a 
	 *			PAN identifier conflict resolution.
	 *	Source address field - 16/64-bit value indicating the address of the originator of the frame.
	 *		This field shall be included in the MAC frame only if the source addressing mode subfield of the frame control 
	 *			field is nonzero.
	 */

	/* Frame type[2:0] | Security Enabled[3] | Frame Pending[4] | ACK Request[5] | Intra-PAN[6] */
	tx_header[0] = ZIGBEE_MAC_FRAME_TYPE | (zigbee_tx_mac_ack_request_flag << 5) | (zigbee_tx_mac_intra_pan_flag << 6);
	/* Destination Address Mode[11:10] | Source Address Mode[15:14] */
	tx_header[1] = (zigbee_tx_mac_dest_addr_mode << 2) | (zigbee_tx_mac_src_addr_mode << 6);
	/* Frame Sequence number */
	tx_header[2] = zigbee_tx_mac_sequence_number;
	
	*tx_header_length = 3;
	switch (zigbee_tx_mac_dest_addr_mode) {
		case ZIGBEE_MAC_ADDR_MODE_NONE:
			break;
		case ZIGBEE_MAC_ADDR_MODE_SHORT:
			tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_dest_pan_id & 0xFF);
			tx_header[(*tx_header_length)+1] = (uint8_t) (zigbee_tx_mac_dest_pan_id >> 8);
			*tx_header_length += 2;
			tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_dest_short_addr & 0xFF);
			tx_header[(*tx_header_length)+1] = (uint8_t) (zigbee_tx_mac_dest_short_addr >> 8);
			*tx_header_length += 2;
			break;
		case ZIGBEE_MAC_ADDR_MODE_EXTENDED:
			tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_dest_pan_id & 0xFF);
			tx_header[(*tx_header_length)+1] = (uint8_t) (zigbee_tx_mac_dest_pan_id >> 8);
			*tx_header_length += 2;
			tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_dest_extended_addr & 0xFF);
			tx_header[(*tx_header_length)+1] = (uint8_t) ((zigbee_tx_mac_dest_extended_addr >> 8) & 0xFF);
			tx_header[(*tx_header_length)+2] = (uint8_t) ((zigbee_tx_mac_dest_extended_addr >> 16) & 0xFF);
			tx_header[(*tx_header_length)+3] = (uint8_t) ((zigbee_tx_mac_dest_extended_addr >> 24) & 0xFF);
			tx_header[(*tx_header_length)+4] = (uint8_t) ((zigbee_tx_mac_dest_extended_addr >> 32) & 0xFF);
			tx_header[(*tx_header_length)+5] = (uint8_t) ((zigbee_tx_mac_dest_extended_addr >> 40) & 0xFF);
			tx_header[(*tx_header_length)+6] = (uint8_t) ((zigbee_tx_mac_dest_extended_addr >> 48) & 0xFF);
			tx_header[(*tx_header_length)+7] = (uint8_t) ((zigbee_tx_mac_dest_extended_addr >> 56) & 0xFF);
			*tx_header_length += 8;
			break;
	}
	switch (zigbee_tx_mac_src_addr_mode) {
		case ZIGBEE_MAC_ADDR_MODE_NONE:
			break;
		case ZIGBEE_MAC_ADDR_MODE_SHORT:
			if (!zigbee_tx_mac_intra_pan_flag) {
				tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_src_pan_id & 0xFF);
				tx_header[(*tx_header_length)+1] = (uint8_t) (zigbee_tx_mac_src_pan_id >> 8);
				*tx_header_length += 2;
			}
			tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_src_short_addr & 0xFF);
			tx_header[(*tx_header_length)+1] = (uint8_t) (zigbee_tx_mac_src_short_addr >> 8);
			*tx_header_length += 2;
			break;
		case ZIGBEE_MAC_ADDR_MODE_EXTENDED:
			if (!zigbee_tx_mac_intra_pan_flag) {
				tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_src_pan_id & 0xFF);
				tx_header[(*tx_header_length)+1] = (uint8_t) (zigbee_tx_mac_src_pan_id >> 8);
				*tx_header_length += 2;
			}
			tx_header[(*tx_header_length)] = (uint8_t) (zigbee_tx_mac_src_extended_addr & 0xFF);
			tx_header[(*tx_header_length)+1] = (uint8_t) ((zigbee_tx_mac_src_extended_addr >> 8) & 0xFF);
			tx_header[(*tx_header_length)+2] = (uint8_t) ((zigbee_tx_mac_src_extended_addr >> 16) & 0xFF);
			tx_header[(*tx_header_length)+3] = (uint8_t) ((zigbee_tx_mac_src_extended_addr >> 24) & 0xFF);
			tx_header[(*tx_header_length)+4] = (uint8_t) ((zigbee_tx_mac_src_extended_addr >> 32) & 0xFF);
			tx_header[(*tx_header_length)+5] = (uint8_t) ((zigbee_tx_mac_src_extended_addr >> 40) & 0xFF);
			tx_header[(*tx_header_length)+6] = (uint8_t) ((zigbee_tx_mac_src_extended_addr >> 48) & 0xFF);
			tx_header[(*tx_header_length)+7] = (uint8_t) ((zigbee_tx_mac_src_extended_addr >> 56) & 0xFF);
			*tx_header_length += 8;
			break;
	}
}

/**
 *
 */
void zigbee_enable_extended_tx(void) {
	/* Configure CSMA-CA */
	//spi_write_register(ZIGBEE_REG_CSMA_SEED_0, 0b11101010);
	//spi_write_register(ZIGBEE_REG_CSMA_SEED_1, 0b11100010);
	zigbee_set_max_frame_retries(0);		// Avoid Errata of Rev. A
	zigbee_set_max_csma_retries(5);			// Use maximum retries
	//zigbee_set_csma_seed();				// Use default
	zigbee_set_min_be(3);					// Use maximum delay for CSMA
	
	/* Configure CCA */
	zigbee_set_crc(TRUE); 

	zigbee_set_trx_status(ZIGBEE_TRX_STATUS_TX_ARET_ON);
}

/**
 *
 * The SPI data rate must be higher than 250 kBit/s to ensure that no Frame Buffer under run occurs
 */
uint8_t zigbee_extended_tx(uint8_t *tx_data, uint8_t tx_data_length, uint8_t tx_on_the_fly) {
	sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_0);
	uint8_t tx_frame[ZIGBEE_PHY_DATA_LENGTH];
	uint8_t tx_frame_length;
	zigbee_extended_frame_header(tx_frame, &tx_frame_length);
	cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_0);

	for (uint8_t i=0; i<tx_data_length; i++) {
		tx_frame[tx_frame_length++] = tx_data[i];
	}
	tx_frame_length = tx_frame_length + ZIGBEE_MAC_FOOTER_LENGTH;
	
	if (tx_frame_length > ZIGBEE_PHY_DATA_LENGTH) {
		return ZIGBEE_RXTX_INVALID_ARGUMENT;
	}
	
	#if PROTOBOARD_LED_DEBUG
		cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	if (tx_on_the_fly) {
		#if ZIGBEE_TX_START_REG_ENABLE
			zigbee_tx_start_reg();			// Start transmission by sending TX_START cmd
		#else
			zigbee_tx_start_pin();		// Start transmission by toggling SLP_TR pin
		#endif
		spi_write_frame_buffer(tx_frame_length, tx_frame);	
	}
	else {			// Upload the entire frame before transmission to avoid a buffer underrun
		spi_write_frame_buffer(tx_frame_length, tx_frame);	
		#if ZIGBEE_TX_START_REG_ENABLE
			zigbee_tx_start_reg();			// Start transmission by sending TX_START cmd
		#else
			zigbee_tx_start_pin();		// Start transmission by toggling SLP_TR pin
		#endif
	}

	while (!zigbee_irq_flag);		// Wait for interrupt flag to be raised
	zigbee_irq_flag = FALSE;
	zigbee_update_irq_status(); 	// TRX_END interrupt occurs
	
	#if PROTOBOARD_LED_DEBUG
		sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	if (zigbee_irq_trx_ur_flag) {	// TRX_UR interrupt occurs when the frame was incorrectly transmitted
		zigbee_irq_trx_ur_flag = FALSE;
		return ZIGBEE_RXTX_BUFFER_UNDERRUN;
	}
	switch (zigbee_get_trac_status()) {
		case ZIGBEE_TRAC_STATUS_SUCCESS:
			return ZIGBEE_RXTX_SUCCESS;
		case ZIGBEE_TRAC_STATUS_SUCCESS_DATA_PENDING:
			return ZIGBEE_RXTX_SUCCESS_DATA_PENDING;
		case ZIGBEE_TRAC_STATUS_CHANNEL_ACCESS_FAILURE:
			return ZIGBEE_RXTX_CHANNEL_ACCESS_FAILURE;
		case ZIGBEE_TRAC_STATUS_NO_ACK:
			return ZIGBEE_RXTX_NO_ACK;
		case ZIGBEE_TRAC_STATUS_INVALID:
			return ZIGBEE_RXTX_INVALID;
	}
	return ZIGBEE_RXTX_INVALID;
}

/**
 *
 */
void zigbee_set_tx_ack_request(uint8_t enable_flag) {
	zigbee_tx_mac_ack_request_flag = enable_flag;
}

/**
 *
 */
uint8_t zigbee_get_tx_ack_request(void) {
	return zigbee_tx_mac_ack_request_flag;
}

/**
 *
 */
void zigbee_inc_tx_sequence_number(void) {
	zigbee_tx_mac_sequence_number++;
}

/**
 *
 */
void zigbee_set_tx_sequence_number(uint8_t sequence_value) {
	zigbee_tx_mac_sequence_number = sequence_value;
}

/**
 *
 */
uint8_t zigbee_get_tx_sequence_number(void) {
	return zigbee_tx_mac_sequence_number;
}

/**
 *
 */
void zigbee_set_tx_intra_pan(uint8_t enable_flag) {
	zigbee_tx_mac_intra_pan_flag = enable_flag;
}

/**
 *
 */
uint8_t zigbee_get_tx_intra_pan(void) {
	return zigbee_tx_mac_intra_pan_flag;
}

/**
 *
 */
void zigbee_set_tx_dest_pan_id(uint16_t pan_id_value) {
	zigbee_tx_mac_dest_pan_id = pan_id_value;
}

/**
 *
 */
void zigbee_set_tx_dest_short_addr(uint16_t short_addr_value) {
	zigbee_tx_mac_dest_short_addr = short_addr_value;
	zigbee_tx_mac_dest_addr_mode = ZIGBEE_MAC_ADDR_MODE_SHORT;
}

/**
 *
 */
void zigbee_set_tx_dest_extended_addr(uint64_t extended_addr_value) {
	zigbee_tx_mac_dest_extended_addr = extended_addr_value;
	zigbee_tx_mac_dest_addr_mode = ZIGBEE_MAC_ADDR_MODE_EXTENDED;
}

/**
 *
 */
void zigbee_set_tx_src_pan_id(uint16_t pan_id_value) {
	zigbee_tx_mac_src_pan_id = pan_id_value;
}

/**
 *
 */
void zigbee_set_tx_src_short_addr(uint16_t short_addr_value) {
	zigbee_tx_mac_src_short_addr = short_addr_value;
	zigbee_tx_mac_src_addr_mode = ZIGBEE_MAC_ADDR_MODE_SHORT;
}

/**
 *
 */
void zigbee_set_tx_src_extended_addr(uint64_t extended_addr_value) {
	zigbee_tx_mac_src_extended_addr = extended_addr_value;
	zigbee_tx_mac_src_addr_mode = ZIGBEE_MAC_ADDR_MODE_EXTENDED;
}
















/**
 *
 */
void zigbee_enable_extended_rx(void) {
	zigbee_set_aack_set_pd(TRUE);
	zigbee_set_i_am_coord(FALSE);
	zigbee_set_crc(TRUE);
	
	zigbee_clear_rx_ready();
	zigbee_set_trx_status(ZIGBEE_TRX_STATUS_RX_AACK_ON);
}

/**
 *
 */
void zigbee_enable_extended_rx_sleep(void) {
	zigbee_enable_extended_rx();
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(1);
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	/* The CLKM pin is disabled 35 clock cycles after the rising edge at SLP_TR pin. */
	//_delay_us(880);
}

/**
 *
 */
uint8_t zigbee_extended_rx(uint8_t *rx_data, uint8_t *rx_data_length) {
	#if PROTOBOARD_LED_DEBUG
		cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif

	uint8_t rx_frame[ZIGBEE_PHY_DATA_LENGTH], rx_frame_length, rx_frame_lqi;
	*rx_data_length = 0;

	//zigbee_get_trac_status();	// Determine whether the transmission was successful
	
	spi_read_frame_buffer(&rx_frame_length, rx_frame, &rx_frame_lqi);
	
	#if UART0_ZIGBEE_RX_PHY_LQI_DEBUG
		uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_PHY_LQI);
		uart0_send_byte(rx_frame_lqi);
	#endif
	#if UART0_ZIGBEE_RX_PHY_DATA_DEBUG
		uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_PHY_DATA);
		uart0_send_byte(rx_frame_length);
		uart0_send_byte_array(rx_frame, rx_frame_length);
	#endif
	
	if ((rx_frame_lqi > 0) && (rx_frame_length > 0)) {
		zigbee_rx_mac_intra_pan_flag = FALSE;
		if (rx_frame[0] & _BV(6)) {
			zigbee_rx_mac_intra_pan_flag = TRUE;
		}
		
		zigbee_rx_mac_sequence_number = rx_frame[2];
		zigbee_rx_mac_dest_addr_mode = (rx_frame[1] >> 2) & 0b11;
		zigbee_rx_mac_src_addr_mode = (rx_frame[1] >> 6) & 0b11;
		
		uint8_t rx_frame_index = 3;
		switch(rx_frame[0] & 0b111) {		// Parse frame type
			case ZIGBEE_MAC_FRAME_TYPE_BEACON:
				/*
				src_pan_id = (((uint16_t) zigbee_rx_frame[4]) << 8) | ((uint16_t) zigbee_rx_frame[3]);
				switch(src_address_mode) {
					case ZIGBEE_MAC_ADDR_MODE_SHORT:
						src_short_address = (((uint16_t) zigbee_rx_frame[6]) << 8) | ((uint16_t) zigbee_rx_frame[5]);
						break;
					case ZIGBEE_MAC_ADDR_MODE_EXTENDED:
						src_extended_address = (((uint64_t) zigbee_rx_frame[12]) << 56) | 
							(((uint64_t) zigbee_rx_frame[11]) << 48) | 
							(((uint64_t) zigbee_rx_frame[10]) << 40) | 
							(((uint64_t) zigbee_rx_frame[9]) << 32) | 
							(((uint64_t) zigbee_rx_frame[8]) << 24) | 
							(((uint64_t) zigbee_rx_frame[7]) << 16) | 
							(((uint64_t) zigbee_rx_frame[6]) << 8) | 
							((uint64_t) zigbee_rx_frame[5]);
							
						break;
				}
				*/
				break;
			case ZIGBEE_MAC_FRAME_TYPE_DATA:
				switch(zigbee_rx_mac_dest_addr_mode) {
					case ZIGBEE_MAC_ADDR_MODE_NONE:
						break;
					case ZIGBEE_MAC_ADDR_MODE_SHORT:
						zigbee_rx_mac_dest_pan_id = (((uint16_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint16_t) rx_frame[rx_frame_index]);
						#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
							uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_DEST_PAN_ID);
							uart0_send_int(zigbee_rx_mac_dest_pan_id);
						#endif
						rx_frame_index += 2;
						zigbee_rx_mac_dest_short_addr = (((uint16_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint16_t) rx_frame[rx_frame_index]);
						#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
							uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_DEST_ADDR);
							uart0_send_byte(2);
							uart0_send_int(zigbee_rx_mac_dest_short_addr);
						#endif
						rx_frame_index += 2;
						break;
					case ZIGBEE_MAC_ADDR_MODE_EXTENDED:
						zigbee_rx_mac_dest_pan_id = (((uint16_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint16_t) rx_frame[rx_frame_index]);
						#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
							uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_DEST_PAN_ID);
							uart0_send_int(zigbee_rx_mac_dest_pan_id);
						#endif
						rx_frame_index += 2;
						zigbee_rx_mac_dest_extended_addr = (((uint64_t) rx_frame[rx_frame_index+7]) << 56) | 
							(((uint64_t) rx_frame[rx_frame_index+6]) << 48) | 
							(((uint64_t) rx_frame[rx_frame_index+5]) << 40) | 
							(((uint64_t) rx_frame[rx_frame_index+4]) << 32) | 
							(((uint64_t) rx_frame[rx_frame_index+3]) << 24) | 
							(((uint64_t) rx_frame[rx_frame_index+2]) << 16) | 
							(((uint64_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint64_t) rx_frame[rx_frame_index]);
						#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
							uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_DEST_ADDR);
							uart0_send_byte(8);
							uart0_send_extended(zigbee_rx_mac_dest_extended_addr);
						#endif
						rx_frame_index += 8;
						break;
				}
				switch(zigbee_rx_mac_src_addr_mode) {
					case ZIGBEE_MAC_ADDR_MODE_NONE:
						break;
					case ZIGBEE_MAC_ADDR_MODE_SHORT:
						if (!zigbee_rx_mac_intra_pan_flag) {
							zigbee_rx_mac_src_pan_id = (((uint16_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint16_t) rx_frame[rx_frame_index]);
							#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
								uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_SRC_PAN_ID);
								uart0_send_int(zigbee_rx_mac_src_pan_id);
							#endif
							rx_frame_index += 2;
						}
						zigbee_rx_mac_src_short_addr = (((uint16_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint16_t) rx_frame[rx_frame_index]);
						#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
							uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_SRC_ADDR);
							uart0_send_byte(2);
							uart0_send_int(zigbee_rx_mac_src_short_addr);
						#endif
						rx_frame_index += 2;
						break;
					case ZIGBEE_MAC_ADDR_MODE_EXTENDED:
						if (!zigbee_rx_mac_intra_pan_flag) {
							zigbee_rx_mac_src_pan_id = (((uint16_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint16_t) rx_frame[rx_frame_index]);
							#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
								uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_SRC_PAN_ID);
								uart0_send_int(zigbee_rx_mac_src_pan_id);
							#endif
							rx_frame_index += 2;
						}
						zigbee_rx_mac_src_extended_addr = (((uint64_t) rx_frame[rx_frame_index+7]) << 56) | 
							(((uint64_t) rx_frame[rx_frame_index+6]) << 48) | 
							(((uint64_t) rx_frame[rx_frame_index+5]) << 40) | 
							(((uint64_t) rx_frame[rx_frame_index+4]) << 32) | 
							(((uint64_t) rx_frame[rx_frame_index+3]) << 24) | 
							(((uint64_t) rx_frame[rx_frame_index+2]) << 16) | 
							(((uint64_t) rx_frame[rx_frame_index+1]) << 8) | 
							((uint64_t) rx_frame[rx_frame_index]);
						#if UART0_ZIGBEE_RX_MAC_FRAME_DEBUG
							uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_MAC_SRC_ADDR);
							uart0_send_byte(8);
							uart0_send_extended(zigbee_rx_mac_src_extended_addr);
						#endif
						rx_frame_index += 8;
						break;
				}
				break;
			case ZIGBEE_MAC_FRAME_TYPE_ACK:
				break;
			case ZIGBEE_MAC_FRAME_TYPE_MAC:
				break;
		}
	
		*rx_data_length = 0;
		for(uint8_t i=rx_frame_index; i<(rx_frame_length-ZIGBEE_MAC_FOOTER_LENGTH); i++) {
			rx_data[(*rx_data_length)++] = rx_frame[i];
		}
	}
	else {
		return ZIGBEE_RXTX_LOW_LQI;
	}
	
	#if PROTOBOARD_LED_DEBUG
		sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	return ZIGBEE_RXTX_SUCCESS;
}

/**
 *
 */
uint8_t zigbee_get_rx_sequence_number(void) {
	return zigbee_rx_mac_sequence_number;
}

/**
 *
 */
uint16_t zigbee_get_rx_dest_pan_id(void) {
	return zigbee_rx_mac_dest_pan_id;
}

/**
 *
 */
uint8_t zigbee_get_rx_dest_addr_mode(void) {
	return zigbee_rx_mac_dest_addr_mode;
}

/**
 *
 */
uint16_t zigbee_rx_dest_short_addr(void) {
	return zigbee_rx_mac_dest_short_addr;
}

/**
 *
 */
uint64_t zigbee_rx_dest_extended_addr(void) {
	return zigbee_rx_mac_dest_extended_addr;
}

/**
 *
 */
uint16_t zigbee_get_rx_src_pan_id(void) {
	return zigbee_rx_mac_src_pan_id;
}

/**
 *
 */
uint8_t zigbee_get_rx_src_addr_mode(void) {
	return zigbee_rx_mac_src_addr_mode;
}

/**
 *
 */
uint16_t zigbee_rx_src_short_addr(void) {
	return zigbee_rx_mac_src_short_addr;
}

/**
 *
 */
uint64_t zigbee_rx_src_extended_addr(void) {
	return zigbee_rx_mac_src_extended_addr;
}

/**
 * Defines the maximum number of frame retransmissions.
 *
 * Note: MAX_FRAME_RETRIES must always equal zero. 
 * A frame will not be retried after the CSMA-CA algorithm has failed. 
 * This must be handled in software.
 */
void zigbee_set_max_frame_retries(uint8_t max_frame_retries_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_XAH_CTRL);
	reg_value = (reg_value & ~0b11110000) | (max_frame_retries_value << 4);
	spi_write_register(ZIGBEE_REG_XAH_CTRL, reg_value);
}

/**
 * Configures how often the radio transceiver retries the CSMA-CA algorithm 
 * after a busy channel is detected
 *
 * MAX_CSMA_RETRIES specifies the maximum number of retries in TX_ARET
 * transaction to repeat the random back-off/CCA procedure before the transaction gets
 * cancelled.
 */
void zigbee_set_max_csma_retries(uint8_t max_csma_retries_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_XAH_CTRL);
	reg_value = (reg_value & ~0b00001110) | (max_csma_retries_value << 1);
	spi_write_register(ZIGBEE_REG_XAH_CTRL, reg_value);
}

/**
 * Defines a random seed for the back-off-time randomnumber generator in the transmitter
 * Lower 8-bits in CSMA_SEED_0, and upper 3-bits in CSMA_SEED_1
 *
 * Note: The CSMA_SEED parameter should not be altered from its reset value, 
 * since this will corrupt the operation of the random number generator.
 */
void zigbee_set_csma_seed(uint16_t csma_seed_value) {
	uint8_t reg_value = (uint8_t) (csma_seed_value & 0xFF);
	spi_write_register(ZIGBEE_REG_CSMA_SEED_0, reg_value);
	
	reg_value = spi_read_register(ZIGBEE_REG_CSMA_SEED_1);
	reg_value = (reg_value & ~0b00000111) | (((uint8_t) (csma_seed_value >> 8)) & 0b00000111) ;
	spi_write_register(ZIGBEE_REG_CSMA_SEED_1, reg_value);
}

/**
 * Sets the minimum back-off exponent exponent used in the CSMA-CA
 * algorithm to generate a pseudo random number for back-off the CCA.
 * If set to zero, the start of the CCA algorithm is not delayed.
 */
void zigbee_set_min_be(uint8_t min_be_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_CSMA_SEED_1);
	reg_value = (reg_value & ~0b11000000) | (min_be_value << 6);
	spi_write_register(ZIGBEE_REG_CSMA_SEED_1, reg_value);
}

/**
 * Sets the content of the frame pending subfield for acknowledgement
 * frames in RX_AACK mode.
 *
 * If this bit is enabled the frame pending subfield of the
 * acknowledgment frame is set in response to a MAC command data request frame,
 * otherwise not. The register bit has to be set before finishing the SHR transmission of
 * the acknowledgment frame. This is 352 μs (192 μs ACK wait time + 160 μs SHR
 * transmission) after the TRX_END interrupt issued by the frame to be acknowledged.
 */
void zigbee_set_aack_set_pd(uint8_t aack_set_pd_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_CSMA_SEED_1);
	reg_value = (reg_value & ~0b00100000) | (aack_set_pd_value << 5);
	spi_write_register(ZIGBEE_REG_CSMA_SEED_1, reg_value);
}

/**
 * This register bit has to be set if the node is a PAN coordinator.
 * This register bit is used for address filtering in RX_AACK.
 *
 * If I_AM_COORD = 1 and if only source addressing fields are included in a data 
 * or MAC command frame, the frame shall be accepted only if the device is the 
 * PAN coordinator and the source PAN identifier matches macPANId,
 */
void zigbee_set_i_am_coord(uint8_t i_am_coord_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_CSMA_SEED_1);
	reg_value = (reg_value & ~0b00001000) | (i_am_coord_value << 3);
	spi_write_register(ZIGBEE_REG_CSMA_SEED_1, reg_value);
}

/**
 * Set the CRC mode
 *
 * When using the automatic CRC mechanism the data length must be increased 
 * by two (16 bit CRC) bytes and two bytes must be added to the frame payload. 
 * Typically two 0x00 bytes will be appended at the end of the frame during 
 * download to the radio transceiver’s frame buffer.
 *
 * The last two bytes need not to be downloaded, because the FCS is 
 * calculated internally by the radio transceiver
 */
void zigbee_set_crc(uint8_t enable_flag) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_TX_PWR);
	reg_value = (reg_value & ~_BV(7)) | (enable_flag << 7);
	spi_write_register(ZIGBEE_REG_PHY_TX_PWR, reg_value);
}

/**
 * Sets the 16 bit short address for address filtering.
 */
void zigbee_set_addr_filter_short_addr(uint16_t short_addr_value) {
	zigbee_addr_filter_mac_short_addr = short_addr_value;
	zigbee_addr_filter_mac_addr_mode = ZIGBEE_MAC_ADDR_MODE_SHORT;
	
	uint8_t reg_value = (uint8_t) (short_addr_value & 0xFF);
	spi_write_register(ZIGBEE_REG_SHORT_ADDR_0, reg_value);
	reg_value = (uint8_t) ((short_addr_value >> 8) & 0xFF);
	spi_write_register(ZIGBEE_REG_SHORT_ADDR_1, reg_value);
}

/**
 * Sets the 16 bit PAN ID for address filtering.
 */
void zigbee_set_addr_filter_pan_id(uint16_t pan_id_value) {
	zigbee_addr_filter_mac_pan_id = pan_id_value;

	uint8_t reg_value = (uint8_t) (pan_id_value & 0xFF);
	spi_write_register(ZIGBEE_REG_PAN_ID_0, reg_value);
	
	reg_value = (uint8_t) ((pan_id_value >> 8) & 0xFF);
	spi_write_register(ZIGBEE_REG_PAN_ID_1, reg_value);
}

/**
 * Sets the 64 bit IEEE address for address filtering.
 */
void zigbee_set_addr_filter_extended_addr(uint64_t extended_addr_value) {
	zigbee_addr_filter_mac_extended_addr = extended_addr_value;
	zigbee_addr_filter_mac_addr_mode = ZIGBEE_MAC_ADDR_MODE_EXTENDED;

	uint8_t reg_value = (uint8_t) (extended_addr_value & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_0, reg_value);
	reg_value = (uint8_t) ((extended_addr_value >> 8) & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_1, reg_value);
	reg_value = (uint8_t) ((extended_addr_value >> 16) & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_2, reg_value);
	reg_value = (uint8_t) ((extended_addr_value >> 24) & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_3, reg_value);
	reg_value = (uint8_t) ((extended_addr_value >> 32) & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_4, reg_value);
	reg_value = (uint8_t) ((extended_addr_value >> 40) & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_5, reg_value);
	reg_value = (uint8_t) ((extended_addr_value >> 48) & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_6, reg_value);
	reg_value = (uint8_t) ((extended_addr_value >> 56) & 0xFF);
	spi_write_register(ZIGBEE_REG_IEEE_ADDR_7, reg_value);
}
















/**
 * Set CCA Mode
 */
void zigbee_set_cca_mode(uint8_t cca_mode) {
	/**
	 * Mode	Description
	 * 1	Energy Above Threshold: A busy channel shall be reported upon detecting any energy above the threshold.
	 * 2	Carrier Sense Only: A busy channel is reported if signals with the same 
	 *		modulation and spreading characteristics of IEEE 802.15.4 are detected. 
	 *		The energy of these signals is not checked.
	 * 3	Carrier Sense with Energy Above Threshold: A busy channel is reported if 
	 *		signals with the same modulation and spreading characteristics of IEEE 802.15.4 
	 *		and energy above a threshold are detected.
	 */
	
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = (reg_value & ~0b01100000) | (cca_mode << 5);	// CCA Mode is defined in [6:5] of ZIGBEE_REG_PHY_CC_CCA
	spi_write_register(ZIGBEE_REG_PHY_CC_CCA, reg_value);
}

/**
 * Read CCA Mode
 */
uint8_t zigbee_get_cca_mode(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = (reg_value & 0b01100000) >> 5;
	return reg_value;
}

/**
 * Set the CCA ED threshold value
 */
void zigbee_set_cca_ed_thres(uint8_t ed_thres_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_CCA_THRES);
	reg_value = (reg_value & ~0b00001111) | ed_thres_value;
	spi_write_register(ZIGBEE_REG_CCA_THRES, reg_value);
}

/**
 * Read the CCA ED threshold value
 */
uint8_t zigbee_get_cca_ed_thres(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_CCA_THRES);
	reg_value = (reg_value & 0b00001111);
	return reg_value;
}

/**
 * Perform a manual CCA (Clear Channel Assessment)
 *
 * Using the Basic Operating Mode, a CCA request can be initiated manually by setting
 * CCA_REQUEST = 1 in register 0x08 (PHY_CC_CCA), if the AT86RF230 is in any RX
 * state. The CCA computation is done over eight symbol periodes and the result is
 * accessible 140 μs after the request.
 * Note, it is not recommended to initiate manually a CCA measurement when using the
 * Extended Operating Mode.
 * 
 * CCA measurements can be done from state PLL_ON. 
 * The measurement is started by switching to state RX_ON and writing the value 1 to the sub register SR_CCA_REQUEST. 
 * The measurement is finished after 140us and the radio transceiver is switched back to state PLL_ON.
 *
 * Return value:	0 = medium is busy
 *					1 - medium is free
 */
uint8_t zigbee_man_cca(void) {
	zigbee_set_trx_status(ZIGBEE_TRX_STATUS_RX_ON);
	
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = reg_value | _BV(7);
	spi_write_register(ZIGBEE_REG_PHY_CC_CCA, reg_value);
	_delay_us(140);
	reg_value = spi_read_register(ZIGBEE_REG_TRX_STATUS);
	while (!(reg_value & _BV(7))) {
		reg_value = spi_read_register(ZIGBEE_REG_TRX_STATUS);
	}
	reg_value = reg_value & _BV(6);
	
	#if UART0_ZIGBEE_CCA_DEBUG
		uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_CCA_VALUE);
		uart0_send_byte(reg_value >> 6);
	#endif
	
	/* RX_ON state*/
	
	return reg_value;
}

/**
 * Read the RSSI value
 *
 * The RSSI is a 5-bit value indicating the receive power in the selected channel, in steps of 3 dB.
 * An RSSI value of 0 indicates an RF input power of < -91 dBm. For an RSSI value in the 
 * range of 1 to 28, the RF input power can be calculated as follows:
 * PRF = RSSI_BASE_VAL + 3•(RSSI - 1)
 *
 * Using the Basic Operating Mode, the RSSI value is valid at any RX state, and is
 * updated every 2 μs.
 * The RSSI value for the currently received frame can be read from the sub register SR_RSSI 
 * right after the TRX_IRQ_RX_START interrupt and before the corresponding TRX_IRQ_TRX_END interrupt.
 * Note, it is not recommended to read the RSSI value when using the Extended Operating Mode. 
 * The automatically generated ED value should be used alternatively.
 */
uint8_t zigbee_get_rssi(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_RSSI);
	return reg_value & 0b00011111;
}

/**
 * Perform a manual ED value request
 *
 * The receiver Energy Detection measurement is used by a network layer as part of a
 * channel selection algorithm. It is an estimation of the received signal power within the
 * bandwidth of an IEEE 802.15.4-2003 channel. No attempt is made to identify or decode
 * signals on the channel. The ED value is calculated by averaging RSSI values over eight
 * symbols (128 μs).
 *
 * For manually initiated ED measurement the radio transceiver needs to be in one of the
 * states RX_ON or BUSY_RX.
 */
uint8_t zigbee_get_ed(void) {
	spi_write_register(ZIGBEE_REG_PHY_ED_LEVEL, 0);
    _delay_us(140);
    uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_ED_LEVEL);
	return reg_value;
}

/**
 * Read the ED measurement value of the last frame
 *
 * An ED measurement is automatically started after detection of an SFD. 
 * The register RG_PHY_ED_LEVEL can be read in a time period which starts 140 us 
 * after the TRX_IRQ_RX_START interrupt (time to complete the measurement) 
 * and ends 96 us after the TRX_IRQ_TRX_END interrupt (minimum time which is needed 
 * to detect the next SFD).
 *
 * An automated ED measurement is started, if a SFD field
 * is detected. A valid SFD detection is signalized by an RX_START interrupt in Basic
 * Operating Mode.
 */
uint8_t zigbee_get_frame_ed(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_ED_LEVEL);
	return reg_value;
}















/**
 * The fast CLKM frequency change sequence can be used, if a clock change can happen on the fly, 
 * e.g. if a GPIO or a timer input is driven.
 */
void zigbee_set_clkm_fast(uint8_t clkm_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_TRX_CTRL_0);
	reg_value = (reg_value & ~ZIGBEE_CLKM_MASK & ~ZIGBEE_CLKM_SHA_MASK) | (ZIGBEE_CLKM_SHA_IMM << ZIGBEE_CLKM_SHA_MASK_SH) | clkm_value;
	spi_write_register(ZIGBEE_REG_TRX_CTRL_0, reg_value);
	_delay_us(1);
}

/**
 * If the CLKM signal is used as clock source for the MCU and therefore no on the fly frequency change is allowed, 
 * the safe CLKM frequency change sequence is used doing a clock change using a wake-sleep-wake transition.
 *
 * The MCU needs to be prepped to bring the MCU to a state, where a clock change is allowed. 
 * After asserting ZIGBEE_SLP_TR = HIGH the CLKM signal delivers 35 cycles before it is turned off.
 */
void zigbee_set_clkm_safe(uint8_t clkm_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_TRX_CTRL_0);
	reg_value = (reg_value & ~ZIGBEE_CLKM_MASK & ~ZIGBEE_CLKM_SHA_MASK) | (ZIGBEE_CLKM_SHA_SLP << ZIGBEE_CLKM_SHA_MASK_SH) | clkm_value;
	spi_write_register(ZIGBEE_REG_TRX_CTRL_0, reg_value);
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	/* 35 clk cycles (35 us at 1 MHz) to Prep MCU */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(880);		// Takes up to 880 us to move back to TRX_OFF
}

/**
 *
 */
uint8_t zigbee_get_clkm(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_TRX_CTRL_0);
	return reg_value & ZIGBEE_CLKM_MASK;
}

void zigbee_set_pad_io_clkm(uint8_t pad_io_clkm_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_TRX_CTRL_0);
	reg_value = (reg_value & ~ZIGBEE_PAD_IO_CLKM_MASK) | (pad_io_clkm_value << ZIGBEE_PAD_IO_CLKM_MASK_SH);
	spi_write_register(ZIGBEE_REG_TRX_CTRL_0, reg_value);
}

uint8_t zigbee_get_pad_io_clkm(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_TRX_CTRL_0);
	return (reg_value & ZIGBEE_PAD_IO_CLKM_MASK) >> ZIGBEE_PAD_IO_CLKM_MASK_SH;
}

/**
 * The channel is set by writing the channel number to the sub register 
 * under ZIGBEE_PHY_CC_CCA.
 * The channel number which must be in the range 11<=channel<=26.
 *
 * Depending on the state of the radio transceiver, the current channel number 
 * and the selected channel number, the sequence may cause a PLL Lock interrupt
 */
void zigbee_set_channel(uint8_t chan_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = (reg_value & ~ZIGBEE_CHANNEL_MASK) | chan_value;
	spi_write_register(ZIGBEE_REG_PHY_CC_CCA, reg_value);
	
	switch(zigbee_get_trx_status()) {
		case ZIGBEE_TRX_STATUS_RX_ON:
		case ZIGBEE_TRX_STATUS_PLL_ON:
		case ZIGBEE_TRX_STATUS_TX_ARET_ON:
		case ZIGBEE_TRX_STATUS_RX_AACK_ON:
			while (!zigbee_irq_flag);		// 150us
			zigbee_irq_flag = FALSE;
			zigbee_update_irq_status();			// PLL_LOCK interrupt
			break;
	}
}

/**
 *
 */
uint8_t zigbee_get_channel(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	return reg_value & ZIGBEE_CHANNEL_MASK;
}

/**
 * The filter calibration algorithm is done automatically, whenever a state change from the states P_ON 
 * or SLEEP to the state TRX_OFF occurs or a transceiver reset is performed. The PLL is calibrated during 
 * a state change from state TRX_OFF to any of the [PLL_ACTIVE] states. If these state changes does not 
 * occur during the normal operation of the radio transceiver frequently every 5 minutes, it is recommended 
 * to start it manually within this period. If the operating conditions of the radio transceiver (temperature, voltage) 
 * are nearly constant, the recalibration period may be increased.
 */
void zigbee_calibrate_ftn(void) {
	uint8_t reg_value;
	switch(zigbee_get_trx_status()) {	// Only possible to do filter calibration from TRX_OFF or PLL_ON
		case ZIGBEE_TRX_STATUS_TRX_OFF:
		case ZIGBEE_TRX_STATUS_PLL_ON:
			reg_value = spi_read_register(ZIGBEE_REG_FTN_CTRL);
			reg_value = (reg_value & ~_BV(7)) | _BV(7);
			spi_write_register(ZIGBEE_REG_FTN_CTRL, reg_value);
			
			_delay_us(25);			//Wait for the calibration to finish
			while(spi_read_register(ZIGBEE_REG_FTN_CTRL) & _BV(7));	// Verify the calibration result
			break;
		default:
			break;
	}
}

/**
 *
 */
void zigbee_calibrate_pll(void) {
	uint8_t reg_value;
	switch(zigbee_get_trx_status()) {	// Only possible to calibrate PLL from PLL_ON state
		case ZIGBEE_TRX_STATUS_PLL_ON:
			reg_value = spi_read_register(ZIGBEE_REG_PLL_DCU);
			reg_value = (reg_value & ~_BV(7)) | _BV(7);
			spi_write_register(ZIGBEE_REG_PLL_DCU, reg_value);
			
			reg_value = spi_read_register(ZIGBEE_REG_PLL_CF);
			reg_value = (reg_value & ~_BV(7)) | _BV(7);
			spi_write_register(ZIGBEE_REG_PLL_CF, reg_value);
			
			while (!zigbee_irq_flag);		// 150us
			zigbee_irq_flag = FALSE;
			zigbee_update_irq_status();			// PLL_LOCK interrupt
			break;

			while(spi_read_register(ZIGBEE_REG_PLL_DCU) & _BV(7));	// Verify the calibration result
			while(spi_read_register(ZIGBEE_REG_PLL_CF) & _BV(7));
			break;
	}
}

/**
 *
 */
void zigbee_set_tx_pwr(uint8_t pwr_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_TX_PWR);
	reg_value = (reg_value & ~0b1111) | pwr_value;
	spi_write_register(ZIGBEE_REG_PHY_TX_PWR, reg_value);
}

/**
 *
 */
uint8_t zigbee_get_tx_pwr(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_TX_PWR);
	return reg_value & 0b1111;
}

/**
 *
 */
void zigbee_set_batmon(uint8_t vth_value, uint8_t high_flag) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_BATMON);
	reg_value = (reg_value & ~0b11111) | (high_flag << 4) | vth_value;
	spi_write_register(ZIGBEE_REG_BATMON, reg_value);
}

/**
 *
 */
uint8_t zigbee_get_batmon_vth(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_BATMON);
	return reg_value & 0b111;
}

/**
 *
 */
uint8_t zigbee_get_batmon_range(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_BATMON);
	return (reg_value & _BV(4)) >> 4;
}

/**
 *
 */
uint8_t zigbee_get_batmon_status(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_BATMON);
	return (reg_value & _BV(5)) >> 5;
}












/**
 *
 */
void zigbee_set_irq_mask_bit(uint8_t irq_bit, uint8_t enable_flag) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_IRQ_MASK);
	reg_value = (reg_value & ~_BV(irq_bit)) | (enable_flag << irq_bit);
	spi_write_register(ZIGBEE_REG_IRQ_MASK, reg_value);
}

/**
 *
 */
void zigbee_set_irq_mask(uint8_t reg_value) {
	spi_write_register(ZIGBEE_REG_IRQ_MASK, reg_value);
}

/**
 *
 */
void zigbee_update_irq_status(void) {
	uint8_t irq_state = spi_read_register(ZIGBEE_REG_IRQ_STATUS);

	#if UART0_ZIGBEE_IRQ_STATUS_DEBUG
		uart0_send_byte(SENTINEL_UART_TX_ZIGBEE_IRQ_STATUS);
		uart0_send_byte(irq_state);
	#endif

	if (irq_state & _BV(ZIGBEE_IRQ_MASK_BAT_LOW)) {
		zigbee_irq_bat_low_flag = TRUE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_TRX_UR)) {
		zigbee_irq_trx_ur_flag = TRUE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_TRX_END)) {
		zigbee_irq_trx_end_flag = TRUE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_RX_START)) {
		zigbee_irq_rx_start_flag = TRUE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_PLL_UNLOCK)) {
		zigbee_irq_pll_lock_flag = FALSE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_PLL_LOCK)) {
		zigbee_irq_pll_lock_flag = TRUE;
	}
}

/**
 *
 */
void zigbee_clear_irq_status(void) {
	zigbee_irq_bat_low_flag = FALSE;
	zigbee_irq_trx_ur_flag = FALSE;
	zigbee_irq_trx_end_flag = FALSE;
	zigbee_irq_rx_start_flag = FALSE;
	zigbee_irq_pll_lock_flag = FALSE;
}















/**
 * Brings the radio transceiver into the TRX_OFF state 
 * after a power-on event.
 */
void zigbee_init(void) {
	/* Set PORT for ZigBee Functionality */
	sbi(ZIGBEE_DDR, ZIGBEE_SLP_TR);
	sbi(ZIGBEE_DDR, ZIGBEE_RST);

	/* Transceiver Initialization */
	_delay_us(510);		// Delay 510us for transition to P_ON state
	//_delay_us(510/CPU_FREQ_RATIO);
	
	/* Transceiver Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	cbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST low
	_delay_us(6);		// Delay 6us for hardware reset
	//_delay_us(6/CPU_FREQ_RATIO);
	sbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST high
    
	/* Read Hardware Identification Data */
	zigbee_hw_part_num = spi_read_register(ZIGBEE_REG_PART_NUM);
	zigbee_hw_ver_num = spi_read_register(ZIGBEE_REG_VER_NUM);
	zigbee_hw_man_id_0 = spi_read_register(ZIGBEE_REG_MAN_ID_0);
	zigbee_hw_man_id_1 = spi_read_register(ZIGBEE_REG_MAN_ID_1);
	
	/* Disable IRQ Mask and dummy read IRQ Status to avoid unhandled interrupts */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_NONE);
	spi_read_register(ZIGBEE_REG_IRQ_STATUS);
	
	/* Send TRX_OFF Command */
	zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
	_delay_us(510);		// Delay 510us for transition from P_ON to TRX_OFF
	//_delay_us(510/CPU_FREQ_RATIO);

	/* Get current state */
	while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_TRX_OFF);
	
	/* Enable IRQ Mask */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_ALL);
}

/**
 * Resets the Zigbee Transceiver from any state except P_ON
 *
 * The programming sequence will typically be used to recover 
 * from fatal errors and bring the radio transceiver to a known state.
 * All registers will have their default values and the state machine 
 * will be in TRX_OFF state.
 */
void zigbee_hw_rst(void) {
	/* Transceiver Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	cbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST low
	_delay_us(6);		// Delay 6us for hardware reset
	sbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST high
	
	/* Disable IRQ Mask and dummy read IRQ Status to avoid unhandled interrupts */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_NONE);
	spi_read_register(ZIGBEE_REG_IRQ_STATUS);
	
	_delay_us(120);
	
	while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_TRX_OFF);
	
	/* Enable IRQ Mask */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_ALL);
}

/**
 * Resets the state machine of the radio transceiver to the TRX_OFF state.
 */
void zigbee_sw_rst(void) {
	/* State Machine Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	_delay_us(6);		// Delay 6us for hardware reset
	
	/* Set to TRX_OFF state */
	zigbee_set_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
	_delay_us(6);		// Delay 6us for hardware reset
	
	while(zigbee_get_trx_status() != ZIGBEE_TRX_STATUS_TRX_OFF);
}
