/* External Interrupt Constants */
#define EXT_INT_SC_LOW_LEVEL	0
#define EXT_INT_SC_ANY_EDGE		1
#define EXT_INT_SC_FALLING_EDGE	2
#define EXT_INT_SC_RISING_EDGE	3

#define EXT_INT0_ENABLE		TRUE
#define EXT_INT0_ISR_ENABLE	FALSE
#define EXT_INT0_SC			EXT_INT_SC_RISING_EDGE
#define EXT_INT0_DDR		DDRD
#define EXT_INT0_PORT		PORTD
#define EXT_INT0_PIN		PIND
#define EXT_INT0_INPUT		PD2

#define EXT_INT1_ENABLE		TRUE
#define EXT_INT1_ISR_ENABLE	TRUE
#define EXT_INT1_SC			EXT_INT_SC_FALLING_EDGE
#define EXT_INT1_DDR		DDRD
#define EXT_INT1_PORT		PORTD
#define EXT_INT1_PIN		PIND
#define EXT_INT1_INPUT		PD3

#define EXT_INT2_ENABLE		FALSE
#define EXT_INT2_ISR_ENABLE	FALSE
#define EXT_INT2_SC			EXT_INT_SC_LOW_LEVEL
#define EXT_INT2_DDR		DDRB
#define EXT_INT2_PORT		PORTB
#define EXT_INT2_PIN		PINB
#define EXT_INT2_INPUT		PB2

/* Prototype Functions */
void init_ext_int(void);

/**
 * Interrupt vectors:
 *
 * INT0_vect
 * INT1_vect
 * INT2_vect
 * PCINT1_vect
 * PCINT2_vect
 * PCINT3_vect
 */

/**
 * Initialize External Int0
 *
 * For thread-based interrupt flag detection
 */
void init_ext_int(void) {
	EICRA = 0b00000000;	// 7:6 - Reserved Bits
						// 5:4 - ISC21, ISC20 - External Int 2 Sense Control Bits
						// 3:2 - ISC11, ISC10 - External Int 1 "
						// 1:0 - ISC01, ISC00 - External Int 0 "
						//	00 - Low level
						//	01 - Any edge
						//	10 - Falling edge
						//	11 - Rising edge
		
	#if EXT_INT0_ENABLE
		EICRA |= (EXT_INT0_SC << 0);
	#endif
	#if EXT_INT1_ENABLE
		EICRA |= (EXT_INT1_SC << 2);
	#endif
	#if EXT_INT2_ENABLE
		EICRA |= (EXT_INT2_SC << 4);
	#endif
			
	EIMSK = 0b00000000;	// 7:3 - Reserved Bits
							// 2:0  INT2:0: External Interrupt Request 2 - 0 Enable
	
	#if EXT_INT0_ENABLE
		EIMSK |= (EXT_INT0_ISR_ENABLE << 0);
	#endif
	#if EXT_INT1_ENABLE
		EIMSK |= (EXT_INT1_ISR_ENABLE << 1);
	#endif
	#if EXT_INT2_ENABLE
		EIMSK |= (EXT_INT2_ISR_ENABLE << 2);
	#endif
	
	EIFR = 0b00000000;	// 7:3 - Reserved Bits
						// 2:0  INTF2:0: External Interrupt Flags 2 - 0
						
	PCICR = 0b00000000;	// 7:4 - Reserved Bits
						// 3  PCIE3: Pin Change Interrupt Enable 3
						// 2  PCIE2: Pin Change Interrupt Enable 2
						// 1  PCIE1: Pin Change Interrupt Enable 1
						// 0  PCIE0: Pin Change Interrupt Enable 0
						
	PCIFR = 0b00000000;	// 7:4 - Reserved Bits
						// 3  PCIF3: Pin Change Interrupt Flag 3
						// 2  PCIF2: Pin Change Interrupt Flag 2
						// 1  PCIF1: Pin Change Interrupt Flag 1
						// 0  PCIF0: Pin Change Interrupt Flag 0
						
	PCMSK3 = 0b00000000;	// 7:0  PCINT31:24: Pin Change Enable Mask 31:24
	
	PCMSK2 = 0b00000000;	// 7:0  PCINT23:16: Pin Change Enable Mask 23:16
	
	PCMSK1 = 0b00000000;	// 7:0  PCINT15:8: Pin Change Enable Mask 15:8
	
	PCMSK0 = 0b00000000;	// 7:0  PCINT7:0: Pin Change Enable Mask 7:0
}