/* UART Constants */
/**
 * CPU_FREQ (MHz)	BAUD_RATE (bps)		Error (%)
 * 1				2400				0.2
 *					4800				0.2
 *					9600				-7.0 (Errors)
 *					14400				8.5 (Errors)
 *					19200				8.5 (Errors)
 *					28800				8.5 (Errors)
 *					38400				-18.6
 * 2				2400				0.2
 *					4800				0.2
 *					9600				0.2
 *					14400				-3.5
 *					19200				-7.0
 *					28800				8.5
 *					38400				8.5	
 * 4				2400				0.2
 *					4800				0.2
 *					9600				0.2
 *					14400				2.1
 *					19200				0.2
 *					28800				-3.5
 *					38400				-7.0					
 * 8				2400				0.2
 *					4800				0.2
 *					9600				0.2
 *					14400				-0.8
 *					19200				0.2
 *					28800				2.1
 *					38400				0.2					
 */
#define UART0_BAUD_RATE		4800
#define UART0_UBBR			((uint16_t) (((float)CPU_FREQUENCY_HZ) / (16*(float)UART0_BAUD_RATE) - 0.5))
#define UART1_BAUD_RATE		4800
#define UART1_UBBR			((uint16_t) (((float)CPU_FREQUENCY_HZ) / (16*(float)UART1_BAUD_RATE) - 0.5))

#define UART_RXTX_MODE		0
#define UART_RX_MODE		1
#define UART_TX_MODE		2

#define UART0_MODE			UART_TX_MODE
#define UART1_MODE			UART_TX_MODE

/* Prototype Functions */
void uart0_send_byte_array(uint8_t *data, uint8_t data_length);
void uart0_send_byte(uint8_t data);
void uart0_send_int(uint16_t data);
void uart0_send_long(uint32_t data);
void uart0_send_extended(uint64_t data);
void init_uart0_(uint8_t mode);

void uart1_send_byte_array(uint8_t *data, uint8_t data_length);
void uart1_send_byte(uint8_t data);
void uart1_send_int(uint16_t data);
void uart1_send_long(uint32_t data);
void uart1_send_extended(uint64_t data);
void init_uart1_(uint8_t mode);

/**
 * UART transmission function for byte arrays
 *
 * Can be used to transmitt strings
 * Length terminated (versus null terminated)
 */
void uart0_send_byte_array(uint8_t *data, uint8_t data_length) {
	for (uint8_t i=0; i<data_length; i++) {
		while ( !( UCSR0A & _BV(UDRE0)) );	// Wait for empty buffer
		UDR0 = *data++;						// Put data into buffer, sends the data 
	}
}

/**
 * UART transmission function for a single byte
 */
void uart0_send_byte(uint8_t data) {
	while ( !( UCSR0A & _BV(UDRE0)) );	// Wait for empty buffer
	UDR0 = data;							// Put data into buffer, sends the data 
}

/**
 *
 */
void uart0_send_int(uint16_t data) {
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) (data & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 8) & 0xFF);
}

/**
 *
 */
void uart0_send_long(uint32_t data) {
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) (data & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 8) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 16) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 24) & 0xFF);
}

/**
 *
 */
void uart0_send_extended(uint64_t data) {
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) (data & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 8) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 16) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 24) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 32) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 40) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 48) & 0xFF);
	while ( !( UCSR0A & _BV(UDRE0)) );
	UDR0 = (uint8_t) ((data >> 56) & 0xFF);
}

/**
 * UART transmission function for byte arrays
 *
 * Can be used to transmitt strings
 * Length terminated (versus null terminated)
 */
void uart1_send_byte_array(uint8_t *data, uint8_t data_length) {
	for (uint8_t i=0; i<data_length; i++) {
		while ( !( UCSR1A & _BV(UDRE1)) );	// Wait for empty buffer
		UDR0 = *data++;						// Put data into buffer, sends the data 
	}
}

/**
 * UART transmission function for a single byte
 */
void uart1_send_byte(uint8_t data) {
	while ( !( UCSR1A & _BV(UDRE1)) );	// Wait for empty buffer
	UDR0 = data;							// Put data into buffer, sends the data 
}

/**
 *
 */
void uart1_send_int(uint16_t data) {
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) (data & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 8) & 0xFF);
}

/**
 *
 */
void uart1_send_long(uint32_t data) {
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) (data & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 8) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 16) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 24) & 0xFF);
}

/**
 *
 */
void uart1_send_extended(uint64_t data) {
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) (data & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 8) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 16) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 24) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 32) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 40) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 48) & 0xFF);
	while ( !( UCSR1A & _BV(UDRE1)) );
	UDR0 = (uint8_t) ((data >> 56) & 0xFF);
}

/**
 * Initialize UART0
 *
 * For thread-based operation
 */
void init_uart0(void) {	
	cbi(PRR, PRUSART0);

	UBRR0H = (UART0_UBBR >> 8) & 0xFF;
	UBRR0L = UART0_UBBR & 0xFF;

	UCSR0A = 0b00000000;	// 7 - RXCn: UART RX complete flag
							// 6 - TXCn: UART TX complete flag
							// 5 - UDREn: UART DRE flag
							// 4 - FEn: Frame error flag
							// 3 - DORn: Data overrun flag
							// 2 - UPEn: Parity error flag
							// 1 - U2Xn: Double UART speed mode
							// 0 - MPCMn: Multi processor communication mode

	UCSR0B = 0b00000000; 	// 7 - RXCIEn: RX Complete Interrupt Enable
							// 6 - TXCIEn: TX Complete Interrupt Enable
							// 5 - UDRIEn: UART DRE Interrupt Enable
							// 4 - RXENn: Enable receiver
							// 3 - TXENn: Enable transmitter
							// 2 - UCSZn2: Character size
							// 1 - RXB8n: Receive Data Bit 8
							// 0 - TXB8n: Transmit Data Bit 8
	#if UART0_MODE == UART_RXTX_MODE
		sbi(UCSR0B, RXEN0);
		sbi(UCSR0B, TXEN0);
	#elif UART0_MODE == UART_RX_MODE
		sbi(UCSR0B, RXEN0);
	#else
		sbi(UCSR0B, TXEN0);
	#endif

	UCSR0C = 0b00000110;	// 7:6 - UMSELn1:0: UART Mode Select
							//		00 - Asynchronous UART
							//		01 - Synchronous UART
							//		10 - Reserved
							//		11 - Master SPI
							// 5:4 - UPMn1:0: Parity mode
							//		00 - Disabled
							//		01 - Reserved
							//		10 - Enabled, Even
							//		11 - Enabled, Odd
							// 3 - USBSn: Select stop bit length
							//		0 - 1-bit
							//		1 - 2-bit
							// 2:1 - UCSZn1:0: Character size, with UCSRB.2
							//		000 - 5 bit
							//		001 - 6 bit
							//		010 - 7 bit
							//		011 - 8 bit
							//		100 - Reserved
							//		101 - Reserved
							//		110 - Reserved
							//		111 - 9 bit
							// 0 - UCPOLn: Select clock polarity
							//		0 - Rising XCK edge
							//		1 - Falling XCK edge 						
}

/**
 * Initialize UART1
 *
 * For thread-based operation
 */
void init_uart1(void) {	
	cbi(PRR, PRUSART1);

	UBRR1H = (UART1_UBBR >> 8) & 0xFF;
	UBRR1L = UART1_UBBR & 0xFF;

	UCSR1A = 0b00000000;	// 7 - RXCn: UART RX complete flag
							// 6 - TXCn: UART TX complete flag
							// 5 - UDREn: UART DRE flag
							// 4 - FEn: Frame error flag
							// 3 - DORn: Data overrun flag
							// 2 - UPEn: Parity error flag
							// 1 - U2Xn: Double UART speed mode
							// 0 - MPCMn: Multi processor communication mode

	UCSR1B = 0b00000000; 	// 7 - RXCIEn: RX Complete Interrupt Enable
							// 6 - TXCIEn: TX Complete Interrupt Enable
							// 5 - UDRIEn: UART DRE Interrupt Enable
							// 4 - RXENn: Enable receiver
							// 3 - TXENn: Enable transmitter
							// 2 - UCSZn2: Character size
							// 1 - RXB8n: Receive Data Bit 8
							// 0 - TXB8n: Transmit Data Bit 8
	#if UART1_MODE == UART_RXTX_MODE
		sbi(UCSR1B, RXEN1);
		sbi(UCSR1B, TXEN1);
	#elif UART1_MODE == UART_RX_MODE
		sbi(UCSR1B, RXEN1);
	#else
		sbi(UCSR1B, TXEN1);
	#endif

	UCSR1C = 0b00000110;	// 7:6 - UMSELn1:0: UART Mode Select
							//		00 - Asynchronous UART
							//		01 - Synchronous UART
							//		10 - Reserved
							//		11 - Master SPI
							// 5:4 - UPMn1:0: Parity mode
							//		00 - Disabled
							//		01 - Reserved
							//		10 - Enabled, Even
							//		11 - Enabled, Odd
							// 3 - USBSn: Select stop bit length
							//		0 - 1-bit
							//		1 - 2-bit
							// 2:1 - UCSZn1:0: Character size, with UCSRB.2
							//		000 - 5 bit
							//		001 - 6 bit
							//		010 - 7 bit
							//		011 - 8 bit
							//		100 - Reserved
							//		101 - Reserved
							//		110 - Reserved
							//		111 - 9 bit
							// 0 - UCPOLn: Select clock polarity
							//		0 - Rising XCK edge
							//		1 - Falling XCK edge 						
}