/* External Interrupt Constants */
#define EXT_INT_SC_LOW_LEVEL	0
#define EXT_INT_SC_ANY_EDGE		1
#define EXT_INT_SC_FALLING_EDGE	2
#define EXT_INT_SC_RISING_EDGE	3

#define EXT_INT0_DDR		DDRD
#define EXT_INT0_PORT		PORTD
#define EXT_INT0_PIN		PIND
#define EXT_INT0_INPUT		PD2

#define EXT_INT1_DDR		DDRD
#define EXT_INT1_PORT		PORTD
#define EXT_INT1_PIN		PIND
#define EXT_INT1_INPUT		PD3

#define EXT_INT2_DDR		DDRB
#define EXT_INT2_PORT		PORTB
#define EXT_INT2_PIN		PINB
#define EXT_INT2_INPUT		PB2

/* Prototype Functions */
void init_ext_int0(uint8_t isr_enable, uint8_t sc_mode);
void init_ext_int1(uint8_t isr_enable, uint8_t sc_mode);
void init_ext_int2(uint8_t isr_enable, uint8_t sc_mode);
void init_pc_int(void);
void ext_int_enable_int0(void);
void ext_int_disable_int0(void);
void ext_int_enable_int1(void);
void ext_int_disable_int1(void);
void ext_int_enable_int2(void);
void ext_int_disable_int2(void);

/**
 * Interrupt vectors:
 *
 * INT0_vect
 * INT1_vect
 * INT2_vect
 * PCINT1_vect
 * PCINT2_vect
 * PCINT3_vect
 */
 
void ext_int_enable_int0(void) {
	sbi(EIMSK, 0);
}
 
void ext_int_disable_int0(void) {
	cbi(EIMSK, 0);
}
 
void ext_int_enable_int1(void) {
	sbi(EIMSK, 1);
}
 
void ext_int_disable_int1(void) {
	cbi(EIMSK, 1);
}

void ext_int_enable_int2(void) {
	sbi(EIMSK, 2);
}
 
void ext_int_disable_int2(void) {
	cbi(EIMSK, 2);
}

/**
 * Initialize External Int0
 */
void init_ext_int0(uint8_t isr_enable, uint8_t sc_mode) {
	EICRA = (EICRA & ~0b00000011) | sc_mode;	// 7:6 - Reserved Bits
												// 5:4 - ISC21, ISC20 - External Int 2 Sense Control Bits
												// 3:2 - ISC11, ISC10 - External Int 1 "
												// 1:0 - ISC01, ISC00 - External Int 0 "
												//	00 - Low level
												//	01 - Any edge
												//	10 - Falling edge
												//	11 - Rising edge
			
	EIMSK = (EIMSK & ~0b00000001) | isr_enable;// 7:3 - Reserved Bits
												// 2:0  INT2:0: External Interrupt Request 2 - 0 Enable
	
	EIFR = 0b00000000;	// 7:3 - Reserved Bits
						// 2:0  INTF2:0: External Interrupt Flags 2 - 0
}

/**
 * Initialize External Int1
 */
void init_ext_int1(uint8_t isr_enable, uint8_t sc_mode) {
	EICRA = (EICRA & ~0b00001100) | (sc_mode << 2);// 7:6 - Reserved Bits
													// 5:4 - ISC21, ISC20 - External Int 2 Sense Control Bits
													// 3:2 - ISC11, ISC10 - External Int 1 "
													// 1:0 - ISC01, ISC00 - External Int 0 "
													//	00 - Low level
													//	01 - Any edge
													//	10 - Falling edge
													//	11 - Rising edge
			
	EIMSK = (EIMSK & ~0b00000010) | (isr_enable << 1);	// 7:3 - Reserved Bits
														// 2:0  INT2:0: External Interrupt Request 2 - 0 Enable
	
	EIFR = 0b00000000;	// 7:3 - Reserved Bits
						// 2:0  INTF2:0: External Interrupt Flags 2 - 0
}

/**
 * Initialize External Int2
 */
void init_ext_int2(uint8_t isr_enable, uint8_t sc_mode) {
	EICRA = (EICRA & ~0b00110000) | (sc_mode << 4);// 7:6 - Reserved Bits
													// 5:4 - ISC21, ISC20 - External Int 2 Sense Control Bits
													// 3:2 - ISC11, ISC10 - External Int 1 "
													// 1:0 - ISC01, ISC00 - External Int 0 "
													//	00 - Low level
													//	01 - Any edge
													//	10 - Falling edge
													//	11 - Rising edge
			
	EIMSK = (EIMSK & ~0b00000100) | (isr_enable << 2);	// 7:3 - Reserved Bits
														// 2:0  INT2:0: External Interrupt Request 2 - 0 Enable
	
	EIFR = 0b00000000;	// 7:3 - Reserved Bits
						// 2:0  INTF2:0: External Interrupt Flags 2 - 0
}

void init_pc_int(void) {
	PCICR = 0b00000000;	// 7:4 - Reserved Bits
						// 3  PCIE3: Pin Change Interrupt Enable 3
						// 2  PCIE2: Pin Change Interrupt Enable 2
						// 1  PCIE1: Pin Change Interrupt Enable 1
						// 0  PCIE0: Pin Change Interrupt Enable 0
						
	PCIFR = 0b00000000;	// 7:4 - Reserved Bits
						// 3  PCIF3: Pin Change Interrupt Flag 3
						// 2  PCIF2: Pin Change Interrupt Flag 2
						// 1  PCIF1: Pin Change Interrupt Flag 1
						// 0  PCIF0: Pin Change Interrupt Flag 0
						
	PCMSK3 = 0b00000000;	// 7:0  PCINT31:24: Pin Change Enable Mask 31:24
	
	PCMSK2 = 0b00000000;	// 7:0  PCINT23:16: Pin Change Enable Mask 23:16
	
	PCMSK1 = 0b00000000;	// 7:0  PCINT15:8: Pin Change Enable Mask 15:8
	
	PCMSK0 = 0b00000000;	// 7:0  PCINT7:0: Pin Change Enable Mask 7:0
}