/**
 * A Mega324P LCD Test
 *
 * Ruibing Wang (rw98@cornell.edu)
 */

 /**
 * Mega324P DIP Pinout
 * 							___________
 * PB0 (PCINT8/XCK0/T0)		|1		40| PA0 (ADC0/PCINT0)
 * PB1 (PCINT9/CLK0/T1)		|2		39| PA1 (ADC1/PCINT1)
 * PB2 (PCINT10/INT2/AIN0)	|3		38| PA2 (ADC2/PCINT2)
 * PB3 (PCINT11/OC0A/AIN1)	|4		37| PA3 (ADC3/PCINT3)
 * PB4 (PCINT12/OCOB/SS)	|5		36| PA4 (ADC4/PCINT4)
 * PB5 (PCINT13/MOSI)		|6		35| PA5 (ADC5/PCINT5)
 * PB6 (PCINT14/MISO)		|7		34| PA6 (ADC6/PCINT6)
 * PB7 (PCINT15/SCK)		|8		33| PA7 (ADC7/PCINT7)
 * RESET					|9		32| AREF
 * VCC						|10		31| GND
 * GND						|11		30| AVCC
 * XTAL2					|12		29| PC7 (TOSC2/PCINT23)
 * XTAL1					|13		28| PC6 (TOSC1/PCINT22)
 * PD0 (PCINT24/RXD0)		|14		27| PC5 (TDI/PCINT21)
 * PD1 (PCINT25/TXD0)		|15		26| PC4 (TDO/PCINT20)
 * PD2 (PCINT26/RXD1/INT0)	|16		25| PC3 (TMS/PCINT19)
 * PD3 (PCINT27/TXD1/INT1)	|17		24| PC2 (TCK/PCINT18)
 * PD4 (PCINT28/XCK1/OC1B)	|18		23| PC1 (SDA/PCINT17)
 * PD5 (PCINT29/OC1A)		|19		22| PC0 (SCL/PCINT16)
 * PD6 (PCINT30/OC2B/ICP)	|20		21| PD7 (OC2/PCINT31)
 *							-----------
 */

/** 
 * Use an 2x16 alphanumeric LCD connected to PORTC as follows:
 *  
 *	[LCD] -	[Mega324P Pin]
 *	1 GND -	GND
 *	2 +5V -	VCC
 *	3 VLC 10k trimpot wiper (trimpot ends go to +5 and gnd) 
 *	4 RS -	PC0
 *	5 RD -	PC1
 *	6 EN -	PC2
 *	11 D4 -	PC4
 *	12 D5 -	PC5
 *	13 D6 -	PC6
 *	14 D7 -	PC7 
 */

#include <mega324.h>
#include <stdio.h>
#include <delay.h>

/* Constants for LCD */
#define LCD_CHAR_LENGTH 16
#define LCD_CHAR_WIDTH	2
#asm
    .equ __lcd_port=0x08			// PORTC Register Address (Mega32=0x15, Mega324P=0x08)
#endasm
#include <lcd.h>

unsigned char lcd_buffer[LCD_CHAR_LENGTH + 1];	// LCD display buffer
unsigned int count;								//a number to display on the LCD  
unsigned char anipos, dir;						//move a character around  

void main(void) {
	lcd_init(LCD_CHAR_LENGTH);	//initialize the display
	lcd_clear();				//clear the display
	lcd_gotoxy(8,1);			//print next stuff at position 7, line 2 
	lcd_putsf("line 2");		//string from flash
	lcd_gotoxy(0,0);			//position to upper left on display
	lcd_putsf("Number=");		//another string from flash
	count=0;
	anipos = 0;
	lcd_gotoxy(anipos,1);		//second line
	lcd_putsf("o");

	while (1) {
		sprintf(lcd_buffer,"%-i",count++);	//increment and format string 
		lcd_gotoxy(7,0);                    
		lcd_puts(lcd_buffer);				// display the count   
      	lcd_gotoxy(anipos,1);	   //second line
      	lcd_putsf(" "); 
      	if (anipos>=7) 
      		dir=-1;
		if (anipos<=0 ) dir=1;
			anipos=anipos+dir;
		lcd_gotoxy(anipos,1);	   //second line
		lcd_putsf("o"); 
      	delay_ms(200);    //never do this in a realtime program
   }
}
  
