/**
 * A Mega324P UART RX Test
 *
 * Ruibing Wang (rw98@cornell.edu)
 */
 
 /**
 * Mega324P DIP Pinout
 * 							___________
 * PB0 (PCINT8/XCK0/T0)		|1		40| PA0 (ADC0/PCINT0)
 * PB1 (PCINT9/CLK0/T1)		|2		39| PA1 (ADC1/PCINT1)
 * PB2 (PCINT10/INT2/AIN0)	|3		38| PA2 (ADC2/PCINT2)
 * PB3 (PCINT11/OC0A/AIN1)	|4		37| PA3 (ADC3/PCINT3)
 * PB4 (PCINT12/OCOB/SS)	|5		36| PA4 (ADC4/PCINT4)
 * PB5 (PCINT13/MOSI)		|6		35| PA5 (ADC5/PCINT5)
 * PB6 (PCINT14/MISO)		|7		34| PA6 (ADC6/PCINT6)
 * PB7 (PCINT15/SCK)		|8		33| PA7 (ADC7/PCINT7)
 * RESET					|9		32| AREF
 * VCC						|10		31| GND
 * GND						|11		30| AVCC
 * XTAL2					|12		29| PC7 (TOSC2/PCINT23)
 * XTAL1					|13		28| PC6 (TOSC1/PCINT22)
 * PD0 (PCINT24/RXD0)		|14		27| PC5 (TDI/PCINT21)
 * PD1 (PCINT25/TXD0)		|15		26| PC4 (TDO/PCINT20)
 * PD2 (PCINT26/RXD1/INT0)	|16		25| PC3 (TMS/PCINT19)
 * PD3 (PCINT27/TXD1/INT1)	|17		24| PC2 (TCK/PCINT18)
 * PD4 (PCINT28/XCK1/OC1B)	|18		23| PC1 (SDA/PCINT17)
 * PD5 (PCINT29/OC1A)		|19		22| PC0 (SCL/PCINT16)
 * PD6 (PCINT30/OC2B/ICP)	|20		21| PD7 (OC2/PCINT31)
 *							-----------
 */

#include <mega324.h>
#include <string.h>
#include <stdio.h>
#include <delay.h>

unsigned char serial_buffer[32];
unsigned char serial_buffer_index;
unsigned char temp_char;

void init_mcu(void);
void init_uart_rx(void);
void parse_char(unsigned char c);


/**
 * UART RX interrupt
 */
 
interrupt [USART_RXC] void uart_rx(void) {
	temp_char = UDR;
	parse_char(temp_char);
}

void parse_char(unsigned char c) {

}

void main(void) {
	init_uart_rx();
	delay_ms(100);
	
	while(1) {
	}
}

void init_uart_rx(void) {
	UCSR0A = 0b00000000;// 7 - UART RX complete flag
						// 6 - UART TX complete flag
						// 5 - UART DRE flag
						// 4 - Frame error flag
						// 3 - Data overrun flag
						// 2 - Parity error flag
						// 1 - Double UART speed mode (Disabled)
						// 0 - Multi processor communication mode (Disabled)

	UCSR0B = 0b10010000;// 7 - RX Complete Interrupt (Enabled)
						// 6 - TX Complete Interrupt (Disabled)
						// 5 - UART DRE Interrupt (Disabled)
						// 4 - Enable receiver (Enabled)
						// 3 - Disable transmitter (Disabled)
						// 2 - Character size (8-bit)
						// 1:0 - Ninth data bit

	UCSR0C = 0b10000110;// 7:6 - UART Mode Select
						//		00 - Asynchronous UART
						//		01 - Synchronous UART
						//		10 - Reserved
						//		11 - Master SPI
						// 5:4 - Enable odd parity mode (Disabled)
						//		00 - Disabled
						//		01 - Reserved
						//		10 - Enabled, Even
						//		11 - Enabled, Odd
						// 3 - Select stop bit length (1-bit)
						//		0 - 1-bit
						//		1 - 2-bit
						// 2:1 - Character size, with UCSRB.2 (8-bit)
						//		000 - 5 bit
						//		001 - 6 bit
						//		010 - 7 bit
						//		011 - 8 bit
						//		100 - Reserved
						//		101 - Reserved
						//		110 - Reserved
						//		111 - 9 bit
						// 0 - Select clock polarity (Disabled)
						//		0 - Rising XCK edge
						//		1 - Falling XCK edge
	
	UBRRH = 0b00000011;
	UBRRL = 0b01000000;	// UBRR = 832 = 0b1101000000
	
	#asm("sei");
}

void init_mcu(void) {
	SMCR = 0b00000000;	// 7:4 - Reserved bits
						// 3:1 - Sleep Mode
						//		000 - Idle
						//		001 - ADC Noise Reduction
						//		010 - Power-down
						//		011 - Power-save
						//		100 - Reserved
						//		101 - Reserved
						//		110 - Standby
						//		111 - Extended Standby
						// 0 - Sleep Enable
						
	MCUCR |= (0 << 6);	// 6 - BOD Sleep
	MCUCR |= (0 << 5);	// 5 - BOD Sleep Enable
	
	PRR = 0b11111101;	// 7 - Power Reduction TWI
						// 6 - Power Reduction Timer2
						// 5 - Power Reduction Timer0
						// 4 - Power Reduction UART1
						// 3 - Power Reduction Timer1
						// 2 - Power Reduction SPI
						// 1 - Power Reduction UART0
						// 0 - Power Reduction ADC
} 
	
	
						
	
	
							 
