/**
 * A Mega32 UART RX Test
 *
 * Ruibing Wang (rw98@cornell.edu)
 */
 
 /**
 * Mega32 PDIP Pinout
 * 				___________
 * PB0 (XCK/T0)		|1		40| PA0 (ADC0)
 * PB1 (T1)			|2		39| PA1 (ADC1)
 * PB2 (INT2/AIN0)	|3		38| PA2 (ADC2)
 * PB3 (OC0/AIN1)	|4		37| PA3 (ADC3)
 * PB4 (SS)			|5		36| PA4 (ADC4)
 * PB5 (MOSI)		|6		35| PA5 (ADC5)
 * PB6 (MISO)		|7		34| PA6 (ADC6)
 * PB7 (SCK)		|8		33| PA7 (ADC7)
 * RESET			|9		32| AREF
 * VCC			|10		31| GND
 * GND			|11		30| AVCC
 * XTAL2			|12		29| PC7 (TOSC2)
 * XTAL1			|13		28| PC6 (TOSC1)
 * PD0 (RXD)		|14		27| PC5 (TDI)
 * PD1 (TXD)		|15		26| PC4 (TDO)
 * PD2 (INT0)		|16		25| PC3 (TMS)
 * PD3 (INT1)		|17		24| PC2 (TCK)
 * PD4 (OC1B)		|18		23| PC1 (SDA)
 * PD5 (OC1A)		|19		22| PC0 (SCL)
 * PD6 (ICP1)		|20		21| PD7 (OC2)
 *				-----------
 */

#include <Mega32.h>
#include <string.h>
#include <stdio.h>
#include <delay.h>

unsigned char serial_buffer[32];
unsigned char serial_buffer_index;
unsigned char temp_char;

void init_uart_rx(void);
void parse_char(unsigned char c);

/**
 * UART RX interrupt
 */
 
interrupt [USART_RXC] void uart_rx(void) {
	temp_char = UDR;
	parse_char(temp_char);
}

void parse_char(unsigned char c) {

}

void main(void) {
	init_uart_rx();
	delay_ms(100);
	
	while(1) {
	}
}

void init_uart_rx(void) {
	UCSRA = 0b00000000;	// 7 - UART RX complete flag
						// 6 - UART TX complete flag
						// 5 - UART DRE flag
						// 4 - Frame error flag
						// 3 - Data overrun flag
						// 2 - Parity error flag
						// 1 - Double UART speed mode (Disabled)
						// 0 - Multi processor communication mode (Disabled)

	UCSRB = 0b10010000; // 7 - RX Complete Interrupt (Enabled)
						// 6 - TX Complete Interrupt (Disabled)
						// 5 - UART DRE Interrupt (Disabled)
						// 4 - Enable receiver (Enabled)
						// 3 - Disable transmitter (Disabled)
						// 2 - Character size (8-bit)
						// 1:0 - Ninth data bit

	UCSRC = 0b10000110;	// 7 - Access UBRRH or UCSRC (UCSRC)
						//		0 - UBRRH
						//		1 - UCSRC
						// 6 - Select UART mode (Asynchronous)
						//		0 - Asynchronous
						//		1 - Synchronous
						// 5:4 - Enable odd parity mode (Disabled)
						//		00 - Disabled
						//		01 - Reserved
						//		10 - Enabled, Even
						//		11 - Enabled, Odd
						// 3 - Select stop bit length (1-bit)
						//		0 - 1-bit
						//		1 - 2-bit
						// 2:1 - Character size, with UCSRB.2 (8-bit)
						//		000 - 5 bit
						//		001 - 6 bit
						//		010 - 7 bit
						//		011 - 8 bit
						//		100 - Reserved
						//		101 - Reserved
						//		110 - Reserved
						//		111 - 9 bit
						// 0 - Select clock polarity (Disabled)
						//		0 - Rising XCK edge
						//		1 - Falling XCK edge
	
	UBRRH = 0b00000011;
	UBRRL = 0b01000000;	// UBRR = 832 = 0b1101000000
	
	#asm("sei");
}						 
