/**
 * A Mega644P SPI Master Test
 * - Fixed for the need of junk data
 *
 * Ruibing Wang (rw98@cornell.edu)
 */
 
 /**
 * Mega164P/324P/644P DIP Pinout
 * 							___________
 * PB0 (PCINT8/XCK0/T0)		|1		40| PA0 (ADC0/PCINT0)
 * PB1 (PCINT9/CLK0/T1)		|2		39| PA1 (ADC1/PCINT1)
 * PB2 (PCINT10/INT2/AIN0)	|3		38| PA2 (ADC2/PCINT2)
 * PB3 (PCINT11/OC0A/AIN1)	|4		37| PA3 (ADC3/PCINT3)
 * PB4 (PCINT12/OCOB/SS)	|5		36| PA4 (ADC4/PCINT4)
 * PB5 (PCINT13/MOSI)		|6		35| PA5 (ADC5/PCINT5)
 * PB6 (PCINT14/MISO)		|7		34| PA6 (ADC6/PCINT6)
 * PB7 (PCINT15/SCK)		|8		33| PA7 (ADC7/PCINT7)
 * RESET					|9		32| AREF
 * VCC						|10		31| GND
 * GND						|11		30| AVCC
 * XTAL2					|12		29| PC7 (TOSC2/PCINT23)
 * XTAL1					|13		28| PC6 (TOSC1/PCINT22)
 * PD0 (PCINT24/RXD0)		|14		27| PC5 (TDI/PCINT21)
 * PD1 (PCINT25/TXD0)		|15		26| PC4 (TDO/PCINT20)
 * PD2 (PCINT26/RXD1/INT0)	|16		25| PC3 (TMS/PCINT19)
 * PD3 (PCINT27/TXD1/INT1)	|17		24| PC2 (TCK/PCINT18)
 * PD4 (PCINT28/XCK1/OC1B)	|18		23| PC1 (SDA/PCINT17)
 * PD5 (PCINT29/OC1A)		|19		22| PC0 (SCL/PCINT16)
 * PD6 (PCINT30/OC2B/ICP)	|20		21| PD7 (OC2/PCINT31)
 *							-----------
 */

/* Includes */
#include <mega644p.h>
#include <string.h>
#include <stdio.h>
#include <delay.h>

/* Global Constants */
#define TRUE 	1
#define FALSE 	0
#define SPIF	7

unsigned char counter;

void init_spi_master(void);

unsigned char spi_send_receive_byte(unsigned char tx_data) {
	SPDR = tx_data;
	
	while(!(SPSR & (1 << SPIF)));

	return SPDR;
}

void main() {
	DDRA = 0xFF;
	PORTA = 0xFF;
	
	init_spi_master();
	delay_ms(500);
	
	while(1) {
		spi_send_receive_byte(counter++);
		PORTA = ~counter;
		delay_ms(500);
	}
}


void init_spi_master(void) {
	DDRB.4 = 1; // SS as Output
	DDRB.5 = 1;	// MOSI as Output
	DDRB.6 = 0;	// MISO as Input
	DDRB.7 = 1;	// SCK as Output

	SPCR = 0b01010001;	// 7 - SPI Interrupt Enable
						// 6 - SPI Enable
						// 5 - Data Order
						// 4 - Master/Slave Select
						// 3 - Clock Polarity
						// 2 - Clock Phase
						// 1:0 - SPI Clock Rate
						//	SPI2X = 0
						//		00 - 1/4
						//		01 - 1/16
						//		10 - 1/64
						//		11 - 1/128
						//	SPI2X = 1
						//		00 - 1/2
						//		01 - 1/8
						//		10 - 1/32
						//		11 - 1/64
						
	SPSR = 0b00000000;	// 7 - SPI Interrupt Flag
						// 6 - Write COLlision Flag
						// 5:1 - Reserved Bits
						// 0 - Double SPI Speed Bit
} 

void init_mcu(void) {
	SMCR = 0b00000000;	// 7:4 - Reserved bits
						// 3:1 - Sleep Mode
						//		000 - Idle
						//		001 - ADC Noise Reduction
						//		010 - Power-down
						//		011 - Power-save
						//		100 - Reserved
						//		101 - Reserved
						//		110 - Standby
						//		111 - Extended Standby
						// 0 - Sleep Enable
						
	MCUCR |= (0 << 6);	// 6 - BOD Sleep
	MCUCR |= (0 << 5);	// 5 - BOD Sleep Enable
	
	PRR = 0b11111001;	// 7 - Power Reduction TWI
						// 6 - Power Reduction Timer2
						// 5 - Power Reduction Timer0
						// 4 - Power Reduction UART1
						// 3 - Power Reduction Timer1
						// 2 - Power Reduction SPI
						// 1 - Power Reduction UART0
						// 0 - Power Reduction ADC
}