/**
 * A Mega644P TWI Master Library Test
 *
 * Ruibing Wang (rw98@cornell.edu)
 */

 /**
 * Mega164P/324P/644P DIP Pinout
 * 							___________
 * PB0 (PCINT8/XCK0/T0)		|1		40| PA0 (ADC0/PCINT0)
 * PB1 (PCINT9/CLK0/T1)		|2		39| PA1 (ADC1/PCINT1)
 * PB2 (PCINT10/INT2/AIN0)	|3		38| PA2 (ADC2/PCINT2)
 * PB3 (PCINT11/OC0A/AIN1)	|4		37| PA3 (ADC3/PCINT3)
 * PB4 (PCINT12/OCOB/SS)	|5		36| PA4 (ADC4/PCINT4)
 * PB5 (PCINT13/MOSI)		|6		35| PA5 (ADC5/PCINT5)
 * PB6 (PCINT14/MISO)		|7		34| PA6 (ADC6/PCINT6)
 * PB7 (PCINT15/SCK)		|8		33| PA7 (ADC7/PCINT7)
 * RESET					|9		32| AREF
 * VCC						|10		31| GND
 * GND						|11		30| AVCC
 * XTAL2					|12		29| PC7 (TOSC2/PCINT23)
 * XTAL1					|13		28| PC6 (TOSC1/PCINT22)
 * PD0 (PCINT24/RXD0)		|14		27| PC5 (TDI/PCINT21)
 * PD1 (PCINT25/TXD0)		|15		26| PC4 (TDO/PCINT20)
 * PD2 (PCINT26/RXD1/INT0)	|16		25| PC3 (TMS/PCINT19)
 * PD3 (PCINT27/TXD1/INT1)	|17		24| PC2 (TCK/PCINT18)
 * PD4 (PCINT28/XCK1/OC1B)	|18		23| PC1 (SDA/PCINT17)
 * PD5 (PCINT29/OC1A)		|19		22| PC0 (SCL/PCINT16)
 * PD6 (PCINT30/OC2B/ICP)	|20		21| PD7 (OC2/PCINT31)
 *							-----------
 */

/* Includes */
#include <mega644p.h>
#include <string.h>
#include <stdio.h>
#include <delay.h>
#include "twi_master.h"

/* Global Constants */
#define TRUE 	1
#define FALSE 	0

/* System Constants */
#define CPU_FREQUENCY_HZ	20000000

/* User Settings for TWI */
#define TWI_SLAVE_ADDRESS				2
#define TWI_PULLUP_RESISTOR_ENABLE		TRUE


void main(void) {
	unsigned char rx_data;
	rx_data = 0;

	DDRB = 0xff;
	PORTB = 0xff;

	init_twi_master(CPU_FREQUENCY_HZ, TWI_PULLUP_RESISTOR_ENABLE);
	
	#asm("sei");
	
	while(1) {
		/* Unable to resume after faults from slave */
// 		twi_master_start(TWI_SLAVE_ADDRESS, TWI_WRITE);
// 		twi_write_byte(0xAA);      
// 		twi_master_start(TWI_SLAVE_ADDRESS, TWI_READ);
// 		delay_ms(1);
// 		twi_read_byte(&rx_data);
// 		delay_ms(1);  
// 		PORTB= ~rx_data;
// 		twi_stop();

		/* Able to resume after faults from slave */
		if (!twi_master_start(TWI_SLAVE_ADDRESS, TWI_WRITE)) {
			if (!twi_write_byte(0xAA)) {      
				if (!twi_master_start(TWI_SLAVE_ADDRESS, TWI_READ)) {
					delay_ms(1);
					if (!twi_read_byte(&rx_data)) {
						delay_ms(1);  
						PORTB= ~rx_data;
					}
				}
			}
		}
		twi_stop(); 

 	}
}


