/* ZigBee Constants */
#define ZIGBEE_FRAME_BUFFER_LENGTH	127	// An internal 128 byte RAM for RX and TX (Frame Buffer) 
										// buffers the data to be transmitted or the received data.

/* ZigBee MCU Ports */
#define ZIGBEE_DDR			DDRB
#define ZIGBEE_PORT			PORTB
#define ZIGBEE_SLP_TR		PB0
#define ZIGBEE_RST			PB1

/* ZigBee Registers */
#define ZIGBEE_REG_TRX_STATUS	0x01	// Signals the current state of the radio transceiver as well as
										// the status of the CCA measurement.
										//	A read clears bits CCA_DONE and CCA_STATUS
										//	7 - CCA_DONE		- indicates if a CCA request is completed
										//		0 - CCA calculation CCA_DONE in progress (*)
										//		1 - CCA calculation finished
										//	6 - CCA_STATUS		- indicates the result of a CCA request
										//		0 - Channel is busy (*)
										//		1 - Channel is idle
										//	5 - Reserved
										//	4:0 - TRX_STATUS	- current radio transceiver status
										//		(see below)
#define ZIGBEE_REG_TRX_STATE	0x02	// Controls the radio transceiver states via register bits TRX_CMD
										//	7:5 - TRAC_STATUS	- status of the TX_ARET algorithm
										//		(see below)
										//	4:0 - TRX_CMD		- initiates a state transition towards the new state
										//		(see below)
#define ZIGBEE_REG_TRX_CTRL_0	0x03	// Controls the drive current of the digital output pads and the
										// CLKM clock rate
										//	7:6 - PAD_IO		- sets the output driver current of digital output MISO and IRQ
										//		0 - 2mA (*)
										//		1 - 4mA
										//		2 - 6mA
										//		3 - 8mA
										//	5:6 - PAD_IO_CLKM	- sets the output driver current of pin CLKM
										//		0 - 2mA
										//		1 - 4mA (*)
										//		2 - 6mA
										//		3 - 8mA
										//	3 - CLKM_SHA_SEL	- defines the commencement of the CLKM clock rate modifications when changing register bits CLKM_CTRL
										//		0 - CLKM clock rate changes immediately
										//		1 - CLKM clock rate changes after SLEEP cycle (*)
										//	2:0 - CLKM_CTRL
										//		(see below)
#define ZIGBEE_REG_PHY_TX_PWR	0x05	// Sets the transmit power and controls the FCS algorithm for TX operation
										//	7 - TX_AUTO_CRC_ON 	- controls the automatic FCS generation for TX operation
										//		0 - Disabled (*)
										//		1 - Enabled
										//	6:4 - Reserved
										//	3:0 - TX_PWR		- sets the TX output power (dBm) of the AT86RF230
										//		0b0000 - +3.0
										//		0b0001 - +2.6
										//		0b0010 - +2.1
										//		0b0011 - +1.6
										//		0b0100 - +1.1
										//		0b0101 - +0.5
										//		0b0110 - -0.2
										//		0b0111 - -1.2
										//		0b1000 - -2.2
										//		0b1001 - -3.2
										//		0b1010 - -4.2
										//		0b1011 - -5.2
										//		0b1100 - -7.2
										//		0b1101 - -9.2
										//		0b1110 - -12.2
										//		0b1111 - -17.2
#define ZIGBEE_REG_PHY_RSSI		0x06	// A multi purpose register to indicate the current received signal strength (RSSI) 
										// and the FCS validity of a received frame
										//	7 - RX_CRC_VALID 	- indicates whether a received frame has a valid FCS
										//		0 - Invalid (*)
										//		1 - Valid
										//	6:5 - Reserved
										//	4:0 - RSSI 			- contain the result of the automated RSSI measurement
										//		0 - RF input power of < -91 dBm
										//		...
										//		28 - " ≥ -10 dBm
#define ZIGBEE_REG_PHY_ED_LEVEL	0x07	// ED_LEVEL
#define ZIGBEE_REG_PHY_CC_CCA	0x08	// Contains register bits to initiate and control the CCA measurement 
										// as well as to set the channel center frequency
										//	7 - CCA_REQUEST		- initiates manual CCA measurement request
										//	6:5 - CCA_MODE
										//		0 - Reserved
										//		1 - Mode 1, Energy above threshold (*)
										//		2 - Mode 2, Carrier sense only
										//		3 - Mode 3, Carrier sense with energy above threshold
										//	4:0 - CHANNEL		- define the RX/TX channel
										//		(see below)
#define ZIGBEE_REG_CCA_THRES	0x09	// Contains the threshold level for CCA-ED measurement
										//	7:4 - Reserved
										//	3:0 - CCA_ED_THRES	- define the threshold value of the CCA-ED measurement
										//						A cannel is indicated as busy when RSSI_BASE_VAL + 2 � CCA_ED_THRES [dBm]
										//		0b0111 - (*)
#define ZIGBEE_REG_IRQ_MASK		0x0E	// Used to enable (1) or disable (0) interrupt events
										// Should read the IRQ_STATUS register first to clear the history
										//	7 - MASK_BAT_LOW
										//	6 - MASK_TRX_UR
										//	5:4 - Reserved
										//	3 - MASK_TRX_END
										//	2 - MASK_RX_START
										//	1 - MASK_PLL_UNLOCK
										//	0 - MASK_PLL_LOCK
#define ZIGBEE_REG_IRQ_STATUS	0x0F	// Contains the status of the individual interrupts
										// A read access to this register resets all interrupt bits
										//	7 - BAT_LOW
										//	6 - TRX_UR
										//	5:4 - Reserved
										//	3 - TRX_END
										//	2 - RX_START
										//	1 - PLL_UNLOCK
										//	0 - PLL_LOCK
#define ZIGBEE_REG_VREG_CTRL	0x10	// controls the use of the voltage regulators and indicates the status of these
										//	7 - AVREG_EXT		- defines whether the internal analog voltage regulator or 
										//						an external regulator is used to supply the analog low 
										//						voltage building blocks of the radio transceiver
										//		0 - Use internal voltage regulator (*)
										//		1 - Use external voltage regulator, internal voltage regulator is disabled
										//	6 - AVDD_OK			- indicates if the internal 1.8V regulated supply voltage AVDD
										//						has settled. The bit is set to logic high, if AVREG_EXT = 1.
										//		0 - Analog voltage regulator disabled or supply voltage not stable (*)
										//		1 - Analog supply voltage has settled
										//	5:4 - Reserved
										//	3 - DVREG_EXT		- defines whether the internal digital voltage regulator or an
										//						external regulator is used to supply the digital low voltage 
										//						building blocks of the radio transceiver
										//		0 - Use internal voltage regulator (*)
										//		1 - Use external voltage regulator, internal voltage regulator is disabled
										//	2 - DVDD_OK			- indicates if the internal 1.8V regulated supply voltage DVDD
										//						has settled. The bit is set to logic high, if DVREG_EXT = 1.
										//		0 - Digital voltage regulator disabled or supply voltage not stable (*)
										//		1 - Digital supply voltage has settled
										//	1:0 - Reserved
#define ZIGBEE_REG_BATMON		0x11	// Configures the battery monitor
										//	7:6 - Reserved
										//	5 - BATMON_OK		- indicates the level of the external supply voltage with 
										//						respect to the programmed threshold BATMON_VTH
										//		0 - VDD < BATMON_VTH (*)
										//		1 - VDD > BATMON_VTH
										//	4 - BATMON_HR		- selects the range and resolution of the battery monitor
										//		0 - Enables the low range, see BATMON_VTH (*)
										//		1 - Enables the high range, see BATMON_VTH
										//	3:0 - BATMON_VTH	- The threshold value for the battery monitor
										//				BATMON_HR = 1	" = 2
										//		0x0		2.550			1.70
										//		0x1		2.625			1.75
										//		0x2		2.700			1.80 (*)
										//		0x3		2.775			1.85
										//		0x4		2.850			1.90
										//		0x5		2.925			1.95
										//		0x6		3.000			2.00
										//		0x7		3.075			2.05
										//		0x8		3.150			2.10
										//		0x9		3.225			2.15
										//		0xA		3.300			2.20
										//		0xB		3.375			2.25
										//		0xC		3.450			2.30
										//		0xD		3.525			2.35
										//		0xE		3.600			2.40
										//		0xF		3.675			2.45
#define ZIGBEE_REG_XOSC_CTRL	0x12    // Controls the operation of the crystal oscillator
										//	7:4 - XTAL_MODE		- sets the operating mode of the crystal oscillator
										//		0x0 - Crystal oscillator disabled, no clock signal is fed to the internal circuitry
										//		0x4 - Internal crystal oscillator disabled, use external reference frequency
										//		0xF - Internal crystal oscillator enabled (*)
										//	3:0 - XTAL_TRIM		- control two internal capacitance arrays connected to pins XTAL1 and XTAL2
										//		0x0 - 0.0 pF, trimming capacitors disconnected (*)
										//		0x1 - 0.3 pF, trimming capacitor switched on
										//		...
										//		0xF - 4.5 pF, trimming capacitor switched on
#define ZIGBEE_REG_PLL_CF		0x1A	// Controls the operation of the center frequency calibration loop
										//	7 - PLL_CF_START	- initiates the center frequency calibration, 80us
										//	6:0 - Reserved
#define ZIGBEE_REG_PLL_DCU		0x1B	// Controls the operation of the delay cell calibration loop
										//	7 - PLL_DCU_START	- initiates the delay cell calibration, 6us
										//	6:0 - Reserved
#define ZIGBEE_REG_PART_NUM		0x1C	// Contains the radio transceiver part number.
#define ZIGBEE_REG_VER_NUM		0x1D	// Contains the radio transceiver version number.
#define ZIGBEE_REG_MAN_ID_0		0x1E	// Contains bits [7:0] of the 32 bit JEDEC manufacturer ID
#define ZIGBEE_REG_MAN_ID_1		0x1F	// Contains bits [15:8] of "
#define ZIGBEE_REG_SHORT_ADDR_0	0x20	// Contains bits [7:0] of the 16 bit short address for address filtering
#define ZIGBEE_REG_SHORT_ADDR_1	0x21	// Contains bits [15:8] of "
#define ZIGBEE_REG_PAN_ID_0		0x22	// Contains bits [7:0] of the 16 bit PAN ID for address filtering
#define ZIGBEE_REG_PAN_ID_1		0x23	// Contains bits [15:8] of "
#define ZIGBEE_REG_IEEE_ADDR_0	0x24	// Contains bits [7:0] of the 64 bit IEEE address for address filtering
#define ZIGBEE_REG_IEEE_ADDR_1	0x25	// Contains bits [15:8] of "
#define ZIGBEE_REG_IEEE_ADDR_2	0x26	// Contains bits [23:16] of "
#define ZIGBEE_REG_IEEE_ADDR_3	0x27	// Contains bits [31:24] of "
#define ZIGBEE_REG_IEEE_ADDR_4	0x28	// Contains bits [39:32] of "
#define ZIGBEE_REG_IEEE_ADDR_5	0x29	// Contains bits [47:40] of "
#define ZIGBEE_REG_IEEE_ADDR_6	0x2A	// Contains bits [55:48] of "
#define ZIGBEE_REG_IEEE_ADDR_7	0x2B	// Contains bits [63:56] of "
#define ZIGBEE_REG_XAH_CTRL		0x2C	// Controls the TX_ARET transaction in the Extended Mode
										//	7:4 - MAX_FRAME_RETRIES	- specifies the max # of frame retransmission in TX_ARET transaction
										//	3:1 - MAX_CSMA_RETRIES	- specifies the max # of retries in TX_ARET transaction to repeat 
										//							the random back-off/CCA procedure before the transaction gets cancelled
										//	0 - Reserved
#define ZIGBEE_REG_CSMA_SEED_0	0x2D	// Contains a fraction of the CSMA_SEED value for the CSMA-CA algorithm.
										//	7:0 - CSMA_SEED_0	- contain the lower 8 bits of the CSMA_SEED
#define ZIGBEE_REG_CSMA_SEED_1	0x2E	// Contains a fraction of the CSMA_SEED value, the backoff exponent for 
										// the CSMA-CA algorithm and configuration bits for address filtering in RX_AACK operation
										//	7:6 - MIN_BE		- defines the minimal back-off exponent used in the CSMA-CA algorithm 
										//						to generate a pseudo random number for back-off the CCA
										//	5 - AACK_SET_PD		- defines the content of the frame pending subfield for acknowledgement
										//						frames in RX_AACK mode
										//	4 - Reserved
										//	3 - I_AM_COORD		- to be set if the node is a PAN coordinator
										//						used for address filtering in RX_AACK
										//	2:0 - CSMA_SEED_1	- contain the upper 3 bits of the CSMA_SEED

/**
 * ZigBee Transceiver States (from ZIGBEE_TRX_STATUS[4:0]'s TRX_STATUS subregister)
 */
#define ZIGBEE_TRX_STATUS_MASK				0b00011111
#define ZIGBEE_TRX_STATUS_P_ON 				0x00	// Power-on after VDD (XOSC=On, Pull-up=On)
													//	Write TRX_OFF 		=> TRX_OFF (880us)
#define ZIGBEE_TRX_STATUS_TRX_OFF			0x08	// Clock state (XOSC=On, Pull-up=Off)
													//	Pull SLP_TR high 	=> SLEEP (35us at 1MHz)
													//	Write PLL_ON 		=> PLL_ON (180us)
													//	Write RX_ON 		=> RX_ON (180us) 
#define ZIGBEE_TRX_STATUS_SLEEP 			0x0F	// Sleep State (XOSC=Off, Pull-up=Off)
													//	Pull SLP_TR low 	=> TRX_OFF (880us) 
#define ZIGBEE_TRX_STATUS_PLL_ON 			0x09	// PLL State
													//	Write TRX_OFF 		=> TRX_OFF (1us)
													//	Write RX_ON			=> RX_ON (1us)
													//	Write TX_START 		=> BUSY_TX (16us)
													//	Pull SLP_TR high 	=> "
#define ZIGBEE_TRX_STATUS_BUSY_TX			0x02	// Transmit State
													//	Frame End 			=> PLL_ON (32us) 
#define ZIGBEE_TRX_STATUS_RX_ON 			0x06	// Rx Listen State
													//	Write TRX_OFF		=> TRX_OFF (1us)
													//	Write PLL_ON		=> PLL_ON (1us)
													//	Pull SLP_TR high	=> RX_ON_NOCLK
													//	SFD Detected		=> BUSY_RX
#define ZIGBEE_TRX_STATUS_BUSY_RX 			0x01	// Receive State
													//	Frame End			=> RX_ON 
#define ZIGBEE_TRX_STATUS_RX_ON_NOCLK		0x1C	// Rx Listen State (CLKM=Off)
													//	Pull SLP_TR low		=> RX_ON
													//	SFD Detected		=> BUSY_RX
#define ZIGBEE_TRX_STATUS_BUSY_RX_AACK		0x11
#define ZIGBEE_TRX_STATUS_BUSY_TX_ARET		0x12
#define ZIGBEE_TRX_STATUS_RX_AACK_ON		0x16
#define ZIGBEE_TRX_STATUS_TX_ARET_ON		0x19
#define ZIGBEE_TRX_STATUS_RX_AACK_NOCLK			0x1D
#define ZIGBEE_TRX_STATUS_BUSY_RX_AACK_NOCLK	0x1E
#define ZIGBEE_TRX_STATUS_TRANSITION			0x1F

/* Radio state transition commands (from ZIGBEE_TRX_STATE[4:0]'s TRX_CMD subregister) */
#define ZIGBEE_TRX_CMD_MASK				0b00011111
#define ZIGBEE_TRX_CMD_NOP				0x00
#define ZIGBEE_TRX_CMD_TX_START			0x02
#define ZIGBEE_TRX_CMD_FORCE_TRX_OFF	0x03
#define ZIGBEE_TRX_CMD_RX_ON			0x06
#define ZIGBEE_TRX_CMD_TRX_OFF			0x08
#define ZIGBEE_TRX_CMD_PLL_ON			0x09
#define ZIGBEE_TRX_CMD_RX_AACK_ON		0x16
#define ZIGBEE_TRX_CMD_TX_ARET_ON		0x19

/* ZigBee Transceiver States (from TRAC_STATUS subregister) */
#define ZIGBEE_TRAC_STATUS_SUCCESS					0
#define ZIGBEE_TRAC_STATUS_SUCCESS_DATA_PENDING		1
#define ZIGBEE_TRAC_STATUS_CHANNEL_ACCESS_FAILURE	3
#define ZIGBEE_TRAC_STATUS_NO_ACK					5
#define ZIGBEE_TRAC_STATUS_INVALID					7

/* Clock rate of CLKM (from ZIGBEE_TRX_CTRL_0[2:0]'s CLKM_CTRL subregister) */
#define ZIGBEE_CLKM_MASK	0b00001111
#define ZIGBEE_CLKM_SHA_SEL	3	// ZIGBEE_TRX_CTRL_0[3]'s CLKM_SHA subregister
#define ZIGBEE_CLKM_SHA_IMM	0	// Immediate
#define ZIGBEE_CLKM_SHA_SLP	1	// After sleep
#define ZIGBEE_CLKM_OFF		0
#define ZIGBEE_CLKM_1MHZ	1	// Default
#define ZIGBEE_CLKM_2MHZ	2
#define ZIGBEE_CLKM_4MHZ	3
#define ZIGBEE_CLKM_8MHZ	4
#define ZIGBEE_CLKM_16MHZ	5

/* RXTX Channel (from ZIGBEE_PHY_CC_CCA[4:0]'s CHANNEL subregister) */
#define ZIGBEE_CHANNEL_MASK	0b00011111
#define ZIGBEE_CHANNEL_11	0x0B	// 2405 MHz (Default)
#define ZIGBEE_CHANNEL_12	0x0C	// 2410 MHz
#define ZIGBEE_CHANNEL_13	0x0D	// 2415 MHz
#define ZIGBEE_CHANNEL_14	0x0E	// 2420 MHz
#define ZIGBEE_CHANNEL_15	0x0F	// 2425 MHz
#define ZIGBEE_CHANNEL_16	0x10	// 2430 MHz
#define ZIGBEE_CHANNEL_17	0x11	// 2435 MHz
#define ZIGBEE_CHANNEL_18	0x12	// 2440 MHz
#define ZIGBEE_CHANNEL_19	0x13	// 2445 MHz
#define ZIGBEE_CHANNEL_20	0x14	// 2450 Mhz
#define ZIGBEE_CHANNEL_21	0x15	// 2455 Mhz
#define ZIGBEE_CHANNEL_22	0x16	// 2460 Mhz
#define ZIGBEE_CHANNEL_23	0x17	// 2465 Mhz
#define ZIGBEE_CHANNEL_24	0x18	// 2470 Mhz
#define ZIGBEE_CHANNEL_25	0x19	// 2475 Mhz
#define ZIGBEE_CHANNEL_26	0x1A	// 2480 Mhz

/* Zigbee IRQ Masks from the ZIGBEE_IRQ_MASK & ZIGBEE_IRQ_STATUS */
#define ZIGBEE_IRQ_MASK_ALL			0b11001111
#define ZIGBEE_IRQ_MASK_NONE		0b00000000
#define ZIGBEE_IRQ_MASK_BAT_LOW		7			// 0b10000000
#define	ZIGBEE_IRQ_MASK_TRX_UR		6			// 0b01000000
#define	ZIGBEE_IRQ_MASK_TRX_END		3			// 0b00001000
#define ZIGBEE_IRQ_MASK_RX_START	2			// 0b00000100
#define ZIGBEE_IRQ_MASK_PLL_UNLOCK	1			// 0b00000010
#define ZIGBEE_IRQ_MASK_PLL_LOCK	0			// 0b00000001

/* Zigbee transmission */
#define ZIGBEE_RXTX_SUCCESS			1
#define ZIGBEE_RXTX_FAILURE			0

/* Zigbee Hardware ID data */
uint8_t zigbee_hw_part_num, zigbee_hw_ver_num, zigbee_hw_man_id_0, zigbee_hw_man_id_1;

/* Global Variables */
uint8_t zigbee_status;
uint8_t zigbee_clkm;
uint8_t zigbee_channel;

uint8_t zigbee_irq_pll_lock_flag;	// Indicates locking and unlocking events of the PLL
uint8_t zigbee_irq_trx_end_flag;	// Signals the end of any transmit or receive operation
uint8_t zigbee_irq_rx_start_flag;	// Signals that the radio transceiver has received a valid SFD
uint8_t zigbee_irq_trx_ur_flag;		// Signals a buffer underrrun
uint8_t zigbee_irq_bat_low_flag;	// Signals when the supply voltage drops below the adjusted threshold

uint8_t zigbee_crc_enable_flag;

uint8_t zigbee_tx_data[ZIGBEE_FRAME_BUFFER_LENGTH];
uint8_t zigbee_tx_data_length = 5;
	
uint8_t zigbee_rx_data_length;
uint8_t zigbee_rx_data[ZIGBEE_FRAME_BUFFER_LENGTH];
uint8_t zigbee_rx_data_lqi;		// Link Quality Indication
									// A measure for the quality of a received frame
									// Ranes from 0 (low quality) to 255 (high quality)
									
/* Prototype Functions */
uint8_t zigbee_get_trx_status(void);
void zigbee_set_state_trx_cmd(uint8_t state_trx_cmd);

void zigbee_enable_basic_tx(void);
void zigbee_enable_trx_off(void);
void zigbee_man_basic_tx(uint8_t *tx_data, uint8_t tx_length);
void zigbee_basic_auto_tx(uint8_t *data, uint8_t length);
void zigbee_tx_start_pin(void);
void zigbee_tx_start_reg(void);

void zigbee_enable_basic_rx(void);
void zigbee_enable_basic_rx_sleep(void);
void zigbee_man_basic_rx(uint8_t *rx_data, uint8_t *rx_data_length, uint8_t *rx_data_lqi);
void zigbee_auto_basic_rx(uint8_t *rx_data, uint8_t *rx_data_length, uint8_t *rx_data_lqi);

void zigbee_set_cca_mode(uint8_t mode);
uint8_t zigbee_get_cca_mode(void);
void zigbee_set_cca_ed_thres(uint8_t ed_thres_value);
uint8_t zigbee_get_cca_ed_thres(void);
uint8_t zigbee_man_cca(void);
void zigbee_set_crc(uint8_t mode);
uint8_t zigbee_get_crc_valid(void);
uint8_t zigbee_get_rssi(void);
uint8_t zigbee_get_ed(void);
uint8_t zigbee_get_frame_ed(void);

void zigbee_set_clkm(uint8_t clkm, uint8_t mode);
void zigbee_set_channel(uint8_t chan);
uint8_t zigbee_get_channel(void);

void zigbee_set_irq_mask_bit(uint8_t irq_bit, uint8_t en);
void zigbee_set_irq_mask(uint8_t reg_value);
void zigbee_update_irq_status(void);

void zigbee_init(void);
void zigbee_hw_rst(void);
void zigbee_sw_rst(void);


/**
 * Read the radio transceiver status from TRX_STATUS
 */
uint8_t zigbee_get_trx_status(void) {
	uint8_t trx_status = spi_read_register(ZIGBEE_REG_TRX_STATUS) & ZIGBEE_TRX_STATUS_MASK;
	return trx_status;
}

/**
 * The radio transceiver state is controlled by two signal pins (SLP_TR, RST ) 
 * and the register 0x02 (TRX_STATE). A successful state change shall be 
 * confirmed by reading the radio transceiver status from register 0x01 (TRX_STATUS).
 */
void zigbee_set_state_trx_cmd(uint8_t state_trx_cmd) {
	uint8_t trx_status = zigbee_get_trx_status();
	
	/**
	 * If TRX_STATUS = 0x1F (STATE_TRANSITION_IN_PROGRESS) the AT86RF230 is
	 * on a state transition. Do not try to initiate a further state change while 
	 * the radio transceiver is in STATE_TRANSITION_IN_PROGRESS.
	 */
	while (trx_status == ZIGBEE_TRX_STATUS_TRANSITION) {
		trx_status = zigbee_get_trx_status();
	}
	
	uint8_t trx_state_value = spi_read_register(ZIGBEE_REG_TRX_STATE);
	trx_state_value = (trx_state_value & ~ZIGBEE_TRX_CMD_MASK) | state_trx_cmd; // TRX_STATE = TRAC_STATUS | TRX_CMD
	spi_write_register(ZIGBEE_REG_TRX_STATE, trx_state_value);
	
	/* Transition Specific Instructions */
	switch(state_trx_cmd) {
		case ZIGBEE_TRX_CMD_NOP:
			break;
		case ZIGBEE_TRX_CMD_TX_START:
			break;
		case ZIGBEE_TRX_CMD_FORCE_TRX_OFF:
			break;
		case ZIGBEE_TRX_CMD_RX_ON:
			switch(trx_status) {
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					while (!(EIFR & _BV(INTF0)));		// 180us
					EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);
					zigbee_update_irq_status();			// ZIGBEE_IRQ_MASK_PLL_LOCK
					break;
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
					while (!(EIFR & _BV(INTF0)));
					EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);
					zigbee_update_irq_status();			// ZIGBEE_IRQ_MASK_TRX_END
					_delay_us(1);
					break;
			}
			/* ZIGBEE_TRX_STATUS_RX_ON */
			break;
		case ZIGBEE_TRX_CMD_TRX_OFF:
			switch(trx_status) {
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
					while (!(EIFR & _BV(INTF0)));
					EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);
					zigbee_update_irq_status();			// ZIGBEE_IRQ_MASK_TRX_END
					break;
				default:							// Any Active states (except P_ON)
					_delay_us(1);
			}
			/* ZIGBEE_TRX_STATUS_TRX_OFF */
			break;
		case ZIGBEE_TRX_CMD_PLL_ON:
			switch(trx_status) {
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					while (!(EIFR & _BV(INTF0)));		// 180us
					EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);
					zigbee_update_irq_status();			// ZIGBEE_IRQ_MASK_PLL_LOCK
					break;
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
					while (!(EIFR & _BV(INTF0)));
					EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);
					zigbee_update_irq_status();			// ZIGBEE_IRQ_MASK_TRX_END
					_delay_us(1);
					break;
			}
			/* ZIGBEE_TRX_STATUS_PLL_ON */
			break;
		case ZIGBEE_TRX_CMD_RX_AACK_ON:
			break;
		case ZIGBEE_TRX_CMD_TX_ARET_ON:
			break;
	}
}

/**
 * Changes the TRX to the PLL_ON state for transmission
 */
void zigbee_enable_basic_tx(void) {
	zigbee_status = zigbee_get_trx_status();	
	switch(zigbee_status) {
		case ZIGBEE_TRX_STATUS_TRX_OFF:
		case ZIGBEE_TRX_STATUS_RX_ON:
			zigbee_irq_trx_end_flag = TRUE;
			zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_PLL_ON);
			break;
		case ZIGBEE_TRX_STATUS_PLL_ON:
			break;
		default:
			break;
	}
	while(zigbee_status != ZIGBEE_TRX_STATUS_PLL_ON) {
		zigbee_status = zigbee_get_trx_status();
	}
}

/**
 * Changes the TRX to the TRX_OFF state
 */
void zigbee_enable_trx_off(void) {
	zigbee_status = zigbee_get_trx_status();	
	switch(zigbee_status) {
		case ZIGBEE_TRX_STATUS_PLL_ON:
		case ZIGBEE_TRX_STATUS_RX_ON:
			zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
			break;
		case ZIGBEE_TRX_STATUS_TRX_OFF:
			break;
		default:
			zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_FORCE_TRX_OFF);
			break;
	}
	while(zigbee_status != ZIGBEE_TRX_STATUS_TRX_OFF) {
		zigbee_status = zigbee_get_trx_status();
	}
}

/**
 * Performs a manual basic transmission
 * 
 * The transceiver must be in the PLL_ON state beforehand
 */
void zigbee_man_basic_tx(uint8_t *tx_data, uint8_t tx_length) {
	zigbee_status = zigbee_get_trx_status();	
	if (zigbee_status == ZIGBEE_TRX_STATUS_PLL_ON) {
		#if PROTOBOARD_LED_DEBUG
			cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
		#endif
		spi_write_frame_buffer(tx_length, tx_data);	// Write the entire frame before the transmission 
													// is initiated to prevent a buffer underrun interrupt
		zigbee_tx_start_pin();		// Start transmission
		
		while(zigbee_status != ZIGBEE_TRX_STATUS_BUSY_TX) {
			zigbee_status = zigbee_get_trx_status();
		}
		
		while (!(EIFR & _BV(INTF0)));
		EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);	// Clear the flag to make sure next interrupt is not missed
		zigbee_update_irq_status(); 	// ZIGBEE_IRQ_MASK_TRX_END
	
		#if PROTOBOARD_LED_DEBUG
			sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
		#endif
	}
}

/** 
 * Performs am automatic basic transmission
 *
 * 1) For transmission of frames in the basic mode, the radio transceiver needs to be in the state PLL_ON
 * 2) Frame transmit start commands: There are two methods to initiate a frame transmission, 
 * either by writing CMD_TX_START to sub register SR_TRX_CMD (register start) or with a rising 
 * edge on the pin TRX_PIN_SLP_TR (pin start)
 * 3) 16 us after one of the two frame start commands has been issued, 
 * the radio transceiver sends out the 802.15.4 synchronisation header (SHR), 
 * which consists of 4 octets preamble and 1 octet start-of-frame delimitter (SFD). 
 * The transmission of the whole SHR takes 160 us. After transmitting SHR, PHR and PSDU, 
 * the TRX_IRQ_TRX_END interrupt is generated.
 * 4) An TRX_IRQ_TRX_UR interrupt occurs when the frame was incorrectly transmitted
 */
void zigbee_auto_basic_tx(uint8_t *tx_data, uint8_t tx_data_length) {
	zigbee_status = zigbee_get_trx_status();	
	switch(zigbee_status) {
		case ZIGBEE_TRX_STATUS_TRX_OFF:
		case ZIGBEE_TRX_STATUS_RX_ON:
			zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_PLL_ON);
			break;
		case ZIGBEE_TRX_STATUS_PLL_ON:
			break;
	}
	while(zigbee_status != ZIGBEE_TRX_STATUS_PLL_ON) {
		zigbee_status = zigbee_get_trx_status();
	}
	
	#if PROTOBOARD_LED_DEBUG
		cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
			
	spi_write_frame_buffer(tx_data_length, tx_data);	// Write the entire frame before the transmission 
												// is initiated to prevent a buffer underrun interrupt			
	//zigbee_tx_start_pin();		// Start transmission
	zigbee_tx_start_reg();

	while(zigbee_status != ZIGBEE_TRX_STATUS_BUSY_TX) {
		zigbee_status = zigbee_get_trx_status();
	}
	
	while (!(EIFR & _BV(INTF0)));
	EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);	// Clear the flag to make sure next interrupt is not missed
	zigbee_update_irq_status(); 	// ZIGBEE_IRQ_MASK_TRX_END
	
	/* ZIGBEE_TRX_STATUS_PLL_ON */
	
	#if PROTOBOARD_LED_DEBUG
		sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
}

void zigbee_tx_start_pin(void) {
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(1);						// Actual delay should be 65 ns
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
}

void zigbee_tx_start_reg(void) {
	zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_TX_START);
}

void zigbee_enable_basic_rx(void) {
	zigbee_status = zigbee_get_trx_status();	
	switch(zigbee_status) {
		case ZIGBEE_TRX_STATUS_TRX_OFF:
		case ZIGBEE_TRX_STATUS_PLL_ON:
			zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_RX_ON);
			break;
		case ZIGBEE_TRX_STATUS_RX_ON:
			break;
		default:
			break;
	}
	while(zigbee_status != ZIGBEE_TRX_STATUS_RX_ON) {
		zigbee_status = zigbee_get_trx_status();
	}
}

void zigbee_enable_basic_rx_sleep(void) {
	zigbee_enable_basic_rx();
	//cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
}

void zigbee_man_basic_rx(uint8_t *rx_data, uint8_t *rx_data_length, uint8_t *rx_data_lqi) {
	spi_read_frame_buffer(rx_data_length, rx_data, rx_data_lqi);
}

void zigbee_auto_basic_rx(uint8_t *rx_data, uint8_t *rx_data_length, uint8_t *rx_data_lqi) {
	zigbee_status = zigbee_get_trx_status();	
	switch(zigbee_status) {
		case ZIGBEE_TRX_STATUS_TRX_OFF:
		case ZIGBEE_TRX_STATUS_PLL_ON:
			zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_RX_ON);
			break;
		case ZIGBEE_TRX_STATUS_RX_ON:
			break;
		default:
			break;
	}
	while(zigbee_status != ZIGBEE_TRX_STATUS_RX_ON) {
		zigbee_status = zigbee_get_trx_status();
	}
	
	while (!(EIFR & _BV(INTF0)));
	EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);	// Clear the flag to make sure next interrupt is not missed
	zigbee_update_irq_status(); 	// ZIGBEE_IRQ_MASK_RX_START
	
	/* ZIGBEE_TRX_STATUS_BUSY_RX */
	
	#if PROTOBOARD_LED_DEBUG
		cbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	while (!(EIFR & _BV(INTF0)));
	EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);	// Clear the flag to make sure next interrupt is not missed
	zigbee_update_irq_status(); 	// ZIGBEE_IRQ_MASK_TRX_END
	
	/* ZIGBEE_TRX_STATUS_RX_ON */
	
	#if PROTOBOARD_LED_DEBUG
		sbi(PROTOBOARD_LED_PORT, PROTOBOARD_LED_1);
	#endif
	
	spi_read_frame_buffer(rx_data_length, rx_data, rx_data_lqi);
}



/**
 * Set CCA Mode
 *
 * Mode		Description
 * 1		Energy Above Threshold: A busy channel shall be reported upon detecting any energy above the threshold.
 * 2		Carrier Sense Only: A busy channel is reported if signals with the same 
 *			modulation and spreading characteristics of IEEE 802.15.4 are detected. 
 *			The energy of these signals is not checked.
 * 3		Carrier Sense with Energy Above Threshold: A busy channel is reported if 
 *			signals with the same modulation and spreading characteristics of IEEE 802.15.4 
 *			and energy above a threshold are detected.
 */
void zigbee_set_cca_mode(uint8_t mode) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = (reg_value & ~0b01100000) | (mode << 5);	// CCA Mode is defined in [6:5] of ZIGBEE_REG_PHY_CC_CCA
	spi_write_register(ZIGBEE_REG_PHY_CC_CCA, reg_value);
}

/**
 * Read CCA Mode
 */
uint8_t zigbee_get_cca_mode(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = (reg_value & 0b01100000) >> 5;
	return reg_value;
}

/**
 * Set the CCA ED threshold value
 */
void zigbee_set_cca_ed_thres(uint8_t ed_thres_value) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_CCA_THRES);
	reg_value = (reg_value & ~0b00001111) | ed_thres_value;
	spi_write_register(ZIGBEE_REG_CCA_THRES, reg_value);
}

/**
 * Read the CCA ED threshold value
 */
uint8_t zigbee_get_cca_ed_thres(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_CCA_THRES);
	reg_value = (reg_value & 0b00001111);
	return reg_value;
}

/**
 * Perform a manual CCA (Clear Channel Assessment)
 *
 * Using the Basic Operating Mode, a CCA request can be initiated manually by setting
 * CCA_REQUEST = 1 in register 0x08 (PHY_CC_CCA), if the AT86RF230 is in any RX
 * state. The CCA computation is done over eight symbol periodes and the result is
 * accessible 140 μs after the request.
 * Note, it is not recommended to initiate manually a CCA measurement when using the
 * Extended Operating Mode.
 * 
 * CCA measurements can be done from state PLL_ON. 
 * The measurement is started by switching to state RX_ON and writing the value 1 to the sub register SR_CCA_REQUEST. 
 * The measurement is finished after 140us and the radio transceiver is switched back to state PLL_ON.
 *
 * Return value:	0 = medium is busy
 *					1 - medium is free
 */
uint8_t zigbee_man_cca(void) {
	zigbee_status = zigbee_get_trx_status();	
	switch(zigbee_status) {
		case ZIGBEE_TRX_STATUS_TRX_OFF:
		case ZIGBEE_TRX_STATUS_PLL_ON:
			zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_RX_ON);
			break;
		case ZIGBEE_TRX_STATUS_RX_ON:
			break;
		default:
			break;
	}
	while(zigbee_status != ZIGBEE_TRX_STATUS_RX_ON) {
		zigbee_status = zigbee_get_trx_status();
	}
	
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = reg_value | _BV(7);
	spi_write_register(ZIGBEE_REG_PHY_CC_CCA, reg_value);
	_delay_us(140);
	reg_value = spi_read_register(ZIGBEE_REG_TRX_STATUS);
	while (!(reg_value & _BV(7))) {
		reg_value = spi_read_register(ZIGBEE_REG_TRX_STATUS);
	}
	reg_value = reg_value & _BV(6);
	#if UART_TX_ZIGBEE_CCA_VALUE
		uart0_send_byte(UART_TX_ZIGBEE_CCA_VALUE);
		uart0_send_byte(reg_value >> 6);
	#endif
	/* ZIGBEE_TRX_STATUS_RX_ON */
	return reg_value;
}

/**
 * Set the CRC mode
 *
 * When using the automatic CRC mechanism the data length must be increased 
 * by two (16 bit CRC) bytes and two bytes must be added to the frame payload. 
 * Typically two 0x00 bytes will be appended at the end of the frame during 
 * download to the radio transceiver’s frame buffer.
 *
 * The last two bytes need not to be downloaded, because the FCS is 
 * calculated internally by the radio transceiver
 */
void zigbee_set_crc(uint8_t mode) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_TX_PWR);
	reg_value = (reg_value & ~_BV(7)) | (mode << 7);
	spi_write_register(ZIGBEE_REG_PHY_TX_PWR, reg_value);
}

/**
 * Read the validity of the CRC check
 */
uint8_t zigbee_crc_valid(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_RSSI);
	return reg_value & _BV(7);
}

/**
 * Read the RSSI value
 *
 * The RSSI is a 5-bit value indicating the receive power in the selected channel, in steps of 3 dB.
 * An RSSI value of 0 indicates an RF input power of < -91 dBm. For an RSSI value in the 
 * range of 1 to 28, the RF input power can be calculated as follows:
 * PRF = RSSI_BASE_VAL + 3•(RSSI - 1)
 *
 * Using the Basic Operating Mode, the RSSI value is valid at any RX state, and is
 * updated every 2 μs.
 * The RSSI value for the currently received frame can be read from the sub register SR_RSSI 
 * right after the TRX_IRQ_RX_START interrupt and before the corresponding TRX_IRQ_TRX_END interrupt.
 * Note, it is not recommended to read the RSSI value when using the Extended Operating Mode. 
 * The automatically generated ED value should be used alternatively.
 */
uint8_t zigbee_get_rssi(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_RSSI);
	return reg_value & 0b00011111;
}

/**
 * Perform a manual ED value request
 *
 * The receiver Energy Detection measurement is used by a network layer as part of a
 * channel selection algorithm. It is an estimation of the received signal power within the
 * bandwidth of an IEEE 802.15.4-2003 channel. No attempt is made to identify or decode
 * signals on the channel. The ED value is calculated by averaging RSSI values over eight
 * symbols (128 μs).
 *
 * For manually initiated ED measurement the radio transceiver needs to be in one of the
 * states RX_ON or BUSY_RX.
 */
uint8_t zigbee_get_ed(void) {
	spi_write_register(ZIGBEE_REG_PHY_ED_LEVEL, 0);
    _delay_us(140);
    uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_ED_LEVEL);
	return reg_value;
}

/**
 * Read the ED measurement value of the last frame
 *
 * An ED measurement is automatically started after detection of an SFD. 
 * The register RG_PHY_ED_LEVEL can be read in a time period which starts 140 us 
 * after the TRX_IRQ_RX_START interrupt (time to complete the measurement) 
 * and ends 96 us after the TRX_IRQ_TRX_END interrupt (minimum time which is needed 
 * to detect the next SFD).
 *
 * An automated ED measurement is started, if a SFD field
 * is detected. A valid SFD detection is signalized by an RX_START interrupt in Basic
 * Operating Mode.
 */
uint8_t zigbee_get_frame_ed(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_ED_LEVEL);
	return reg_value;
}

void zigbee_set_clkm(uint8_t clkm, uint8_t mode) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_TRX_CTRL_0);
	reg_value = (reg_value & ~ZIGBEE_CLKM_MASK) | (mode << ZIGBEE_CLKM_SHA_SEL) | clkm;
	spi_write_register(ZIGBEE_REG_TRX_CTRL_0, reg_value);
}

/**
 * The channel is set by writing the channel number to the sub register 
 * under ZIGBEE_PHY_CC_CCA.
 * The channel number which must be in the range 11<=channel<=26.
 *
 * Depending on the state of the radio transceiver, the current channel number 
 * and the selected channel number, the sequence may cause a PLL Lock interrupt
 */
void zigbee_set_channel(uint8_t chan) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	reg_value = (reg_value & ~ZIGBEE_CHANNEL_MASK) | chan;
	spi_write_register(ZIGBEE_REG_PHY_CC_CCA, reg_value);
	
	while (!(EIFR & _BV(INTF0)));		// 150us
	EIFR = (EIFR & INT0_FLAG_MASK) | _BV(INTF0);
	zigbee_update_irq_status();			// ZIGBEE_IRQ_MASK_PLL_LOCK
}

uint8_t zigbee_get_channel(void) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_PHY_CC_CCA);
	return reg_value & ZIGBEE_CHANNEL_MASK;
}

void zigbee_set_irq_mask_bit(uint8_t irq_bit, uint8_t en) {
	uint8_t reg_value = spi_read_register(ZIGBEE_REG_IRQ_MASK);
	reg_value = (reg_value & ~_BV(irq_bit)) | (en << irq_bit);
	spi_write_register(ZIGBEE_REG_IRQ_MASK, reg_value);
}

void zigbee_set_irq_mask(uint8_t reg_value) {
	spi_write_register(ZIGBEE_REG_IRQ_MASK, reg_value);
}

void zigbee_update_irq_status(void) {
	uint8_t irq_state = spi_read_register(ZIGBEE_REG_IRQ_STATUS);

	#if UART0_ZIGBEE_IRQ_STATUS_DEBUG
		uart0_send_byte(UART_TX_ZIGBEE_IRQ_STATUS);
		uart0_send_byte(irq_state);
	#endif

	if (irq_state & _BV(ZIGBEE_IRQ_MASK_BAT_LOW)) {
		zigbee_irq_bat_low_flag = TRUE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_TRX_UR)) {
		zigbee_irq_trx_ur_flag = TRUE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_TRX_END)) {
		zigbee_irq_trx_end_flag = TRUE;
		//#if ZIGBEE_RX
		//	uart0_send_byte(zigbee_get_crc_valid());
		//#endif
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_RX_START)) {
		zigbee_irq_rx_start_flag = TRUE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_PLL_UNLOCK)) {
		zigbee_irq_pll_lock_flag = FALSE;
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_PLL_LOCK)) {
		zigbee_irq_pll_lock_flag = TRUE;
		
	}
}

/**
 * Brings the radio transceiver into the TRX_OFF state 
 * after a power-on event.
 */
void zigbee_init(void) {
	/* Transceiver Initialization */
	_delay_us(510);		// Delay 510us for transition to P_ON state
	//_delay_us(510/CPU_FREQ_RATIO);
	
	/* Transceiver Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	cbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST low
	_delay_us(6);		// Delay 6us for hardware reset
	//_delay_us(6/CPU_FREQ_RATIO);
	sbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST high
    
	/* Read Hardware Identification Data */
	zigbee_hw_part_num = spi_read_register(ZIGBEE_REG_PART_NUM);
	zigbee_hw_ver_num = spi_read_register(ZIGBEE_REG_VER_NUM);
	zigbee_hw_man_id_0 = spi_read_register(ZIGBEE_REG_MAN_ID_0);
	zigbee_hw_man_id_1 = spi_read_register(ZIGBEE_REG_MAN_ID_1);
	
	/* Disable IRQ Mask and dummy read IRQ Status to avoid unhandled interrupts */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_NONE);
	spi_read_register(ZIGBEE_REG_IRQ_STATUS);
	
	/* Send TRX_OFF Command */
	zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
	_delay_us(510);		// Delay 510us for transition from P_ON to TRX_OFF
	//_delay_us(510/CPU_FREQ_RATIO);

	/* Get current state */
	while(zigbee_status != ZIGBEE_TRX_STATUS_TRX_OFF) {
		zigbee_status = zigbee_get_trx_status();
	}
	
	/* Enable IRQ Mask */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_ALL);
}

/**
 * Resets the Zigbee Transceiver from any state except P_ON
 *
 * The programming sequence will typically be used to recover 
 * from fatal errors and bring the radio transceiver to a known state.
 * All registers will have their default values and the state machine 
 * will be in TRX_OFF state.
 */
void zigbee_hw_rst(void) {
	/* Transceiver Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	cbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST low
	_delay_us(6);		// Delay 6us for hardware reset
	sbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST high
	
	/* Disable IRQ Mask and dummy read IRQ Status to avoid unhandled interrupts */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_NONE);
	spi_read_register(ZIGBEE_REG_IRQ_STATUS);
	
	_delay_us(120);
	
	while(zigbee_status != ZIGBEE_TRX_STATUS_TRX_OFF) {
		zigbee_status = zigbee_get_trx_status();
	}
	
	/* Enable IRQ Mask */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_ALL);
}

/**
 * Resets the state machine of the radio transceiver to the TRX_OFF state.
 */
void zigbee_sw_rst(void) {
	/* State Machine Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	_delay_us(6);		// Delay 6us for hardware reset
	
	/* Set to TRX_OFF state */
	zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
	_delay_us(6);		// Delay 6us for hardware reset
	
	while(zigbee_status != ZIGBEE_TRX_STATUS_TRX_OFF) {
		zigbee_status = zigbee_get_trx_status();
	}
}
