/* ZigBee Constants */
#define ZIGBEE_FRAME_BUFFER_LENGTH	127

/* ZigBee MCU Ports */
#define ZIGBEE_DDR			DDRB
#define ZIGBEE_PORT			PORTB
#define ZIGBEE_SLP_TR		PB0
#define ZIGBEE_RST			PB1

/* ZigBee Registers */
#define ZIGBEE_TRX_STATUS	0x01	// Current state of the radio
									//	A read clears bits CCA_DONE and CCA_STATUS
									//	7 - CCA_DONE		- indicates if a CCA request is completed
									//		0 - CCA calculation CCA_DONE in progress (*)
									//		1 - CCA calculation finished
									//	6 - CCA_STATUS		- indicates the result of a CCA request
									//		0 - Channel is busy (*)
									//		1 - Channel is idle
									//	5 - Reserved
									//	4:0 - TRX_STATUS	- current radio transceiver status
									//		(see below)
#define ZIGBEE_TRX_STATE	0x02	// Controls the radio transceiver states via register bits TRX_CMD
									//	7:5 - TRAC_STATUS	- status of the TX_ARET algorithm
									//		(see below)
									//	4:0 - TRX_CMD		- initiates a state transition towards the new state
									//		(see below)
#define ZIGBEE_TRX_CTRL_0	0x03	// Controls the drive current of the digital output pads and the
									// CLKM clock rate
									//	7:6  PAD_IO		- sets the output driver current of digital output MISO and IRQ
									//		0 - 2mA (*)
									//		1 - 4mA
									//		2 - 6mA
									//		3 - 8mA
									//	5:6  PAD_IO_CLKM	- sets the output driver current of pin CLKM
									//		0 - 2mA
									//		1 - 4mA (*)
									//		2 - 6mA
									//		3 - 8mA
									//	3  CLKM_SHA_SEL
									//		0 - CLKM clock rate changes immediately
									//		1 - CLKM clock rate changes after SLEEP cycle (*)
									//	2:0  CLKM_CTRL
									//		(see below)

#define ZIGBEE_PHY_TX_PWR	0x05	//
									//	7 - TX_AUTO_CRC_ON
									//	6:4 - Reserved
									//	3:0 - TX_PWR
#define ZIGBEE_PHY_RSSI		0x06	//
									//	7 - RX_CRC_VALID
									//	6:5 - Reserved
									//	4:0 - RSSI
#define ZIGBEE_PHY_ED_LEVEL	0x07	// ED_LEVEL
#define ZIGBEE_PHY_CC_CCA	0x08	// Contains register bits to initiate and control the CCA measurement 
									// as well as to set the channel center frequency
									//	7  CCA_REQUEST		- initiates manual CCA measurement request
									//	6:5  CCA_MODE
									//		0 - Reserved
									//		1 - Mode 1, Energy above threshold (*)
									//		2 - Mode 2, Carrier sense only
									//		3 - Mode 3, Carrier sense with energy above threshold
									//	4:0  CHANNEL		- define the RX/TX channel
									//		(see below)
#define ZIGBEE_CCA_THRES	0x09	// Contains the threshold level for CCA-ED measurement
									//	7:4  Reserved
									//	3:0  CCA_ED_THRES	- define the threshold value of the CCA-ED measurement
									//						A cannel is indicated as busy when RSSI_BASE_VAL + 2  CCA_ED_THRES [dBm]		
#define ZIGBEE_IRQ_MASK		0x0E	// Used to enable (1) or disable (0) interrupt events
									//	Should read the IRQ_STATUS register first to clear the history
#define ZIGBEE_IRQ_STATUS	0x0F	// Contains the status of the individual interrupts
									//	A read access to this register resets all interrupt bits
#define ZIGBEE_VREG_CTRL	0x10	//
									//	7 - AVREG_EXT
									//	6 - AVDD_OK
									//	5:4 - Reserved
									//	3 - DVREG_EXT
									//	2 - DVDD_OK
									//	1:0 - Reserved
#define ZIGBEE_BATMON		0x11	// 
									//	7:6 - Reserved
									//	5 - BATMON_OK
									//	4 - BATMON_HR
									//	3:0 - BATMON_VTH
#define ZIGBEE_XOSC_CTRL	0x12    // Controls the operation of the crystal oscillator
									//	7:4  XTAL_MODE		- sets the operating mode of the crystal oscillator
									//		0x0 - Crystal oscillator disabled, no clock signal is fed to the internal circuitry
									//		0x4 - Internal crystal oscillator disabled, use external reference frequency
									//		0xF - Internal crystal oscillator enabled (*)
									//	3:0  XTAL_TRIM		- control two internal capacitance arrays connected to pins XTAL1 and XTAL2
									//		0x0 - 0.0 pF, trimming capacitors disconnected (*)
									//		0x1 - 0.3 pF, trimming capacitor switched on
									//		...
									//		0xF - 4.5 pF, trimming capacitor switched on
#define ZIGBEE_PLL_CF		0x1A	// Controls the operation of the center frequency calibration loop
									//	7  PLL_CF_START	- initiates the center frequency calibration, 80us
									//	6:0 - Reserved
#define ZIGBEE_PLL_DCU		0x1B	// Controls the operation of the delay cell calibration loop
									//	7  PLL_DCU_START	- initiates the delay cell calibration, 6us
									//	6:0 - Reserved
#define ZIGBEE_PART_NUM		0x1C	// Contains the radio transceiver part number.
#define ZIGBEE_VER_NUM		0x1D	// Contains the radio transceiver version number.
#define ZIGBEE_MAN_ID_0		0x1E	// Contains bits [7:0] of the 32 bit JEDEC manufacturer ID
#define ZIGBEE_MAN_ID_1		0x1F	// Contains bits [15:8] of "
#define ZIGBEE_SHORT_ADDR_0	0x20	// Contains bits [7:0] of the 16 bit short address for address filtering
#define ZIGBEE_SHORT_ADDR_1	0x21	// Contains bits [15:8] of "
#define ZIGBEE_PAN_ID_0		0x22	// Contains bits [7:0] of the 16 bit PAN ID for address filtering
#define ZIGBEE_PAN_ID_1		0x23	// Contains bits [15:8] of "
#define ZIGBEE_IEEE_ADDR_0	0x24	// Contains bits [7:0] of the 64 bit IEEE address for address filtering
#define ZIGBEE_IEEE_ADDR_1	0x25	// Contains bits [15:8] of "
#define ZIGBEE_IEEE_ADDR_2	0x26	// Contains bits [23:16] of "
#define ZIGBEE_IEEE_ADDR_3	0x27	// Contains bits [31:24] of "
#define ZIGBEE_IEEE_ADDR_4	0x28	// Contains bits [39:32] of "
#define ZIGBEE_IEEE_ADDR_5	0x29	// Contains bits [47:40] of "
#define ZIGBEE_IEEE_ADDR_6	0x2A	// Contains bits [55:48] of "
#define ZIGBEE_IEEE_ADDR_7	0x2B	// Contains bits [63:56] of "
#define ZIGBEE_XAH_CTRL		0x2C	// Controls the TX_ARET transaction in the Extended Mode
									//	7:4  MAX_FRAME_RETRIES	- specifies the max # of frame retransmission in TX_ARET transaction
									//	3:1  MAX_CSMA_RETRIES	- specifies the max # of retries in TX_ARET transaction to repeat 
									//							the random back-off/CCA procedure before the transaction gets cancelled
									//	0 - Reserved
#define ZIGBEE_CSMA_SEED_0	0x2D	// Contains a fraction of the CSMA_SEED value for the CSMA-CA algorithm.
									//	7:0  CSMA_SEED_0	- contain the lower 8 bits of the CSMA_SEED
#define ZIGBEE_CSMA_SEED_1	0x2E	// Contains a fraction of the CSMA_SEED value, the backoff exponent for 
									// the CSMA-CA algorithm and configuration bits for address filtering in RX_AACK operation
									//	7:6  MIN_BE		- defines the minimal back-off exponent used in the CSMA-CA algorithm 
									//						to generate a pseudo random number for back-off the CCA
									//	5  AACK_SET_PD		- defines the content of the frame pending subfield for acknowledgement
									//						frames in RX_AACK mode
									//	4  Reserved
									//	3  I_AM_COORD		- to be set if the node is a PAN coordinator
									//						used for address filtering in RX_AACK
									//	2:0  CSMA_SEED_1	- contain the upper 3 bits of the CSMA_SEED

/**
 * ZigBee Transceiver States (from TRX_STATUS register)
 *
 * Note:	
 *	Pull RST low => RESET
 *	A reset of all registers
 *	Forces into TRX_OFF state, but if the device is in the P_ON 
 *	state it remains in the P_ON state.	
 *
 *	Send FORCE_TRX_OFF => TRX_OFF (all states except SLEEP), 1us
 *	If in the BUSY_RX or BUSY_TX state, it interrupts the active 
 *	receiving or transmitting process, and forces an immediate transition.
 *
 *	All writes are SPI Write to Register TRX_STATE (0x02).
 *	Confirm state change by reading from register TRX_STATUS (0x01).
 *	If TRX_STATUS = 0x1F (STATE_TRANSITION_IN_PROGRESS), the AT86RF230 is
 *	on a state transition.
 */
#define ZIGBEE_TRX_STATUS_MASK				0b00011111
#define ZIGBEE_TRX_STATUS_P_ON 				0x00	// Power-on after VDD (XOSC=On, Pull-up=On)
													//	Write TRX_OFF 		=> TRX_OFF (880us)
#define ZIGBEE_TRX_STATUS_TRX_OFF			0x08	// Clock state (XOSC=On, Pull-up=Off)
													//	Pull SLP_TR high 	=> SLEEP (35us at 1MHz)
													//	Write PLL_ON 		=> PLL_ON (180us)
													//	Write RX_ON 		=> RX_ON (180us) 
#define ZIGBEE_TRX_STATUS_SLEEP 			0x0F	// Sleep State (XOSC=Off, Pull-up=Off)
													//	Pull SLP_TR low 	=> TRX_OFF (880us) 
#define ZIGBEE_TRX_STATUS_PLL_ON 			0x09	// PLL State
													//	Write TRX_OFF 		=> TRX_OFF (1us)
													//	Write RX_ON			=> RX_ON (1us)
													//	Write TX_START 		=> BUSY_TX (16us)
													//	Pull SLP_TR high 	=> "
#define ZIGBEE_TRX_STATUS_BUSY_TX			0x02	// Transmit State
													//	Frame End 			=> PLL_ON (32us) 
#define ZIGBEE_TRX_STATUS_RX_ON 			0x06	// Rx Listen State
													//	Write TRX_OFF		=> TRX_OFF (1us)
													//	Write PLL_ON		=> PLL_ON (1us)
													//	Pull SLP_TR high	=> RX_ON_NOCLK
													//	SFD Detected		=> BUSY_RX
#define ZIGBEE_TRX_STATUS_BUSY_RX 			0x01	// Receive State
													//	Frame End			=> RX_ON 
#define ZIGBEE_TRX_STATUS_RX_ON_NOCLK		0x1C	// Rx Listen State (CLKM=Off)
													//	Pull SLP_TR low		=> RX_ON
													//	SFD Detected		=> BUSY_RX
#define ZIGBEE_TRX_STATUS_BUSY_RX_AACK		0x11
#define ZIGBEE_TRX_STATUS_BUSY_TX_ARET		0x12
#define ZIGBEE_TRX_STATUS_RX_AACK_ON		0x16
#define ZIGBEE_TRX_STATUS_TX_ARET_ON		0x19
#define ZIGBEE_TRX_STATUS_RX_AACK_NOCLK			0x1D
#define ZIGBEE_TRX_STATUS_BUSY_RX_AACK_NOCLK	0x1E
#define ZIGBEE_TRX_STATUS_TRANSITION			0x1F

/* ZigBee Transceiver States (from TRAC_STATUS subregister) */
#define ZIGBEE_TRAC_STATUS_SUCCESS					0
#define ZIGBEE_TRAC_STATUS_SUCCESS_DATA_PENDING		1
#define ZIGBEE_TRAC_STATUS_CHANNEL_ACCESS_FAILURE	3
#define ZIGBEE_TRAC_STATUS_NO_ACK					5
#define ZIGBEE_TRAC_STATUS_INVALID					7

/* Radio state transition commands (from ZIGBEE_TRX_STATE's TRX_CMD subregister) */
#define ZIGBEE_TRX_CMD_MASK				0b11100000
#define ZIGBEE_TRX_CMD_NOP				0x00
#define ZIGBEE_TRX_CMD_TX_START			0x02
#define ZIGBEE_TRX_CMD_FORCE_TRX_OFF	0x03
#define ZIGBEE_TRX_CMD_RX_ON			0x06
#define ZIGBEE_TRX_CMD_TRX_OFF			0x08
#define ZIGBEE_TRX_CMD_PLL_ON			0x09
#define ZIGBEE_TRX_CMD_RX_AACK_ON		0x16
#define ZIGBEE_TRX_CMD_TX_ARET_ON		0x19

/* Clock rate of CLKM (from ZIGBEE_TRX_CTRL_0's CLKM_CTRL subregister) */
#define ZIGBEE_CLKM_SHA_SEL	3	// Immediate or after sleep
#define ZIGBEE_CLKM_SHA_IMM	0
#define ZIGBEE_CLKM_SHA_SLP	1
#define ZIGBEE_CLKM_MASK	0b11110000
#define ZIGBEE_CLKM_OFF		0
#define ZIGBEE_CLKM_1MHZ	1	// Default
#define ZIGBEE_CLKM_2MHZ	2
#define ZIGBEE_CLKM_4MHZ	3
#define ZIGBEE_CLKM_8MHZ	4
#define ZIGBEE_CLKM_16MHZ	5

/* RXTX Channel (from ZIGBEE_PHY_CC_CCA's CHANNEL subregister) */
#define ZIGBEE_CHANNEL_MASK	0b11100000
#define ZIGBEE_CHANNEL_11	0x0B	// 2405 MHz (Default)
#define ZIGBEE_CHANNEL_12	0x0C	// 2410 MHz
#define ZIGBEE_CHANNEL_13	0x0D	// 2415 MHz
#define ZIGBEE_CHANNEL_14	0x0E	// 2420 MHz
#define ZIGBEE_CHANNEL_15	0x0F	// 2425 MHz
#define ZIGBEE_CHANNEL_16	0x10	// 2430 MHz
#define ZIGBEE_CHANNEL_17	0x11	// 2435 MHz
#define ZIGBEE_CHANNEL_18	0x12	// 2440 MHz
#define ZIGBEE_CHANNEL_19	0x13	// 2445 MHz
#define ZIGBEE_CHANNEL_20	0x14	// 2450 Mhz
#define ZIGBEE_CHANNEL_21	0x15	// 2455 Mhz
#define ZIGBEE_CHANNEL_22	0x16	// 2460 Mhz
#define ZIGBEE_CHANNEL_23	0x17	// 2465 Mhz
#define ZIGBEE_CHANNEL_24	0x18	// 2470 Mhz
#define ZIGBEE_CHANNEL_25	0x19	// 2475 Mhz
#define ZIGBEE_CHANNEL_26	0x1A	// 2480 Mhz

/* Zigbee IRQ Masks from the ZIGBEE_IRQ_MASK & ZIGBEE_IRQ_STATUS */
#define ZIGBEE_IRQ_MASK_ENABLE		1
#define ZIGBEE_IRQ_MASK_DISABLE		0
#define ZIGBEE_IRQ_MASK_ALL			0b11001111
#define ZIGBEE_IRQ_MASK_NONE		0b00000000
#define ZIGBEE_IRQ_MASK_BAT_LOW		7			// 0b10000000
#define	ZIGBEE_IRQ_MASK_TRX_UR		6			// 0b01000000
#define	ZIGBEE_IRQ_MASK_TRX_END		3			// 0b00001000
#define ZIGBEE_IRQ_MASK_RX_START	2			// 0b00000100
#define ZIGBEE_IRQ_MASK_PLL_UNLOCK	1			// 0b00000010
#define ZIGBEE_IRQ_MASK_PLL_LOCK	0			// 0b00000001

/* Zigbee Hardware ID data */
uint8_t zigbee_hw_part_num, zigbee_hw_ver_num;
uint8_t zigbee_hw_man_id_0, zigbee_hw_man_id_1;

/* Global Variables */
uint8_t zigbee_state = ZIGBEE_TRX_STATUS_P_ON;
uint8_t zigbee_clkm = ZIGBEE_CLKM_1MHZ;
uint8_t zigbee_channel = ZIGBEE_CHANNEL_11;

volatile uint8_t zigbee_pll_state = FALSE;
volatile uint8_t zigbee_irq_trx_end_flag = TRUE;
volatile uint8_t zigbee_irq_rx_start_flag = FALSE;
volatile uint8_t zigbee_irq_trx_ur_flag = FALSE;
volatile uint8_t zigbee_irq_bat_low_flag = FALSE;

uint8_t frame_buffer[ZIGBEE_FRAME_BUFFER_LENGTH];	

/* Prototype Functions */
void zigbee_init(void);
void zigbee_hw_rst(void);
void zigbee_sw_rst(void);

uint8_t zigbee_get_trx_status(void);
void zigbee_set_state_trx_cmd(uint8_t state_trx_cmd);

void zigbee_set_clkm(uint8_t clkm, uint8_t mode);
void zigbee_set_channel(uint8_t chan);

void zigbee_set_irq_mask_bit(uint8_t irq_bit, uint8_t en);
void zigbee_set_irq_mask(uint8_t reg_value);
void zigbee_update_irq_status(void);


uint8_t zigbee_get_trx_status(void) {
	return spi_read_register(ZIGBEE_TRX_STATUS) & ZIGBEE_TRX_STATUS_MASK;
}

void zigbee_set_state_trx_cmd(uint8_t state_trx_cmd) {
	uint8_t trx_status_old = zigbee_get_trx_status();
	
	uint8_t trx_state_value = spi_read_register(ZIGBEE_TRX_STATE);
	trx_state_value = (trx_state_value & ZIGBEE_TRX_CMD_MASK) | state_trx_cmd;
	spi_write_register(ZIGBEE_TRX_STATE, trx_state_value);
	
	/* Transition Specific Instructions */
	switch(state_trx_cmd) {
		case ZIGBEE_TRX_CMD_NOP:
			break;
		case ZIGBEE_TRX_CMD_TX_START:
			break;
		case ZIGBEE_TRX_CMD_FORCE_TRX_OFF:
			break;
		case ZIGBEE_TRX_CMD_RX_ON:
			switch(trx_status_old) {
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					_delay_us(180);
					/* ZIGBEE_IRQ_MASK_PLL_LOCK */
					break;
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
					/* ZIGBEE_IRQ_MASK_TRX_END */
					_delay_us(1);
					break;
			}
			/* ZIGBEE_TRX_STATUS_RX_ON */
			break;
		case ZIGBEE_TRX_CMD_TRX_OFF:
			switch(trx_status_old) {
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
					/* ZIGBEE_IRQ_MASK_TRX_END */
					break;
				default:	// Any Active states (except P_ON)
					_delay_us(1);
			}
			/* ZIGBEE_TRX_STATUS_TRX_OFF */
			break;
		case ZIGBEE_TRX_CMD_PLL_ON:
			switch(trx_status_old) {
				case ZIGBEE_TRX_STATUS_TRX_OFF:
					_delay_us(180);
					/* IRQ Interrupt: ZIGBEE_IRQ_MASK_PLL_LOCK */
					
					break;
				case ZIGBEE_TRX_STATUS_PLL_ON:
				case ZIGBEE_TRX_STATUS_RX_ON:
					_delay_us(1);
					break;
				case ZIGBEE_TRX_STATUS_BUSY_TX:
				case ZIGBEE_TRX_STATUS_BUSY_RX:
					/* ZIGBEE_IRQ_MASK_TRX_END */
					_delay_us(1);
					break;
			}
			/* ZIGBEE_TRX_STATUS_PLL_ON */
			break;
		case ZIGBEE_TRX_CMD_RX_AACK_ON:
			break;
		case ZIGBEE_TRX_CMD_TX_ARET_ON:
			break;
	}
	
	zigbee_state = zigbee_get_trx_status();
}

void zigbee_tx_start_pin(void) {
	sbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	_delay_us(1);
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);
	
	zigbee_state = zigbee_get_trx_status();
}

void zigbee_set_clkm(uint8_t clkm, uint8_t mode) {
	uint8_t reg_value = spi_read_register(ZIGBEE_TRX_CTRL_0);
	reg_value = (reg_value & ZIGBEE_CLKM_MASK) | (mode << ZIGBEE_CLKM_SHA_SEL) | clkm;
	spi_write_register(ZIGBEE_TRX_CTRL_0, reg_value);
	zigbee_clkm = clkm;
}

void zigbee_set_channel(uint8_t chan) {
	uint8_t reg_value = spi_read_register(ZIGBEE_PHY_CC_CCA);
	reg_value = (reg_value & ZIGBEE_CHANNEL_MASK) | chan;
	spi_write_register(ZIGBEE_PHY_CC_CCA, reg_value);
	zigbee_channel = chan;
}

void zigbee_set_irq_mask_bit(uint8_t irq_bit, uint8_t en) {
	uint8_t reg_value = spi_read_register(ZIGBEE_IRQ_MASK);
	reg_value = (reg_value & ~_BV(irq_bit)) | (en << irq_bit);
	spi_write_register(ZIGBEE_IRQ_MASK, reg_value);
}

void zigbee_set_irq_mask(uint8_t reg_value) {
	spi_write_register(ZIGBEE_IRQ_MASK, reg_value);
}

void zigbee_update_irq_status(void) {
	uint8_t irq_state = spi_read_register(ZIGBEE_IRQ_STATUS);
	
	_delay_us(50);			// Need to delay for at least 50 us or it will hang
							// This is most likely due to bits not settling
	
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_BAT_LOW)) {
		zigbee_irq_bat_low_flag = TRUE;
		
		#if ZIGBEE_UART0_DEBUG
		uart0_send_byte(0xC0);
		#endif
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_TRX_UR)) {
		zigbee_irq_trx_ur_flag = TRUE;
		
		#if ZIGBEE_UART0_DEBUG
		uart0_send_byte(0xC1);
		#endif
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_TRX_END)) {
		zigbee_irq_trx_end_flag = TRUE;
		
		#if ZIGBEE_UART0_DEBUG
		uart0_send_byte(0xC2);
		#endif
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_RX_START)) {
		zigbee_irq_rx_start_flag = TRUE;
		
		#if ZIGBEE_UART0_DEBUG
		uart0_send_byte(0xC3);
		#endif
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_PLL_UNLOCK)) {
		zigbee_pll_state = FALSE;
		
		#if ZIGBEE_UART0_DEBUG
		uart0_send_byte(0xC4);
		#endif
	}
	if (irq_state & _BV(ZIGBEE_IRQ_MASK_PLL_LOCK)) {
		zigbee_pll_state = TRUE;
		
		#if ZIGBEE_UART0_DEBUG
		uart0_send_byte(0xC5);
		#endif
	}
}

/**
 * A dedicated reset signal for hardware/software synchronization
 * 
 * Brings the radio transceiver into the TRX_OFF state 
 * after a power-on event.
 */
void zigbee_init(void) {
	/* Transceiver Initialization */
	_delay_us(510);		// Delay 510us for transition to P_ON
	//_delay_us(510/CPU_FREQ_RATIO);
	
	/* Transceiver Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	cbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST low
	_delay_us(6);		// Delay 6us for hardware reset
	//_delay_us(6/CPU_FREQ_RATIO);
	sbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST high
    
	/* Read Hardware Identification Data */
	zigbee_hw_part_num = spi_read_register(ZIGBEE_PART_NUM);
	zigbee_hw_ver_num = spi_read_register(ZIGBEE_VER_NUM);
	zigbee_hw_man_id_0 = spi_read_register(ZIGBEE_MAN_ID_0);
	zigbee_hw_man_id_1 = spi_read_register(ZIGBEE_MAN_ID_1);
	
	/* Disable IRQ Mask and dummy read IRQ Status to avoid unhandled interrupts */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_NONE);
	spi_read_register(ZIGBEE_IRQ_STATUS);
	
	/* Send TRX_OFF Command */
	zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
	_delay_us(510);		// Delay 510us for transition from P_ON to TRX_OFF
	//_delay_us(510/CPU_FREQ_RATIO);

	/* Get current state */
	zigbee_state = zigbee_get_trx_status();
	
	/* Enable IRQ Mask */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_ALL);
}

/**
 * Resets the Zigbee Transceiver from any state except P_ON
 *
 * The programming sequence will typically be used to recover 
 * from fatal errors and bring the radio transceiver to a known state.
 * All registers will have their default values and the state machine 
 * will be in TRX_OFF state.
 */
void zigbee_hw_rst(void) {
	/* Transceiver Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	cbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST low
	_delay_us(6);		// Delay 6us for hardware reset
	sbi(ZIGBEE_PORT, ZIGBEE_RST);		// Pull RST high
	
	/* Disable IRQ Mask and dummy read IRQ Status to avoid unhandled interrupts */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_NONE);
	spi_read_register(ZIGBEE_IRQ_STATUS);
	
	_delay_us(120);
	
	zigbee_state = zigbee_get_trx_status();
	
	/* Enable IRQ Mask */
	zigbee_set_irq_mask(ZIGBEE_IRQ_MASK_ALL);
}

/**
 * Resets the state machine of the radio transceiver to the TRX_OFF state.
 */
void zigbee_sw_rst(void) {
	/* State Machine Reset */
	cbi(ZIGBEE_PORT, ZIGBEE_SLP_TR);	// Pull SLP low
	_delay_us(6);		// Delay 6us for hardware reset
	
	/* Set to TRX_OFF state */
	zigbee_set_state_trx_cmd(ZIGBEE_TRX_CMD_TRX_OFF);
	_delay_us(6);		// Delay 6us for hardware reset
	
	zigbee_state = zigbee_get_trx_status();
}
