/**
 * The data manipulation class, containing static methods responsible for
 * converting between different formats such as hex, byte, int, and binary.
 * 
 * @author Ruibing Wang (rw98@cornell.edu)
 * 
 */
public class Data {
	/**
	 * A static method that converts a single byte to hex.
	 * @param b - a byte
	 * @return a hex character in a String
	 */
	public static String byteToHex(byte b) {
		Byte by = new Byte(b);
		String str = Integer.toString((b & 0xff) + 0x100, 16 /* radix */).substring(1);
		str = str.toUpperCase();
		return str;
		//return Integer.toHexString(by.intValue());
	}

	/**
	 * A static method that converts an array of bytes to hex.
	 * 
	 * @param in - a byte array
	 * @return a String of hex characters representing the byte array
	 */
	public static String byteArrayToHex(byte in[]) {
		byte ch = 0x00;
		int i = 0;
		if (in == null || in.length <= 0)
			return null;
		String pseudo[] = { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
				"A", "B", "C", "D", "E", "F" };
		StringBuffer out = new StringBuffer(in.length * 2);
		while (i < in.length) {
			ch = (byte) (in[i] & 0xF0);
			ch = (byte) (ch >>> 4);
			ch = (byte) (ch & 0x0F);
			out.append(pseudo[(int) ch]);
			ch = (byte) (in[i] & 0x0F);
			out.append(pseudo[(int) ch]);
			i++;

		}
		String rslt = new String(out);
		rslt = rslt.toUpperCase();
		return rslt;
	}

	/**
	 * A static method that converts a hex to an array of bytes.
	 * 
	 * @param s - a String of hex
	 * @return a byte array representing the hex String
	 */
	public static byte[] hexToByteArray(String s) {
		int stringLength = s.length();
		if ((stringLength & 0x1) != 0) {
			System.out.println("hexToByteArray() requires an even number of hex characters");
			return null;
		}
		byte[] b = new byte[stringLength / 2];

		for (int i = 0, j = 0; i < stringLength; i += 2, j++) {
			int high = charToNibble(s.charAt(i));
			int low = charToNibble(s.charAt(i + 1));
			if (high == 666 | low == 666) {
				System.out.println("Invalid hex character");
				return null;
			}
			b[j] = (byte) ((high << 4) | low);
		}
		return b;
	}

	/**
	 * A static method that converts a hex character to an int.
	 * 
	 * @param char - a char
	 * @return an int representing the char
	 */
	private static int charToNibble(char c) {
		if ('0' <= c && c <= '9') {
			return c - '0';
		} else if ('a' <= c && c <= 'f') {
			return c - 'a' + 0xa;
		} else if ('A' <= c && c <= 'F') {
			return c - 'A' + 0xa;
		} else {
			return 666;
		}
	}

	/**
	 * A static method that converts a hex to int.
	 * No lead 0x, case sensitive, lower case.
	 * 
	 * @param str - a single hex with no leading 0x in lower case
	 * @return an int representing the hex
	 */ 
	public static int hexToInt(String str) {
		return Integer.parseInt(str.trim(), 16);
	}

	/**
	 * A static method that converts an int to hex.
	 * 
	 * @param i - an int
	 * @return a hex String representing the int
	 */
	public static String intToHex(int i) {
		//String hex = Integer.toHexString(i);
		String hex = Integer.toHexString(0x100 | (0x0ff & i)).substring(1);
		if (hex.length() == 1) {
			hex = "0" + hex;
		}
		hex = hex.toUpperCase();
		return hex;
	}

	/**
	 * A static method that converts an int to an array of bytes
	 * 
	 * @param integer - an int
	 * @return a byte array representing the int
	 */
	public static byte[] intToByteArray(int integer) {
		int byteNum = (40 - Integer.numberOfLeadingZeros(integer < 0 ? ~integer
				: integer)) / 8;
		byte[] byteArray = new byte[4];

		for (int n = 0; n < byteNum; n++)
			byteArray[3 - n] = (byte) (integer >>> (n * 8));

		return (byteArray);
	}

	/**
	 * A static method that converts a hex string to a binary string.
	 * It doesn't keep leading zeroes
	 *
	 * @param hex - hex string being converted
	 * @return a binary string
	 */
	public static String hexToBinary(String hex) {
		return Integer.toString(hexToInt(hex),2);
	}
	
	/**
	 * A static method that converts an integer value to a binary String
	 * zero padded to ensure a length of 8.
	 * 
	 * @param value
	 * @return String
	 */
	public static String intToBinary(int value) {
		String data = Integer.toString(value,2);
		for (int i=data.length(); i<8; i++) {
			data = "0" + data;
		}
		return data;
	}
}
