/**
 * The sensor node class, responsible for representing a sensor node and latest times
 * of sensor detections
 * 
 * @author Ruibing Wang (rw98@cornell.edu)
 *
 */
public class Node {
	/* Local protected variables */
	protected String shortAddr;
	protected String[] sensorDetectionList = {"","","",""};
	
	/**
	 * This method constructs the Node class with the short address ID of the sensor node
	 * 
	 * @param shortAddr
	 */
	public Node(String shortAddr) {
		this.shortAddr = shortAddr;
	}

	/**
	 * This method returns the short address of the sensor node
	 * 
	 * @return String
	 */
	public String getShortAddr() {
		return shortAddr;
	}

	/**
	 * This method sets the short address of the sensor node
	 * 
	 * @param shortAddr
	 */
	public void setShortAddr(String shortAddr) {
		this.shortAddr = shortAddr;
	}
	
	/**
	 * This method appends the sensor detection array of occurance date/time with a new one
	 * while moving older data down (disgarding the oldest if neccessary).
	 * 
	 * @param detectionTime
	 */
	public void appendSensorDetection(String detectionTime) {
		sensorDetectionList[3] = sensorDetectionList[2];
		sensorDetectionList[2] = sensorDetectionList[1];
		sensorDetectionList[1] = sensorDetectionList[0];
		sensorDetectionList[0] = detectionTime;
	}
     
	/**
	 * This method returns the occurrance time for the sensor node of the given index
	 * 
	 * @param index
	 * @return String
	 */
	public String getSensorDetection(int index) {
		if (index > 3) {
			return "";
		}
		return sensorDetectionList[index];
	}
	
	public int getSensorDetectionLength() {
		return sensorDetectionList.length;
	}
}
