import javax.swing.SwingUtilities;
import javax.swing.JPanel;
import javax.swing.JFrame;
import java.awt.Rectangle;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JTabbedPane;
import java.util.GregorianCalendar;
import java.util.Vector;
import java.util.Calendar;
import javax.swing.JTextField;
import javax.swing.JTable;

/**
 * The main Sentinel UI class responsible for the UI, initializing the Communicator class,
 * and managing all high level funcitonalities.
 * 
 * @author Ruibing Wang (rw98@cornell.edu)
 *
 */
public class Sentinel extends JFrame {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JTextArea textArea = null;
	private JScrollPane textScrollPane = null;
	private JTabbedPane tabbedPane = null;
	private JPanel accessPanel = null;
	private JPanel monitorPanel = null;
	private JPanel debugPanel = null;
	private JButton discoverButton = null;
	private JButton syncButton = null;
	private JButton mapButton = null;
	private JTextField inputTextField = null;
	private JButton sendTextButton = null;
	private JButton clearButton = null;
	private JButton infoButton = null;
	private JButton ResetLocalDHCPButton = null;
	private JButton ClearAllLeaseButton = null;
	private JButton ClearRelayLeaseButton = null;
	private JButton ClearSensorLeaseButton = null;
	private Vector sensorNodeList = null;
	private JScrollPane jScrollPane = null;
	private JTable jNodeTable = null;
	
	private Communicator communicator = null;
	private GregorianCalendar calendar = null;

	/**
	 * This method initializes displayPane	
	 * 	
	 * @return javax.swing.JTextPane	
	 */
	private JTextArea getTextArea() {
		if (textArea == null) {
			textArea = new JTextArea();
			textArea.setEditable(false);
			textArea.setLineWrap(true);
			//textArea.setFont(new Font(Font.MONOSPACED, Font.BOLD, 12));
		}
		return textArea;
	}
	
	/**
	 * This method initializes textScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getTextScrollPane() {
		if (textScrollPane == null) {
			textScrollPane = new JScrollPane();
			textScrollPane.setBounds(new Rectangle(2, 198, 629, 460));
			textScrollPane.setViewportView(getTextArea());
			textScrollPane.setWheelScrollingEnabled(true);
			
		}
		return textScrollPane;
	}

	/**
	 * This method initializes tabbedPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getTabbedPane() {
		if (tabbedPane == null) {
			tabbedPane = new JTabbedPane();
			tabbedPane.setBounds(new Rectangle(0, 0, 631, 196));
			tabbedPane.addTab("Access", null, getAccessPanel(), null);
			tabbedPane.addTab("Monitor", null, getMonitorPanel(), null);
			tabbedPane.addTab("Debug", null, getDebugPanel(), null);
			tabbedPane.addChangeListener(new javax.swing.event.ChangeListener() {
				public void stateChanged(javax.swing.event.ChangeEvent e) {
					if (tabbedPane.getSelectedComponent() == getAccessPanel()) {
						communicator.disableSerialDebug();
					}
					else if (tabbedPane.getSelectedComponent() == getMonitorPanel()) {
						communicator.disableSerialDebug();
					}
					else if (tabbedPane.getSelectedComponent() == getDebugPanel()) {
						communicator.enableSerialDebug();
					}
				}
			});
		}
		return tabbedPane;
	}

	/**
	 * This method initializes accessPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getAccessPanel() {
		if (accessPanel == null) {
			accessPanel = new JPanel();
			accessPanel.setLayout(null);
			accessPanel.add(getDiscoverButton(), null);
			//accessPanel.add(getSyncButton(), null);
			//accessPanel.add(getMapButton(), null);
			accessPanel.add(getInfoButton(), null);
			accessPanel.add(getResetLocalDHCPButton(), null);
			accessPanel.add(getClearAllLeaseButton(), null);
			accessPanel.add(getClearRelayLeaseButton(), null);
			accessPanel.add(getClearSensorLeaseButton(), null);
		}
		return accessPanel;
	}

	/**
	 * This method initializes monitorPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getMonitorPanel() {
		if (monitorPanel == null) {
			monitorPanel = new JPanel();
			monitorPanel.setLayout(null);
			monitorPanel.add(getJScrollPane(), null);
		}
		return monitorPanel;
	}

	/**
	 * This method initializes debugPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getDebugPanel() {
		if (debugPanel == null) {
			debugPanel = new JPanel();
			debugPanel.setLayout(null);
		}
		return debugPanel;
	}

	/**
	 * This method initializes discoverButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getDiscoverButton() {
		if (discoverButton == null) {
			discoverButton = new JButton();
			discoverButton.setText("Discover");
			discoverButton.setBounds(new Rectangle(14, 15, 84, 26));
			discoverButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					communicator.requestDiscover();
				}
			});
		}
		return discoverButton;
	}

	/**
	 * This method initializes syncButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSyncButton() {
		if (syncButton == null) {
			syncButton = new JButton();
			syncButton.setText("Sync");
			syncButton.setBounds(new Rectangle(120, 15, 62, 26));
		}
		return syncButton;
	}

	/**
	 * This method initializes mapButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getMapButton() {
		if (mapButton == null) {
			mapButton = new JButton();
			mapButton.setText("Map");
			mapButton.setBounds(new Rectangle(210, 15, 58, 26));
		}
		return mapButton;
	}

	/**
	 * This method initializes inputTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getInputTextField() {
		if (inputTextField == null) {
			inputTextField = new JTextField();
			inputTextField.setBounds(new Rectangle(2, 658, 475, 37));
		}
		return inputTextField;
	}

	/**
	 * This method initializes sendTextButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSendTextButton() {
		if (sendTextButton == null) {
			sendTextButton = new JButton();
			sendTextButton.setBounds(new Rectangle(478, 658, 76, 37));
			sendTextButton.setText("Send");
			sendTextButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					
					//println("Send"); // TODO Auto-generated Event stub actionPerformed()
				}
			});
		}
		return sendTextButton;
	}

	/**
	 * This method initializes clearButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getClearButton() {
		if (clearButton == null) {
			clearButton = new JButton();
			clearButton.setBounds(new Rectangle(555, 658, 75, 37));
			clearButton.setText("Clear");
			clearButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					textArea.setText("");
					//println("Clear"); // TODO Auto-generated Event stub actionPerformed()
				}
			});
			
		}
		return clearButton;
	}

	/**
	 * This method initializes infoButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getInfoButton() {
		if (infoButton == null) {
			infoButton = new JButton();
			infoButton.setBounds(new Rectangle(14, 60, 84, 26));
			infoButton.setText("Info");
			infoButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					communicator.requestInfo();
				}
			});
		}
		return infoButton;
	}

	/**
	 * This method initializes ResetLocalDHCPButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getResetLocalDHCPButton() {
		if (ResetLocalDHCPButton == null) {
			ResetLocalDHCPButton = new JButton();
			ResetLocalDHCPButton.setBounds(new Rectangle(465, 15, 151, 26));
			ResetLocalDHCPButton.setText("Reset Local DHCP");
			ResetLocalDHCPButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					communicator.requestClearLocalDHCP();
				}
			});
		}
		return ResetLocalDHCPButton;
	}

	/**
	 * This method initializes ClearAllLeaseButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getClearAllLeaseButton() {
		if (ClearAllLeaseButton == null) {
			ClearAllLeaseButton = new JButton();
			ClearAllLeaseButton.setBounds(new Rectangle(465, 45, 151, 26));
			ClearAllLeaseButton.setText("Clear All Lease");
			ClearAllLeaseButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					communicator.requestClearClientList();
				}
			});
		}
		return ClearAllLeaseButton;
	}

	/**
	 * This method initializes ClearRelayLeaseButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getClearRelayLeaseButton() {
		if (ClearRelayLeaseButton == null) {
			ClearRelayLeaseButton = new JButton();
			ClearRelayLeaseButton.setBounds(new Rectangle(465, 75, 151, 26));
			ClearRelayLeaseButton.setText("Clear Relay Lease");
			ClearRelayLeaseButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					communicator.requestClearRelayList();
				}
			});
		}
		return ClearRelayLeaseButton;
	}

	/**
	 * This method initializes ClearSensorLeaseButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getClearSensorLeaseButton() {
		if (ClearSensorLeaseButton == null) {
			ClearSensorLeaseButton = new JButton();
			ClearSensorLeaseButton.setBounds(new Rectangle(465, 105, 151, 26));
			ClearSensorLeaseButton.setText("Clear Sensor Lease");
			ClearSensorLeaseButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					communicator.requestClearSensorList();
				}
			});
		}
		return ClearSensorLeaseButton;
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(0, 3, 623, 162));
			jScrollPane.setViewportView(getJNodeTable());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jNodeTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getJNodeTable() {
		if (jNodeTable == null) {
			// Adds column names to the node monitor table
			String[] columnNames = {"Sensor ID", "Time1", "Time2", "Time3", "Time4"};
			// Adds blank data to node monitor table
			Object[][] data = {
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""},
				    {"", "", "", "", ""}
			};
			jNodeTable = new JTable(data, columnNames);
		}
		return jNodeTable;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Sentinel thisClass = new Sentinel();
				thisClass.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				thisClass.setVisible(true);
			}
		});
	}

	/**
	 * This is the default constructor
	 */
	public Sentinel() {
		super();
		initialize();
		communicator = new Communicator(this);
		communicator.start();
		sensorNodeList = new Vector();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(649, 733);
		this.setContentPane(getJContentPane());
		this.setTitle("Sentinel");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getTabbedPane(), null);
			jContentPane.add(getInputTextField(), null);
			jContentPane.add(getSendTextButton(), null);
			jContentPane.add(getClearButton(), null);
			jContentPane.add(getTextScrollPane(), null);
		}
		return jContentPane;
	}
	
	/**
	 * This method returns the date with the parameter divider
	 * For example, a parameter of "/" would yield "01/01/08"
	 * 
	 * @param divider
	 * @return String
	 */
	public String getDate(String divider) {
		calendar = new GregorianCalendar();
		String date;
		int month = calendar.get(Calendar.MONTH);
		if (month < 10) {
			date = "0" + month;
		}
		else {
			date = "" + month;
		}
		int day = calendar.get(Calendar.DATE);
		if (day < 10) {
			date = date + divider + "0" + day;
		}
		else {
			date = date + divider + day;
		}
		int year = calendar.get(Calendar.YEAR);
		if (year < 10) {
			date = date + divider + "0" + year;
		}
		else {
			date = date + divider + year;
		}
		return date;
	}
	
	/**
	 * This method returns the time with the parameter divider
	 * For example, a parameter of ":" would yield "01:23:45"
	 * 
	 * @param divider
	 * @return String
	 */
	public String getTime(String divider) {
		calendar = new GregorianCalendar();
		String time;
		int hour = calendar.get(Calendar.HOUR);
		if (hour < 10) {
			time = "0" + hour;
		}
		else {
			time = "" + hour;
		}
		int minute = calendar.get(Calendar.MINUTE);
		if (minute < 10) {
			time = time + divider + "0" + minute;
		}
		else {
			time = time + divider + minute;
		}
		int second = calendar.get(Calendar.SECOND);
		if (second < 10) {
			time = time + divider + "0" + second;
		}
		else {
			time = time + divider + second;
		}
		return time;
	}
	
	/**
	 * This method inserts a new line of text to the UI textArea
	 * 
	 * @param msg
	 */
	public void println(String msg) {
		/* Insert from the Top */
		msg = "\n" + getTime(":") + " - " + msg;
		textArea.insert(msg, 0);
		
		/* Insert from the Bottom */
		//msg = getTime(":") + " - " + msg + "\n";
		//textArea.append(msg);
	}
	
	/**
	 * This methods appends a line of text to an existing line in the UI textArea
	 * 
	 * @param msg
	 */
	public void print(String msg) {
		textArea.insert(msg, 0);	// Top
		//textArea.append(msg);		// Bottom
	}
	
	/**
	 * This method updates the monitorTable with the sensorNodeList data
	 *
	 */
	public void updateMonitorTable() {
		Node sensorNode = null;
		for (int i=0; i<sensorNodeList.size(); i++) {
			sensorNode = ((Node)sensorNodeList.get(i));
			jNodeTable.setValueAt(sensorNode.getShortAddr(), i, 0);
			for (int j=0; j<sensorNode.getSensorDetectionLength(); j++) {
				System.out.println(sensorNode.getSensorDetection(j));
				jNodeTable.setValueAt(sensorNode.getSensorDetection(j), i, j+1);
			}
		}
	}
	
	/**
	 * This method updates the sensorNodeList with a new sensor detection source address.
	 * If the source address matches an existing sensorNode, it will update the occurance time.
	 * Otherwise, it would creaste a new sensorNode, add the new date, and append the sensorNode to the list.
	 * 
	 * @param shortAddr
	 */
	public void sensorDetection(String shortAddr) {
		Node sensorNode = null;
		boolean sensorFoundFlag = false;
		for (int i=0; i<sensorNodeList.size(); i++) {
			sensorNode = ((Node)sensorNodeList.get(i));
			if (sensorNode.getShortAddr().equals(shortAddr)) {
				sensorNode.appendSensorDetection(getDate(":")+"-"+getTime(":"));
				sensorFoundFlag = true;
				i = sensorNodeList.size();
			}
		}
		if (!sensorFoundFlag) {
			sensorNode = new Node(shortAddr);
			sensorNode.appendSensorDetection(getDate("/")+"-"+getTime(":"));
			sensorNodeList.add(sensorNode);
		}
		updateMonitorTable();
	}

}  //  @jve:decl-index=0:visual-constraint="5,-9"
