import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.Vector;
import java.util.Calendar;
import java.sql.Timestamp;


public class Zigbee extends Thread {
	final private int UART_RX_ZIGBEE_STATE =		0x20;
	final private int UART_RX_ZIGBEE_IRQ_STATUS =	0x21;
	final private int UART_RX_ZIGBEE_TRAC_STATUS =	0x22;
	
	final private int UART_RX_ZIGBEE_PHY_DATA =		0x30;	// UART_TX_ZIGBEE_RX_DATA | length | data
	final private int UART_RX_ZIGBEE_PHY_LQI =		0x31;
	
	final private int UART_RX_ZIGBEE_MAC_DATA =		0x40;
	final private int UART_RX_ZIGBEE_MAC_DEST_PAN_ID =	0x41;
	final private int UART_RX_ZIGBEE_MAC_DEST_ADDR = 	0x42;
	final private int UART_RX_ZIGBEE_MAC_SRC_PAN_ID =	0x43;
	final private int UART_RX_ZIGBEE_MAC_SRC_ADDR =		0x44;
	
	final private int UART_RX_ZIGBEE_CCA_MODE =		0x50;
	final private int UART_RX_ZIGBEE_CCA_ED_THRES =	0x51;
	final private int UART_RX_ZIGBEE_CCA_VALUE =	0x52;
	
	final private int UART_RX_ZIGBEE_PART_NUM =		0xE0;
	final private int UART_RX_ZIGBEE_VER_NUM =		0xE1;
	final private int UART_RX_ZIGBEE_MAN_ID_0 = 	0xE2;
	final private int UART_RX_ZIGBEE_MAN_ID_1 =		0xE3;
	
	private Serial serial = null;
	
	private Integer rxHeader, rxLength, rxData;
	private String rxOutput;
	private int oldRxDataValue, newRxDataValue, rxDataErrorCounter;
	
	public Zigbee() {
		serial = new Serial();
	}
	
	public void start() {
		super.start();
	}
	
	public void run() {
		while (true) {
			rxHeader = pollRxQueue();
			if (rxHeader != null) {
				switch (rxHeader.intValue()) {
					case UART_RX_ZIGBEE_STATE:							// ZIGBEE_STATE
						rxOutput = "state:";
						rxData = pollRxQueue();
						if (rxData != null) {
							switch(rxData.intValue()) {
								case 0x00:
									rxOutput = rxOutput + " P_ON";
									break;
								case 0x08:
									rxOutput = rxOutput + " TRX_OFF";
									break;
								case 0x0F:
									rxOutput = rxOutput + " SLEEP";
									break;
								case 0x09:
									rxOutput = rxOutput + " PLL_ON";
									break;
								case 0x02:
									rxOutput = rxOutput + " BUSY_TX";
									break;
								case 0x06:
									rxOutput = rxOutput + " RX_ON";
									break;
								case 0x01:
									rxOutput = rxOutput + " BUSY_RX";
									break;
								case 0x1C:
									rxOutput = rxOutput + " RX_ON_NOCLK";
									break;
								case 0x11:
									rxOutput = rxOutput + " BUSY_RX_AACK";
									break;
								case 0x12:
									rxOutput = rxOutput + " BUSY_TX_ARET";
									break;
								case 0x16:
									rxOutput = rxOutput + " RX_AACK_ON";
									break;
								case 0x19:
									rxOutput = rxOutput + " TX_ARET_ON";
									break;
								case 0x1D:
									rxOutput = rxOutput + " RX_AACK_NOCLK";
									break;
								case 0x1E:
									rxOutput = rxOutput + " BUSY_RX_AACK_NOCLK";
									break;
								case 0x1F:
									rxOutput = rxOutput + " TRANSITION";
									break;
								default:
									rxOutput = rxOutput + " Unknown";
									break;
							}
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_IRQ_STATUS:						// IRQ_STATUS
						rxOutput = "irqStatus:";
						rxData = pollRxQueue();
						if (rxData != null) {
							rxOutput = rxOutput + " 0b" + Data.intToBinary(rxData.intValue());
							if (Data.intToBinary(rxData.intValue()).charAt(7) == '1') {
								rxOutput = rxOutput + " - PLL Lock";
							}
							if (Data.intToBinary(rxData.intValue()).charAt(6) == '1') {
								rxOutput = rxOutput + " - PLL Unlock";
							}
							if (Data.intToBinary(rxData.intValue()).charAt(5) == '1') {
								rxOutput = rxOutput + " - RX Start";
							}
							if (Data.intToBinary(rxData.intValue()).charAt(4) == '1') {
								rxOutput = rxOutput + " - TRX End";
							}
							if (Data.intToBinary(rxData.intValue()).charAt(1) == '1') {
								rxOutput = rxOutput + " - TRX UR";
							}
							if (Data.intToBinary(rxData.intValue()).charAt(0) == '1') {
								rxOutput = rxOutput + " - BAT Low";
							}
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_TRAC_STATUS:					// TRAC_STATUS
						rxOutput = "tracStatus:";
						rxData = pollRxQueue();
						if (rxData != null) {
							switch(rxData.intValue()) {
								case 0:
									rxOutput = rxOutput + " Success";
									break;
								case 1:
									rxOutput = rxOutput + " Success Data Pending";
									break;
								case 3:
									rxOutput = rxOutput + " Success Access Failure";
									break;
								case 5:
									rxOutput = rxOutput + " No Ack";
									break;
								case 7:
									rxOutput = rxOutput + " Invalid";
									break;
								default:
									rxOutput = rxOutput + " Unknown";
									break;
							}
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_PHY_DATA:						// PHY_DATA
						rxOutput = "phyData";
						rxLength = pollRxQueue();
						if (rxLength != null) {
							rxOutput = rxOutput + "[" + rxLength.intValue() + "]:";
							for (int i=0; i<rxLength.intValue(); i++) {
								rxData = pollRxQueue();
								if (rxData != null) {
									rxOutput = rxOutput + " 0x" + Data.intToHex(rxData.intValue());
								}
								else {
									rxOutput = rxOutput + " ERROR";
									i = rxLength.intValue();
								}
							}
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_PHY_LQI:					// PHY_LQI
						rxOutput = "phyLQI:";
						rxData = pollRxQueue();
						if (rxData != null) {
							rxOutput = rxOutput + " " + rxData.intValue();
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_MAC_DATA:
						rxOutput = "macData";
						rxLength = pollRxQueue();
						if (rxLength != null) {
							rxOutput = rxOutput + "[" + rxLength.intValue() + "]:";
							for (int i=0; i<rxLength.intValue(); i++) {
								rxData = pollRxQueue();
								if (rxData != null) {
									rxOutput = rxOutput + " 0x" + Data.intToHex(rxData.intValue());
								}
								else {
									rxOutput = rxOutput + " ERROR";
									
									i = rxLength.intValue();
								}
							}
						}
						else {
							rxOutput = rxOutput + " ERROR";
						}
						System.out.println(rxOutput);
						break;
					case UART_RX_ZIGBEE_MAC_DEST_PAN_ID:
						rxOutput = "macDestPanID: ";
						rxData = pollRxQueue();
						if (rxData != null) {
							String str = Data.intToHex(rxData.intValue());
							rxData = pollRxQueue();
							if (rxData != null) {
								rxOutput = rxOutput + " 0x" + Data.intToHex(rxData.intValue()) + str;
							}
							else {
								rxOutput = rxOutput + " ERROR";
							}
						}
						else {
							rxOutput = rxOutput + " ERROR";
						}
						System.out.println(rxOutput);
						break;
					case UART_RX_ZIGBEE_MAC_DEST_ADDR:
						rxOutput = "macDestAddr: ";
						rxLength = pollRxQueue();
						if (rxLength != null) {
							switch (rxLength.intValue()) {
								case 2:
									rxData = pollRxQueue();
									if (rxData != null) {
										String str = Data.intToHex(rxData.intValue());
										rxData = pollRxQueue();
										if (rxData != null) {
											rxOutput = rxOutput + " 0x" + Data.intToHex(rxData.intValue()) + str;
										}
										else {
											rxOutput = rxOutput + " ERROR";
										}
									}
									break;
								case 8:
									String str = "";
									for (int i=0; i<8; i++) {
										rxData = pollRxQueue();
										if (rxData != null) {
											str = Data.intToHex(rxData.intValue()) + str;
										}
										else {
											rxOutput = rxOutput + " ERROR";
											i = 8;
										}
									}
									rxOutput = rxOutput + " 0x" + str;
									break;
								default:
									rxOutput = rxOutput + " ERROR";
									break;
							}
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_MAC_SRC_PAN_ID:
						rxOutput = "macSrcPanID:";
						rxData = pollRxQueue();
						if (rxData != null) {
							String str = Data.intToHex(rxData.intValue());
							rxData = pollRxQueue();
							if (rxData != null) {
								rxOutput = rxOutput + " 0x" + Data.intToHex(rxData.intValue()) + str;
							}
							else {
								rxOutput = rxOutput + " ERROR";
							}
						}
						else {
							rxOutput = rxOutput + " ERROR";
						}
						System.out.println(rxOutput);
						break;
					case UART_RX_ZIGBEE_MAC_SRC_ADDR:
						rxOutput = "macSrcAddr:";
						rxLength = pollRxQueue();
						if (rxLength != null) {
							switch (rxLength.intValue()) {
								case 2:
									rxData = pollRxQueue();
									if (rxData != null) {
										String str = Data.intToHex(rxData.intValue());
										rxData = pollRxQueue();
										if (rxData != null) {
											rxOutput = rxOutput + " 0x" + Data.intToHex(rxData.intValue()) + str;
										}
										else {
											rxOutput = rxOutput + " ERROR";
										}
									}
									break;
								case 8:
									String str = "";
									for (int i=0; i<8; i++) {
										rxData = pollRxQueue();
										if (rxData != null) {
											str = Data.intToHex(rxData.intValue()) + str;
										}
										else {
											rxOutput = rxOutput + " ERROR";
											i = 8;
										}
									}
									rxOutput = rxOutput + " 0x" + str;
									break;
								default:
									rxOutput = rxOutput + " ERROR";
									break;
							}
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_CCA_VALUE:
						rxOutput = "ccaValue:";
						rxData = pollRxQueue();
						if (rxData != null) {
							if (rxData.intValue() == 1) {
								rxOutput = rxOutput + " Free";
							}
							else {
								rxOutput = rxOutput + " Busy";
							}
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_PART_NUM:
						rxOutput = "partNum:";
						rxData = pollRxQueue();
						if (rxData != null) {
							rxOutput = rxOutput + " " + rxData.intValue();
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_VER_NUM:
						rxOutput = "verNum:";
						rxData = pollRxQueue();
						if (rxData != null) {
							rxOutput = rxOutput + " " + rxData.intValue();
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_MAN_ID_0:
						rxOutput = "manID0:";
						rxData = pollRxQueue();
						if (rxData != null) {
							rxOutput = rxOutput + " " + rxData.intValue();
							System.out.println(rxOutput);
						}
						break;
					case UART_RX_ZIGBEE_MAN_ID_1:
						rxOutput = "manID1:";
						rxData = pollRxQueue();
						if (rxData != null) {
							rxOutput = rxOutput + " " + rxData.intValue();
							System.out.println(rxOutput);
						}
						break;
					default:
				}
			}
		}
	}
	
	public Integer pollRxQueue() {
		Integer data = null;
		try {
			data = (Integer) serial.getRxQueue().poll(1000, TimeUnit.MILLISECONDS);
		}
		catch (InterruptedException e) {
			System.err.println(e);
		}
		if (data != null) {
			//System.out.println("RX: " + Data.intToHex(data.intValue()));
		}
		return data;
	}
	
	public static void main(String[] args) {
		Zigbee zigbee = new Zigbee();	
		zigbee.start();
	}
	
}